package demo.areacharts;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.List ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.draw.ChangeChartShapeAdapter ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.UnitType ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AreaChartDemo1 extends JPanel {

  public AreaChartDemo1() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset1() ;
    final IJChart chart = ChartFactory.createAreaChart("Area Chart", "Category", "Value", dataset,
                                                       ChartOrientation.VERTICAL, true, true) ;
    String text = "An area chart demonstration.  We use this subtitle as an example of what happens \n when you get a really long title or subtitle." ;
    TextTitle texttitle = new TextTitle(text) ;
    texttitle.setPosition(RectangleEdge.TOP) ;
    texttitle.setPadding(new RectangleInsets(UnitType.RELATIVE, 0.050000000000000003D,
                                             0.050000000000000003D, 0.050000000000000003D,
                                             0.050000000000000003D)) ;
    texttitle.setVerticalAlignment(VerticalAlignment.BOTTOM) ;
    chart.addSubtitle(texttitle) ;
    CategoryPlot categoryplot = (CategoryPlot) chart.getPlot() ;
    categoryplot.setDomainGridlinePaint(Color.white) ;
    categoryplot.setValueGridlinePaint(Color.white) ;
    CategoryAxis categoryaxis = categoryplot.getCategoryAxis() ;
    categoryaxis.setTickLabelAngle(IAngle.ANGLE_UP_45) ;
    categoryaxis.setLowerMargin(0.0D) ;
    categoryaxis.setUpperMargin(0.0D) ;
    NumberValueAxis numberaxis = (NumberValueAxis) categoryplot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    categoryplot.getRender().setBackgroundPaint(Color.lightGray) ;

    chart.setChangeChartShape(new ChangeChartShapeAdapter() {
      public void beforeDraw(ChartShapeCollection chartShapes) {
        AxisShapeCollection axisShapes = chart.getChartShapes().getPlotShapes().getXAxisShape() ;
        List tickShapes = axisShapes.getTickLabelShapes() ;

        IChartShape shape = (IChartShape) tickShapes.get(0) ;
        shape.setTooltipText("The first type.") ;
        shape = (IChartShape) tickShapes.get(1) ;
        shape.setTooltipText("The second type.") ;
        shape = (IChartShape) tickShapes.get(2) ;
        shape.setTooltipText("The third type.") ;
      }

    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowKey("Series 3") ;
    for (int i = 1 ; i <= 8 ; i++) {
      dataset.addColKey("Type " + i) ;
    }

    double[][] values = { {1.0D, 4D, 3D, 5D, 5D, 7D, 7D, 8D}, {5D, 7D, 6D, 8D, 4D, 4D, 2D, 1.0D},
                        {4D, 3D, 2D, 3D, 6D, 3D, 4D, 3D}
    } ;

    for (int i = 0 ; i < values.length ; i++) {
      dataset.addRowData() ;
      for (int j = 0 ; j < values[i].length ; j++) {
        dataset.addData(i, new NumberChartData(values[i][j])) ;
      }
    }

    return dataset ;
  }

}
