package demo.barcharts.xy ;

import java.awt.Font ;
import java.awt.GridLayout ;
import java.text.NumberFormat ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.data.IntervalXNumberYBaseChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.IntervalXNumberYBaseBar2DRender ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBarChartDemo5 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  private Calendar cal = Calendar.getInstance() ;

  private NumberFormat labelFormat = NumberFormat.getIntegerInstance() ;

  public XYBarChartDemo5() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createIntervalXBaseYBar2DChart(
      "US Budget Deficit", "Year", "$ Billion", dataset, orientation, false, true, true, false) ;

    TextTitle subTitle = new TextTitle(
      "Source: http://www.cbo.gov/showdoc.cfm?index=1821&sequence=0#table12",
      new Font("Dialog", Font.BOLD, 12)) ;
    chart.addSubtitle(subTitle) ;
    subTitle.setPosition(RectangleEdge.BOTTOM) ;
    subTitle.setExpandToFitSpace(true) ;
    subTitle.setHorizontalAlignment(HorizontalAlignment.RIGHT) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    IntervalXNumberYBaseBar2DRender render = (IntervalXNumberYBaseBar2DRender) xyplot.getRender() ;
    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        IntervalXNumberYBaseChartData data = (IntervalXNumberYBaseChartData)
                                             dataset.getData(row, column) ;
        cal.setTimeInMillis((long) data.getEndXValue()) ;
        return cal.get(Calendar.YEAR) + " = " + labelFormat.format(data.getYValue()) ;
      }
    }) ;

    AbstractValueAxis valueAxis = xyplot.getDomainAxis() ;
    valueAxis.setLowerMargin(0.01D) ;
    valueAxis.setUpperMargin(0.01D) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Executions") ;

    double data[][] = { {1980, -74D}, {1981, -79D}, {1982, -128D}, {1983, -208D}, {1984, -185D},
                      {1985, -212D}, {1986, -221D}, {1987, -150D}, {1988, -155D}, {1989, -153D},
                      {1990, -221D}, {1991, -269D}, {1992, -290D}, {1993, -255D}, {1994, -203D},
                      {1995, -164D}, {1996, -107D}, {1997, -22D}, {1998, 69D}, {1999, 126D}, {2000,
                      236D}, {2001, 128D}, {2002, -158D}, {2003, -378D}, {2004, -412D}
    } ;

    dataset.addRowData() ;

    for (int i = 0 ; i < data.length ; i++) {
      long time1 = DateUtil.dateToTimeInMillis((int) data[i][0] - 1, 8) ;
      long time2 = DateUtil.dateToTimeInMillis((int) data[i][0], 7) ;
      dataset.addData(0, new IntervalXNumberYBaseChartData(time1, time2, data[i][1])) ;
    }

    return dataset ;
  }

}
