package demo.combinecharts;

import java.awt.BorderLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.plot.combine.CombineCategoryAxisCategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.render.category.LineAndShapeRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CombinedCategoryPlotDemo1 extends JPanel {

  public CombinedCategoryPlotDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    ChartOrientation orientation = ChartOrientation.VERTICAL ;

    IMatrixDataset dataset1 = createDataset1() ;
    IMatrixDataset dataset2 = createDataset2() ;

    CategoryAxis xAxis1 = new CategoryAxis("Category", dataset1) ;
    NumberValueAxis yAxis1 = new NumberValueAxis("Value", dataset1) ;

    Bar2DRender render1 = new Bar2DRender(dataset1) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    CategoryPlot plot1 = new CategoryPlot(xAxis1, yAxis1, render1) ;
    plot1.setOrientation(orientation) ;

    CategoryAxis xAxis2 = new CategoryAxis("Category", dataset2) ;
    xAxis2.setAxisLabelType(AxisLabelType.COLUMN) ;
    NumberValueAxis yAxis2 = new NumberValueAxis("Value", dataset2) ;

    LineAndShapeRender render2 = new LineAndShapeRender(dataset2) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    CategoryPlot plot2 = new CategoryPlot(xAxis2, yAxis2, render2) ;
    plot2.setOrientation(orientation) ;

    CombineCategoryAxisCategoryPlot combinePlot = new CombineCategoryAxisCategoryPlot() ;
    combinePlot.addSubPlot(plot1, 1) ;
    combinePlot.addSubPlot(plot2, 1) ;
    combinePlot.setDomainCrosshairVisible(true);
    combinePlot.setValueCrosshairVisible(true);

    IJChart chart = new IJChart("Combined Category Plot Demo 1", IJChart.DEFAULT_TITLE_FONT,
                                combinePlot, true) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("First") ;
    dataset.addColKey("Second") ;
    dataset.addRowData(8) ;

    double[] arr1 = {1.0D, 4D, 3D, 5D, 5D, 7D, 7D, 8D} ;
    double[] arr2 = {5D, 7D, 6D, 8D, 4D, 4D, 2D, 1.0D} ;
    for (int i = 0 ; i < 8 ; i++) {
      dataset.addRowKey("Type " + (i + 1)) ;

      dataset.addData(i, new NumberChartData(arr1[i])) ;
      dataset.addData(i, new NumberChartData(arr2[i])) ;
    }

    return dataset ;
  }

  private static DefaultMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Third") ;
    dataset.addRowKey("Fourth") ;
    dataset.addRowData(2) ;

    double[] arr1 = {11D, 14D, 13D, 15D, 15D, 17D, 17D, 18D} ;
    double[] arr2 = {15D, 17D, 16D, 18D, 14D, 14D, 12D, 11D} ;
    for (int i = 0 ; i < 8 ; i++) {
      dataset.addColKey("Type " + (i + 1)) ;

      dataset.addData(0, new NumberChartData(arr1[i])) ;
      dataset.addData(1, new NumberChartData(arr2[i])) ;
    }

    return dataset ;
  }
}
