package demo.granttcharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.Calendar ;
import java.util.Date ;

import javax.swing.JPanel ;

import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.GranttChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.GranttRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GanttDemo2 extends JPanel {

  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private String[] rowKeys = {"WriteProposal", "ObtainApproval", "RequirementsAnalysis",
                             "DesignPhase",
                             "DesignSignoff", "AlphaImplementation", "DesignReview",
                             "RevisedDesignSignoff", "BetaImplementation", "Testing",
                             "FinalImplementation", "Signoff"} ;

  public GanttDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    IMatrixDataset dataset = createDataset1() ;
    IJChart chart = ChartFactory.createGranttChart(
      "Gantt Chart Demo 2", "Task", "Date", dataset, orientation, true, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setDomainGridlineVisible(false) ;
    plot.getCategoryAxis().setMaximumCategoryLabelWidthRatio(10F) ;
    GranttRender render = (GranttRender) plot.getRender() ;
    render.setAutoSelectFillPaint(false) ;
    render.setColumnItemFillPaint(0, Color.blue) ;

    IMatrixDataset dataset2 = createDataset2() ;
    GranttRender render2 = new GranttRender(dataset2) {
      protected int getLegendItemCount() {
        return 0 ;
      }
    } ;
    render2.setAutoSelectFillPaint(false) ;
    render2.setColumnItemFillPaint(0, Color.blue) ;
    plot.setRender(1, render2) ;

    IMatrixDataset dataset3 = createDataset3() ;
    GranttRender render3 = new GranttRender(dataset3) {
      protected int getLegendItemCount() {
        return 0 ;
      }
    } ;
    render3.setAutoSelectFillPaint(false) ;
    render3.setColumnItemFillPaint(0, Color.blue) ;
    plot.setRender(2, render3) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("Scheduled") ;

    for (int i = 0 ; i < rowKeys.length ; i++) {
      dataset.addRowKey(rowKeys[i]) ;
    }

    Date[][] dateArr1 = { {createDate(1, 3, 2001), createDate(5, 3, 2001)}, {createDate(9, 3, 2001),
                        createDate(9, 3, 2001)}, {createDate(10, 3, 2001), createDate(5, 4, 2001)},
                        {createDate(6, 4, 2001), createDate(30, 4, 2001)}, {createDate(2, 5, 2001),
                        createDate(2, 5, 2001)}, {createDate(3, 5, 2001), createDate(31, 6, 2001)},
                        {createDate(1, 7, 2001), createDate(8, 7, 2001)}, {createDate(10, 7, 2001),
                        createDate(10, 7, 2001)}, {createDate(12, 7, 2001), createDate(12, 8, 2001)},
                        {createDate(13, 8, 2001), createDate(31, 9, 2001)}, {createDate(1, 10, 2001),
                        createDate(15, 10, 2001)}, {createDate(28, 10, 2001),
                        createDate(30, 10, 2001)}
    } ;
    double[] percent = {1.0D, 1.0D, 0.0D, 0.0D, 0.0D, 0.59999999999999998D, 0.0D, 0.0D, 0.0D, 0.0D,
                       0.0D, 0.0D} ;

    dataset.addRowData(dateArr1.length) ;
    for (int i = 0 ; i < dateArr1.length ; i++) {
      dataset.addData(i, new GranttChartData(dateArr1[i][0], dateArr1[i][1], new Double(percent[i]))) ;
    }

    return dataset ;
  }

  private IMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("Scheduled") ;

    dataset.addRowData(rowKeys.length) ;
    dataset.addData(2,
                    new GranttChartData(createDate(10, 4, 2001), createDate(15, 4, 2001),
                                        new Double(1))) ;
    dataset.addData(3,
                    new GranttChartData(createDate(2, 5, 2001), createDate(8, 5, 2001),
                                        new Double(1))) ;
    return dataset ;
  }

  private IMatrixDataset createDataset3() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("Scheduled") ;

    dataset.addRowData(rowKeys.length) ;
    dataset.addData(3,
                    new GranttChartData(createDate(10, 5, 2001), createDate(14, 5, 2001),
                                        new Double(0.5))) ;
    return dataset ;
  }

  private static Date createDate(int day, int month, int year) {
    Calendar calendar = Calendar.getInstance() ;
    calendar.set(year, month - 1, day) ;
    Date date1 = calendar.getTime() ;

    return date1 ;
  }

}
