package demo.miscellaneous.crosshairs ;

import java.awt.BorderLayout ;
import java.awt.geom.Point2D ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;
import java.util.List ;

import javax.swing.JPanel ;
import javax.swing.JScrollPane ;
import javax.swing.JSlider ;
import javax.swing.JTable ;
import javax.swing.event.ChangeEvent ;
import javax.swing.event.ChangeListener ;
import javax.swing.table.AbstractTableModel ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.ChartProgressEvent ;
import com.ijchart.xychart.listener.IChartProgressListener ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.shape.IChartShape ;
import demo.others.common.renderer.DateCellRenderer ;
import demo.others.common.renderer.NumberCellRenderer ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CrosshairDemo1 extends JPanel {
  private String rowKey = "Random 1" ;

  private Calendar cal = Calendar.getInstance() ;

  public CrosshairDemo1() {
    setLayout(new BorderLayout()) ;
    add(new DemoPanel()) ;
  }

  static class DemoTableModel extends AbstractTableModel {
    private Object data[][] ;

    public DemoTableModel(int i) {
      data = new Object[i][7] ;
    }

    public int getColumnCount() {
      return 7 ;
    }

    public int getRowCount() {
      return 1 ;
    }

    public Object getValueAt(int i, int j) {
      return data[i][j] ;
    }

    public void setValueAt(Object obj, int i, int j) {
      data[i][j] = obj ;
      fireTableDataChanged() ;
    }

    public String getColumnName(int i) {
      switch (i) {
      case 0: // '\0'
        return "Series Name:" ;
      case 1: // '\001'
        return "X:" ;
      case 2: // '\002'
        return "Y:" ;
      case 3: // '\003'
        return "X (prev)" ;
      case 4: // '\004'
        return "Y (prev):" ;
      case 5: // '\005'
        return "X (next):" ;
      case 6: // '\006'
        return "Y (next):" ;
      }

      return null ;
    }

  }

  private class DemoPanel extends JPanel implements ChangeListener, IChartProgressListener {
    private ChartPanel chartPanel ;

    private DemoTableModel model ;

    private IJChart chart ;

    private JSlider slider ;

    private IMatrixDataset dataset1 = createDataset1() ;

    private IJChart createChart() {
      chart = ChartFactory.createXYLineAndShapeChart("Crosshair Demo 1", "Time of Day", "Value",
                                                     dataset1, ChartOrientation.VERTICAL, true, false,
                                                     true, false) ;
      XYPlot plot = (XYPlot) chart.getPlot() ;
      plot.setDomainCrosshairVisible(true) ;
      plot.setValueCrosshairVisible(true) ;

      XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
      render.setBaseItemShapeVisible(false) ;

      return chart ;
    }

    private DefaultMatrixDataset createDataset1() {
      DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
      dataset.addRowKey(rowKey) ;
      dataset.addRowData() ;

      cal.set(Calendar.SECOND, 0) ;
      cal.set(Calendar.MILLISECOND, 0) ;

      double d1 = 100 ;
      for (int j = 0 ; j < 200 ; j++) {
        dataset.addData(0, new XYNumberChartData(cal.getTimeInMillis(), d1)) ;
        d1 *= 1.0D + (Math.random() - 0.495D) / 10D ;

        cal.add(Calendar.MINUTE, 1) ;
      }

      return dataset ;
    }

    public void stateChanged(ChangeEvent changeevent) {
      int i = slider.getValue() ;
      XYPlot xyplot = (XYPlot) chart.getPlot() ;
      RenderShapeCollection renderShapes = (RenderShapeCollection) chart.getChartShapes().
                                           getPlotShapes(0).getAllRenderShapes().get(0) ;

      List dataShapes = renderShapes.getDataItemShapes() ;
      IChartShape shape = (IChartShape) dataShapes.get(i) ;
      if (shape != null) {
        Point2D lockPoint = shape.getLockPoint() ;
        xyplot.drawCrosshair(lockPoint) ;
        chartPanel.repaint() ;
      }
    }

    public void chartProgress(ChartProgressEvent chartprogressevent) {
      if (chartprogressevent.getType() != ChartProgressEvent.PROGRESS_DRAWING_SHAPES_FINISHED) {
        return ;
      }

      model.setValueAt(rowKey, 0, 0) ;
      model.setValueAt(null, 0, 1) ;
      model.setValueAt(null, 0, 2) ;
      model.setValueAt(null, 0, 3) ;
      model.setValueAt(null, 0, 4) ;
      model.setValueAt(null, 0, 5) ;
      model.setValueAt(null, 0, 6) ;

      XYPlot xyplot = (XYPlot) chart.getPlot() ;
      xyplot.setDomainCrosshairVisible(true) ;
      xyplot.setValueCrosshairVisible(false) ;

      Point2D crossPoint = xyplot.getCrosshairPoint(chartPanel.getMouseClickPoint()) ;
      if (crossPoint == null) {
        return ;
      }

      IChartShape shape = chartPanel.getShapeForPoint(crossPoint,
                                                      ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;
      if (shape == null) {
        return ;
      }

      XYNumberChartData data = (XYNumberChartData) shape.getData() ;
      if (data != null) {
        int dataIndex = -1 ;
        int colCount = dataset1.getColumnCount(0) ;
        for (int i = 0 ; i < colCount ; i++) {
          if (data == dataset1.getData(0, i)) {
            dataIndex = i ;
            break ;
          }
        }
        if (dataIndex < 0) {
          return ;
        }

        XYNumberChartData previousData = (XYNumberChartData) dataset1.
                                         getData(0, Math.max(0, dataIndex - 1)) ;
        XYNumberChartData nextData = (XYNumberChartData) dataset1.
                                     getData(0, Math.min(colCount - 1, dataIndex + 1)) ;
        if (previousData != null) {
          model.setValueAt(previousData.getXData(), 0, 3) ;
          model.setValueAt(previousData.getYData(), 0, 4) ;
        }

        model.setValueAt(data.getXData(), 0, 1) ;
        model.setValueAt(data.getYData(), 0, 2) ;

        if (nextData != null) {
          model.setValueAt(nextData.getXData(), 0, 5) ;
          model.setValueAt(nextData.getYData(), 0, 6) ;
        }
      }

    }

    public DemoPanel() {
      setLayout(new BorderLayout()) ;
      chart = createChart() ;
      chart.addProgressListener(this) ;
      chartPanel = new ChartPanel(chart) ;
      add(chartPanel, BorderLayout.NORTH) ;

//      chartPanel.setDomainZoomable(true) ;
//      chartPanel.setRangeZoomable(true) ;
      model = new DemoTableModel(3) ;
      model.setValueAt(dataset1.getRowKey(0), 0, 0) ;
      model.setValueAt(new Double("0.00"), 0, 1) ;
      model.setValueAt(new Double("0.00"), 0, 2) ;
      model.setValueAt(new Double("0.00"), 0, 3) ;
      model.setValueAt(new Double("0.00"), 0, 4) ;
      model.setValueAt(new Double("0.00"), 0, 5) ;
      model.setValueAt(new Double("0.00"), 0, 6) ;
      JTable jtable = new JTable(model) ;

      DateCellRenderer datecellrenderer = new DateCellRenderer(new SimpleDateFormat("HH:mm")) ;
      NumberCellRenderer numbercellrenderer = new NumberCellRenderer() ;
      jtable.getColumnModel().getColumn(1).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(2).setCellRenderer(numbercellrenderer) ;
      jtable.getColumnModel().getColumn(3).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(4).setCellRenderer(numbercellrenderer) ;
      jtable.getColumnModel().getColumn(5).setCellRenderer(datecellrenderer) ;
      jtable.getColumnModel().getColumn(6).setCellRenderer(numbercellrenderer) ;

      slider = new JSlider(0, 199, 100) ;
      slider.addChangeListener(this) ;

      JPanel panel1 = new JPanel(new BorderLayout()) ;
      panel1.add(new JScrollPane(jtable)) ;
      panel1.add(slider, BorderLayout.SOUTH) ;
      add(panel1, BorderLayout.CENTER) ;
    }

  }

}
