package demo.miscellaneous.crosshairs ;

import java.awt.BorderLayout ;
import java.awt.geom.Point2D ;
import java.util.List ;

import javax.swing.JPanel ;
import javax.swing.JSlider ;
import javax.swing.event.ChangeEvent ;
import javax.swing.event.ChangeListener ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.util.DatasetUtil ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CrosshairDemo3 extends JPanel {

  private IMatrixDataset dataset1 ;

  public CrosshairDemo3() {
    setLayout(new BorderLayout()) ;
    add(new DemoPanel()) ;
  }

  private class DemoPanel extends JPanel implements ChangeListener {
    private ChartPanel chartPanel ;

    private IJChart chart ;

    private JSlider slider ;

    public DemoPanel() {
      setLayout(new BorderLayout()) ;
      chart = createChart() ;
      chartPanel = new ChartPanel(chart) ;
      add(chartPanel, BorderLayout.NORTH) ;

      int colCount = dataset1.getColumnCount() ;
      slider = new JSlider(0, colCount, colCount / 2) ;
      slider.addChangeListener(this) ;
      add(slider, BorderLayout.CENTER) ;
    }

    private IJChart createChart() {
      dataset1 = createDataset() ;
      chart = ChartFactory.createXYLineAndShapeChart("Legal & General Unit Trust Prices", "Date",
                                                     "Price Per Unit", dataset1,
                                                     ChartOrientation.VERTICAL, true, false,
                                                     true, false) ;
      XYPlot plot = (XYPlot) chart.getPlot() ;
      plot.setDomainCrosshairVisible(true) ;
      plot.setValueCrosshairVisible(false) ;

      return chart ;
    }

    public void stateChanged(ChangeEvent changeevent) {
      int sliderVal = slider.getValue() ;
      XYPlot xyplot = (XYPlot) chart.getPlot() ;
      RenderShapeCollection renderShapes = (RenderShapeCollection) chart.getChartShapes().
                                           getPlotShapes(0).getAllRenderShapes().get(0) ;

      List dataShapes = renderShapes.getDataItemShapes() ;
      Point2D crossPoint = null ;
      Point2D mousePoint = chartPanel.getMouseClickPoint() ;
      if (mousePoint != null) {
        crossPoint = xyplot.getCrosshairPoint(mousePoint) ;
      } else {
        crossPoint = ((IChartShape) dataShapes.get(0)).getLockPoint() ;
      }

      IChartShape shape = chartPanel.getShapeForPoint(crossPoint,
                                                      ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;
      if (shape != null) {
        IChartData data = shape.getData() ;
        int[] findPosition = DatasetUtil.findDataPosition(dataset1, data) ;
        if (findPosition != null) {
          int rowDataCount = dataset1.getColumnCount(findPosition[0]) ;
          if (sliderVal <= rowDataCount - 1) {
            IChartData currData = dataset1.getData(findPosition[0], sliderVal) ;
            for (int i = 0 ; i < dataShapes.size() ; i++) {
              IChartShape currShape = (IChartShape) dataShapes.get(i) ;
              if (currShape != null && currShape.getData() == currData) {
                Point2D lockPoint = currShape.getLockPoint() ;
                xyplot.drawCrosshair(lockPoint) ;
                chartPanel.repaint() ;
                return ;
              }
            }
          }
        }
      }
    }

    private IMatrixDataset createDataset() {
      DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
      dataset.addRowKey("L&G European Index Trust") ;
      dataset.addRowKey("L&G UK Index Trust") ;
      dataset.addRowData(2) ;

      dataset.addData(0, createData(2, 2001, 181.80000000000001D)) ;
      dataset.addData(0, createData(3, 2001, 167.30000000000001D)) ;
      dataset.addData(0, createData(4, 2001, 153.80000000000001D)) ;
      dataset.addData(0, createData(5, 2001, 167.59999999999999D)) ;
      dataset.addData(0, createData(6, 2001, 158.80000000000001D)) ;
      dataset.addData(0, createData(7, 2001, 148.30000000000001D)) ;
      dataset.addData(0, createData(8, 2001, 153.90000000000001D)) ;
      dataset.addData(0, createData(9, 2001, 142.69999999999999D)) ;
      dataset.addData(0, createData(10, 2001, 123.2D)) ;
      dataset.addData(0, createData(11, 2001, 131.80000000000001D)) ;
      dataset.addData(0, createData(12, 2001, 139.59999999999999D)) ;
      dataset.addData(0, createData(1, 2002, 142.90000000000001D)) ;
      dataset.addData(0, createData(2, 2002, 138.69999999999999D)) ;
      dataset.addData(0, createData(3, 2002, 137.30000000000001D)) ;
      dataset.addData(0, createData(4, 2002, 143.90000000000001D)) ;
      dataset.addData(0, createData(5, 2002, 139.80000000000001D)) ;
      dataset.addData(0, createData(6, 2002, 137D)) ;
      dataset.addData(0, createData(7, 2002, 132.80000000000001D)) ;

      dataset.addData(1, createData(2, 2001, 129.59999999999999D)) ;
      dataset.addData(1, createData(3, 2001, 123.2D)) ;
      dataset.addData(1, createData(4, 2001, 117.2D)) ;
      dataset.addData(1, createData(5, 2001, 124.09999999999999D)) ;
      dataset.addData(1, createData(6, 2001, 122.59999999999999D)) ;
      dataset.addData(1, createData(7, 2001, 119.2D)) ;
      dataset.addData(1, createData(8, 2001, 116.5D)) ;
      dataset.addData(1, createData(9, 2001, 112.7D)) ;
      dataset.addData(1, createData(10, 2001, 101.5D)) ;
      dataset.addData(1, createData(11, 2001, 106.09999999999999D)) ;
      dataset.addData(1, createData(12, 2001, 110.3D)) ;
      dataset.addData(1, createData(1, 2002, 111.7D)) ;
      dataset.addData(1, createData(2, 2002, 111D)) ;
      dataset.addData(1, createData(3, 2002, 109.59999999999999D)) ;
      dataset.addData(1, createData(4, 2002, 113.2D)) ;
      dataset.addData(1, createData(5, 2002, 111.59999999999999D)) ;
      dataset.addData(1, createData(6, 2002, 108.8D)) ;
      dataset.addData(1, createData(7, 2002, 101.59999999999999D)) ;

      return dataset ;
    }

    private XYNumberChartData createData(int month, int year, double value) {
      long time = DateUtil.dateToTimeInMillis(year, month) ;
      return new XYNumberChartData(time, value) ;
    }

  }

}
