package demo.others.shapes ;

import java.awt.Color ;
import java.awt.Dimension ;
import java.awt.Graphics ;
import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.RenderingHints ;
import java.awt.Shape ;
import java.awt.event.MouseEvent ;
import java.awt.event.MouseMotionAdapter ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.util.HashMap ;
import java.util.Map ;

import javax.swing.BorderFactory ;
import javax.swing.JLabel ;
import javax.swing.JPanel ;

import com.ijchart.xychart.shape.ErrorCrossChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ErrorCrossChartShapePanel extends JPanel {
  private Map hashMap = new HashMap() ;

  private JLabel label ;

  public ErrorCrossChartShapePanel() {
    init() ;
  }

  private void init() {
    hashMap.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON) ;

    setLayout(new GridLayout(4, 2)) ;
    addPanel1() ;
    addPanel2() ;
    addPanel3() ;
    addPanel4() ;
    addPanel5() ;
    addPanel6() ;

    label = new JLabel("begin") ;
    add(label) ;
  }

  private void addPanel1() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape circle = new Ellipse2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 85, 50, 50, 10, 100, circle) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.setSelected(true) ;
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel2() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape square = new Rectangle2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 75, 50, 50, 10, 80, square) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.setSelected(true);
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel3() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape square = new Rectangle2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 75, 50, 50, 10, 80, square) ;
    shape.setRotate(Math.PI / 4) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.setSelected(true);
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel4() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape square = new Rectangle2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 75, 50, 50, 10, 80, square) ;
    shape.setRotate(Math.PI / 6) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.setSelected(true);
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel5() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape square = new Rectangle2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 35, 50, 50, 10, 80, square) ;
    shape.setRotate(Math.PI / 6) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.setSelected(true);
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel6() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    Shape square = new Rectangle2D.Double( -delta, -delta, size, size) ;

    final ErrorCrossChartShape shape = new ErrorCrossChartShape(25, 75, 50, 50, 10, 80, square) ;
    shape.setRotate(Math.PI / 6) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        g.drawRect(0, 0, 100, 100) ;

        shape.setBorderPaint(Color.red) ;
        shape.setFillPaint(Color.blue) ;

        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

}
