package demo.others.shapes ;

import java.awt.Color ;
import java.awt.Dimension ;
import java.awt.Graphics ;
import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.RenderingHints ;
import java.awt.event.MouseEvent ;
import java.awt.event.MouseMotionAdapter ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.util.HashMap ;
import java.util.Map ;

import javax.swing.BorderFactory ;
import javax.swing.JLabel ;
import javax.swing.JPanel ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.shape.StockChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StockChartShapePanel extends JPanel {
  private Map hashMap = new HashMap() ;

  private JLabel label ;

  public StockChartShapePanel() {
    init() ;
  }

  private void init() {
    hashMap.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON) ;

    setLayout(new GridLayout(2, 3)) ;
    addPanel1() ;
    addPanel2() ;
    addPanel3() ;
    addPanel4() ;
    addPanel5() ;
    label = new JLabel("begin") ;
    add(label) ;
  }

  private void addPanel1() {
    Rectangle2D volumeRect = new Rectangle2D.Double(10, 50, 5, 100) ;
    Rectangle2D openCloseRect = new Rectangle2D.Double(8, 30, 8, 50) ;
    Line2D heightLowLine = new Line2D.Double(12, 2, 12, 125) ;

    final StockChartShape shape = new StockChartShape("abc", volumeRect, Color.lightGray,
                                                      openCloseRect, Color.red, Color.green,
                                                      heightLowLine, Color.red) ;
    shape.setForceInsideTextNoWrap(true) ;
    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }
    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(160, 160)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel2() {
    Rectangle2D volumeRect = new Rectangle2D.Double(10, 50, 5, 100) ;
    Rectangle2D openCloseRect = new Rectangle2D.Double(8, 30, 8, 50) ;
    Line2D heightLowLine = new Line2D.Double(12, 2, 12, 125) ;

    final StockChartShape shape = new StockChartShape("abc", volumeRect, Color.lightGray,
                                                      openCloseRect, Color.red, Color.green,
                                                      heightLowLine, Color.red) ;
    shape.setForceInsideTextNoWrap(true) ;
    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        shape.setSelected(true);
        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }
    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(160, 160)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel3() {
    Rectangle2D volumeRect = new Rectangle2D.Double(30, 50, 5, 100) ;
    Rectangle2D openCloseRect = new Rectangle2D.Double(28, 30, 8, 50) ;
    Line2D heightLowLine = new Line2D.Double(32, 2, 32, 125) ;

    final StockChartShape shape = new StockChartShape("abc", volumeRect, Color.lightGray,
                                                      openCloseRect, Color.red, Color.green,
                                                      heightLowLine, Color.red) ;
    shape.setForceInsideTextNoWrap(true) ;
    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;
    shape.setRotate(0.2) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }
    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(160, 160)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel4() {
    final Rectangle2D volumeRect = null ;
    Rectangle2D openCloseRect = new Rectangle2D.Double(8, 30, 8, 50) ;
    Line2D heightLowLine = new Line2D.Double(12, 2, 12, 125) ;

    final StockChartShape shape = new StockChartShape("abc", volumeRect, Color.lightGray,
                                                      openCloseRect, Color.red, Color.green,
                                                      heightLowLine, Color.red) ;
    shape.setForceInsideTextNoWrap(true) ;
    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }
    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(160, 160)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel5() {
    final Rectangle2D volumeRect = null ;
    Rectangle2D openCloseRect = new Rectangle2D.Double(8, 30, 8, 50) ;
    Line2D heightLowLine = new Line2D.Double(12, 2, 12, 125) ;

    final StockChartShape shape = new StockChartShape("abc", volumeRect, Color.lightGray,
                                                      openCloseRect, Color.red, Color.green,
                                                      heightLowLine, Color.red) ;
    shape.setForceInsideTextNoWrap(true) ;
    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;
    shape.setRotate(-0.3) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        shape.draw((Graphics2D) g) ;

        Point2D p = shape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }
    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(160, 160)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }
}
