package demo.piecharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.text.NumberFormat ;
import java.util.List ;

import javax.swing.JPanel ;

import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.LegendShapeCollection ;
import com.ijchart.xychart.chartinfo.draw.ChangeChartShapeAdapter ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.NoAxisPlot ;
import com.ijchart.xychart.render.noaxis.PieRender ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PieChartDemo2 extends JPanel {
  private NumberFormat format = NumberFormat.getPercentInstance() ;

  public PieChartDemo2() {
    format.setMaximumFractionDigits(0) ;
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final SingleRowDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createPieChart("Pie Chart Demo 2", dataset, true, true) ;
    NoAxisPlot plot = (NoAxisPlot) chart.getPlot() ;

    PieRender render = (PieRender) plot.getRender() ;
    render.setAutoSelectFillPaint(false) ;
    render.setColumnItemFillPaint(0, new Color(160, 160, 255)) ;
    render.setColumnItemFillPaint(1, new Color(128, 128, 223)) ;
    render.setColumnItemFillPaint(2, new Color(96, 96, 191)) ;
    render.setColumnItemFillPaint(3, new Color(64, 64, 159)) ;
    render.setColumnItemFillPaint(4, new Color(32, 32, 127)) ;
    render.setColumnItemFillPaint(5, new Color(0, 0, 111)) ;
    render.setExplodePercent(1, 0.20000000000000001D) ;
    render.setSimpleLabels(true) ;

    double total = 0.0 ;
    for (int i = 0 ; i < dataset.getColumnCount() ; i++) {
      total += ((NumberChartData) dataset.getData(0, i)).getValue() ;
    }
    final double paraTotal = total ;

    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        Object colKey = dataset.getColKey(column) ;
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        double percent = data.getValue() / paraTotal ;
        return colKey + " (" + format.format(percent) + " percent)" ;
      }
    }) ;
    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object colKey = dataset.getColKey(column) ;
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        double percent = data.getValue() / paraTotal ;
        return colKey + ": (" + data.getValue() + "," + format.format(percent) + ")" ;
      }

    }) ;

    render.setLabelBackgroundPaint(new Color(220, 220, 220)) ;
    chart.setChangeChartShape(new ChangeChartShapeAdapter() {
      public void beforeDraw(ChartShapeCollection chartShapes) {
        LegendShapeCollection legendShapes = chart.getChartShapes().getLegendShapes() ;
        List legends = legendShapes.getLegendShapes() ;

        for (int i = 0 ; i < legends.size() ; i++) {
          IChartShape shape = (IChartShape) legends.get(i) ;
          shape.setTooltipText("Tooltip for legend item " + dataset.getColKey(i)) ;
        }
      }

    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static SingleRowDataset createDataset() {
    SingleRowDataset dataset = new SingleRowDataset() ;

    dataset.addColKey("One") ;
    dataset.addColKey("Two") ;
    dataset.addColKey("Three") ;
    dataset.addColKey("Four") ;
    dataset.addColKey("Five") ;
    dataset.addColKey("Six") ;

    dataset.addRowData() ;
    dataset.addData(0, new NumberChartData(43.200000000000003D)) ;
    dataset.addData(0, new NumberChartData(10D)) ;
    dataset.addData(0, new NumberChartData(27.5D)) ;
    dataset.addData(0, new NumberChartData(17.5D)) ;
    dataset.addData(0, new NumberChartData(11D)) ;
    dataset.addData(0, new NumberChartData(19.399999999999999D)) ;

    return dataset ;
  }

}
