package demo.stackedbarcharts ;

import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.StackedNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.NumberFormatter ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.StackedBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedBarChartDemo3 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public StackedBarChartDemo3() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset1() ;
    IJChart chart = ChartFactory.createStackedBarChart("Stacked Bar Chart Demo 3", "Category",
                                                       "Value", dataset, orientation, true, true) ;
    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;

    StackedNumberValueAxis numberaxis = (StackedNumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    numberaxis.setTickValueFormatter(new NumberFormatter("0%")) ;

    StackedBarRender render = (StackedBarRender) plot.getRender() ;
    render.setBaseItemLabelVisible(true) ;
    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return String.valueOf((int) data.getValue()) ;
      }
    }) ;

    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        Object colKey = dataset.getColKey(column) ;
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return "(" + colKey + "," + rowKey + ")=" + (int) data.getValue() ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("Series 1") ;
    dataset.addColKey("Series 2") ;
    dataset.addColKey("Series 3") ;
    dataset.addColKey("Series 4") ;

    dataset.addRowKey("Jan") ;
    dataset.addRowKey("Feb") ;
    dataset.addRowKey("Mar") ;

    double data1[] = {10D, 12D, 13D} ;
    double data2[] = {4D, 3D, 2D} ;
    double data3[] = {2D, 3D, 2D} ;
    double data4[] = {2D, 3D, 4D} ;

    dataset.addRowData(data1.length) ;
    for (int i = 0 ; i < data1.length ; i++) {
      dataset.addData(i, new NumberChartData(data1[i])) ;
      dataset.addData(i, new NumberChartData(data2[i])) ;
      dataset.addData(i, new NumberChartData(data3[i])) ;
      dataset.addData(i, new NumberChartData(data4[i])) ;
    }

    return dataset ;
  }

}
