package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LegendItemChartShape ;
import com.ijchart.xychart.shape.PaintScaleLegendChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢ͼϵͼεļ.</zh>
 * <en>Collection to be used to store the shapes in legend area.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LegendShapeCollection extends AbstractShapeCollection implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3694564307000186602L ;

  /**
   * <zh>߿ͼ.</zh>
   * <en>The border shape.</en>
   */
  private IChartShape borderShape ;

  /**
   * <zh>ͼͼμ.</zh>
   * <en>The list of legend shapes.</en>
   */
  private List legendShapes ;

  /**
   * <zh>ͼͼҪ.</zh>
   * <en>The axis shapes for legend.</en>
   */
  private List axisShapes ;

  /**
   * <zh>޲.</zh>
   * <en>No argument constructor.</en>
   */
  public LegendShapeCollection() {
    this.borderShape = null ;
    this.legendShapes = new ArrayList() ;
    this.axisShapes = new ArrayList() ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>Sets the border shape.</en>
   *
   * @param borderShape IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (null permitted).</en>
   */
  public void setBorderShape(IChartShape borderShape) {
    if (borderShape != null) {
      borderShape.setId(shapeIdSequence++) ;
      borderShape.setShapeDrawType(ChartShapeDrawType.LEGEND_BORDER_SHAPE) ;
      borderShape.addChangeListener(this) ;
    }

    this.borderShape = borderShape ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>The border shape.</en>
   *
   * @return IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (maybe null).</en>
   */
  public IChartShape getBorderShape() {
    return this.borderShape ;
  }

  /**
   * <zh>һͼͼ.</zh>
   * <en>Add a legend shape.</en>
   *
   * @param shape IChartShape <zh>ͼͼ (Ϊnull)</zh>
   * <en>A legend shape (null permitted).</en>
   */
  public void addLegendShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.LEGEND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.legendShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>еͼͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of legend shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of legend shapes and is unmodifiable.</en>
   */
  public List getLegendShapes() {
    return Collections.unmodifiableList(this.legendShapes) ;
  }

  /**
   * <zh>ɾеͼͼ.</zh>
   * <en>Remove all legend shapes.</en>
   */
  public void clearLegendShapes() {
    removeShapesListener(this.legendShapes, this) ;
    this.legendShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>еͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of axis shape collection and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of axis shape collection and is unmodifiable.</en>
   */
  public List getLegendAxisShapeCollection() {
    return Collections.unmodifiableList(this.axisShapes) ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (this.borderShape != null) {
      this.borderShape.draw(g2) ;
    }

    this.axisShapes.clear() ;
    for (int i = 0 ; i < this.legendShapes.size() ; i++) {
      IChartShape shape = (IChartShape)this.legendShapes.get(i) ;
      if (shape != null) {
        shape.draw(g2) ;
        if (shape instanceof LegendItemChartShape) { //paint scaleͼ⴦
          LegendItemChartShape legendShape = (LegendItemChartShape) shape ;
          IChartShape itemShape = legendShape.getShape() ;
          if (itemShape instanceof PaintScaleLegendChartShape) {
            AxisShapeCollection axisShapeCollection = ((PaintScaleLegendChartShape) itemShape).
                                                      getAxisShapes() ;
            axisShapeCollection.addChangeListener(this) ;
            this.axisShapes.add(axisShapeCollection) ;
          }
        }
      }
    }

    for (int i = 0 ; i < this.axisShapes.size() ; i++) {
      AxisShapeCollection shape = (AxisShapeCollection)this.axisShapes.get(i) ;
      if (shape != null) {
        shape.draw(g2) ;
      }
    }
  }

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    if (this.borderShape != null) {
      this.borderShape.translate(x, y) ;
    }

    translate(x, y, this.legendShapes) ;
    for (int i = 0 ; i < this.axisShapes.size() ; i++) {
      AxisShapeCollection shape = (AxisShapeCollection)this.axisShapes.get(i) ;
      if (shape != null) {
        shape.translate(x, y) ;
      }
    }

  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    if (this.borderShape != null) {
      this.borderShape.removeAllChangeListener() ;
    }

    removeShapesListener(this.legendShapes, this) ;
    for (int i = 0 ; i < this.axisShapes.size() ; i++) {
      AxisShapeCollection shape = (AxisShapeCollection)this.axisShapes.get(i) ;
      if (shape != null) {
        shape.removeAllChangeListener() ;
      }
    }

  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.borderShape) ;
    clearSelectedShape(this.legendShapes) ;
    for (int i = 0 ; i < this.axisShapes.size() ; i++) {
      AxisShapeCollection shape = (AxisShapeCollection)this.axisShapes.get(i) ;
      if (shape != null) {
        shape.clearSelectedShapes() ;
      }
    }
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    IChartShape shape = null ;
    if (flag || drawType.equals(ChartShapeDrawType.LEGEND_SHAPE)) {
      shape = findShape(this.legendShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.LEGEND_BORDER_SHAPE)) {
      if (this.borderShape != null && this.borderShape.contains(point)) {
        return this.borderShape ;
      }
    }

    for (int i = 0 ; i < this.axisShapes.size() ; i++) {
      AxisShapeCollection shapes = (AxisShapeCollection)this.axisShapes.get(i) ;
      shape = shapes.findChartShape(point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.LEGEND_BORDER_SHAPE)) {
      addHotspotArea(result, this.borderShape, fragment, ChartShapeDrawType.LEGEND_BORDER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.LEGEND_SHAPE)) {
      addHotspotArea(result, this.legendShapes, fragment, ChartShapeDrawType.LEGEND_SHAPE) ;
    }

    return result ;
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.borderShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.legendShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisShapes) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof RenderShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    LegendShapeCollection that = (LegendShapeCollection) obj ;
    if (!EqualsUtil.equals(this.borderShape, that.borderShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.legendShapes, that.legendShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisShapes, that.axisShapes)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[borderShape=" + this.borderShape
      + ",legend shapes size=" + this.legendShapes.size()
      + ",axis shapes collection size=" + this.axisShapes.size()
      + "]" ;
  }
}
