package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.constant.DataBackgroundShapeDrawType ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.MeterArcChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢renderͼμ.</zh>
 * <en>Collection to be used to store the shapes in render class.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class RenderShapeCollection extends AbstractShapeCollection implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 499608626968604925L ;

  /**
   * <zh>ȱʡݵ㱳ͼλ.</zh>
   * <en>The default draw type of data background shape.</en>
   */
  public static final DataBackgroundShapeDrawType DEFAULT_DATA_BACKGROUND_SHAPE_DRAW_TYPE =
    DataBackgroundShapeDrawType.ALL_BY_ALL ;

  /**
   * <zh>߿ͼ.</zh>
   * <en>The border shape.</en>
   */
  private IChartShape borderShape ;

  /**
   * <zh>ɫͼ.</zh>
   * <en>The background shape.</en>
   */
  private IChartShape backgroundShape ;

  /**
   * <zh>ޱͼμ.</zh>
   * <en>The list of quadrant background shapes.</en>
   */
  private List quadrantBackgroundShapes ;

  /**
   * <zh>ͼƬͼ.</zh>
   * <en>The background image shape.</en>
   */
  private IChartShape backgroundImageShape ;

  /**
   * <zh>ֵ(value line)ͼμ.</zh>
   * <en>The list of value line shapes.</en>
   */
  private List valueLineShapes ;

  /**
   * <zh>domainͼμ.</zh>
   * <en>The list of domin grid line shapes.</en>
   */
  private List domainGridLines ;

  /**
   * <zh>valueͼμ.</zh>
   * <en>The list of value grid line shapes.</en>
   */
  private List valueGridLines ;

  /**
   * <zh>valueͼμ.</zh>
   * <en>The list of value grid band shapes.</en>
   */
  private List valueGridBands ;

  /**
   * <zh>3Dͼμ.</zh>
   * <en>The list of line shapes for close render area of 3D chart.</en>
   */
  private List chart3DCloseLines ;

  /**
   * <zh>ǰmarkerͼμ.</zh>
   * <en>The list of foreground marker shapes.</en>
   */
  private List foregroundMarkerShapes ;

  /**
   * <zh>markerͼμ.</zh>
   * <en>The list of background marker shapes.</en>
   */
  private List backgroundMarkerShapes ;

  /**
   * <zh>ǰinterval markerͼμ.</zh>
   * <en>The list of foreground interval marker shapes.</en>
   */
  private List foregroundIntervalMarkerShapes ;

  /**
   * <zh>interval markerͼμ.</zh>
   * <en>The list of background interval marker shapes.</en>
   */
  private List backgroundIntervalMarkerShapes ;

  /**
   * <zh>ƫͼεļ.</zh>
   * <en>The list of deviation shapes.</en>
   */
  private List deviationShapes ;

  /**
   * <zh>(difference)ͼεıɫ.</zh>
   * <en>The background of difference shape.</en>
   */
  private List differenceBackgroundShapes ;

  /**
   * <zh>ÿݵӦͼεıͼ.</zh>
   * <en>The data item background shape.</en>
   */
  private List dataItemBackgroundShapes ;

  /**
   * <zh>Сֵֵͼεļ.</zh>
   * <en>The list of min-max value shapes.</en>
   */
  private List minMaxValueShapes ;

  /**
   * <zh>ֵͼεļ.</zh>
   * <en>The list of value-line shapes.</en>
   */
  private List minMaxValueLineShapes ;

  /**
   * <zh>ÿݵӦͼε߼.</zh>
   * <en>The list of join line shapes.</en>
   */
  private List joinLineShapes ;

  /**
   * <zh>ͼμ.</zh>
   * <en>The list of guid line shapes.</en>
   */
  private List guideLineShapes ;

  /**
   * <zh>ÿݵӦͼεļ.</zh>
   * <en>The list of data item shapes.</en>
   */
  private List dataItemShapes ;

  /**
   * <zh>ݵ㱳ͼλ.</zh>
   * <en>The draw type of data background shape.</en>
   */
  private DataBackgroundShapeDrawType dataBackgroundShapeDrawType ;

  /**
   * <zh>̶(tick line)ͼμ.</zh>
   * <en>The list of tick line shapes.</en>
   */
  private List tickLineShapes ;

  /**
   * <zh>̶ȱǩͼμ.</zh>
   * <en>The list of tick label shapes.</en>
   */
  private List tickLabelShapes ;

  /**
   * <zh>ǱͼεĿ̶ֵǩ,ͼβ漰ƶ任Ƕ.</zh>
   * <en>The list of meter arc tick labels.</en>
   */
  private List meterArcTickLabels ;

  /**
   * <zh>ǱͼεĿ̶,ͼβ漰ƶ任Ƕ.</zh>
   * <en>The list of meter arc tick lines.</en>
   */
  private List meterArcTickLines ;

  /**
   * <zh>Ǳָͼ.</zh>
   * <en>The needle shape of meter chart.</en>
   */
  private IChartShape meterNeedleShape ;

  /**
   * <zh>Ǳ̵λͼ.</zh>
   * <en>The unit shape of meter chart.</en>
   */
  private IChartShape meterUnitShape ;

  /**
   * <zh>Ӱͼ.</zh>
   * <en>The list of shadow shapes.</en>
   */
  private List shadowShapes ;

  /**
   * <zh>ͼǩͼ.</zh>
   * <en>The list of pie label shapes.</en>
   */
  private List pieLabelShapes ;

  /**
   * <zh>3DЧıͼ.</zh>
   * <en>The pie 3D effect shapes.</en>
   */
  private List pie3DEffectShapes ;

  /**
   * <zh>뾶.</zh>
   * <en>The radius gridline shapes.</en>
   */
  private List radiusGridlines ;

  /**
   * <zh>.</zh>
   * <en>The angle gridline shapes.</en>
   */
  private List angleGridlines ;

  /**
   * <zh>ݼ.</zh>
   * <en>The dataset.</en>
   */
  private IMatrixDataset dataset ;

  /**
   * <zh>޲.</zh>
   * <en>No argument constructor.</en>
   */
  public RenderShapeCollection() {
    this.borderShape = null ;
    this.backgroundShape = null ;
    this.quadrantBackgroundShapes = new ArrayList() ;
    this.backgroundImageShape = null ;
    this.domainGridLines = new ArrayList() ;
    this.valueGridLines = new ArrayList() ;
    this.valueGridBands = new ArrayList() ;
    this.valueLineShapes = new ArrayList() ;

    this.backgroundMarkerShapes = new ArrayList() ;
    this.chart3DCloseLines = new ArrayList() ;
    this.foregroundMarkerShapes = new ArrayList() ;
    this.backgroundIntervalMarkerShapes = new ArrayList() ;
    this.foregroundIntervalMarkerShapes = new ArrayList() ;

    this.tickLabelShapes = new ArrayList() ;
    this.tickLineShapes = new ArrayList() ;

    this.deviationShapes = new ArrayList() ;
    this.differenceBackgroundShapes = new ArrayList() ;
    this.dataItemBackgroundShapes = new ArrayList() ;
    this.minMaxValueShapes = new ArrayList() ;
    this.minMaxValueLineShapes = new ArrayList() ;
    this.joinLineShapes = new ArrayList() ;
    this.guideLineShapes = new ArrayList() ;
    this.dataItemShapes = new ArrayList() ;
    this.dataBackgroundShapeDrawType = DEFAULT_DATA_BACKGROUND_SHAPE_DRAW_TYPE ;

    this.meterNeedleShape = null ;
    this.meterUnitShape = null ;
    this.meterArcTickLabels = new ArrayList() ;
    this.meterArcTickLines = new ArrayList() ;

    this.shadowShapes = new ArrayList() ;
    this.pieLabelShapes = new ArrayList() ;
    this.pie3DEffectShapes = new ArrayList() ;

    this.radiusGridlines = new ArrayList() ;
    this.angleGridlines = new ArrayList() ;

    this.dataset = null ;
  }

  /**
   * <zh>رͼ.</zh>
   * <en>Returns the background shape.</en>
   *
   * @return IChartShape <zh>ͼ(Ϊnull).</zh>
   * <en>The background shape (maybe null).</en>
   */
  public IChartShape getBackgroundShape() {
    return this.backgroundShape ;
  }

  /**
   * <zh>ñͼ.</zh>
   * <en>Sets the background shape.</en>
   *
   * @param shape IChartShape <zh>ͼ(Ϊnull).</zh>
   * <en>The background shape (null permitted).</en>
   */
  public void setBackgroundShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_BACKGROUND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>һޱͼ.</zh>
   * <en>Add a quadrant background shape.</en>
   *
   * @param shape Shape <zh>ޱͼ.</zh>
   * <en>A quadrant background shape.</en>
   */
  public void addQuadrantBackgroundShape(IChartShape shape) {
    if (shape != null) {
      shape.addChangeListener(this) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_QUADRANT_BACKGROUND_SHAPE) ;
    }

    this.quadrantBackgroundShapes.add(shape) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ޱͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of quadrant background shapes and is unmodifiable.</en>
   *
   * @return List <zh>ޱͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of quadrant background shapes and is unmodifiable.</en>
   */
  public List getQuadrantBackgroundShapes() {
    return Collections.unmodifiableList(this.quadrantBackgroundShapes) ;
  }

  /**
   * <zh>ɾеޱͼ.</zh>
   * <en>Remove all quadrant background shapes.</en>
   */
  public void clearQuadrantBackgroundShapes() {
    for (int i = 0 ; i < this.quadrantBackgroundShapes.size() ; i++) {
      IChartShape annoShapes = (IChartShape)this.quadrantBackgroundShapes.get(i) ;
      if (annoShapes != null) {
        annoShapes.removeChangeListener(this) ;
      }
    }

    this.quadrantBackgroundShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>رͼƬͼ.</zh>
   * <en>Returns the background image shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (maybe null).</en>
   */
  public IChartShape getBackgroundImageShape() {
    return this.backgroundImageShape ;
  }

  /**
   * <zh>ñͼƬͼ.</zh>
   * <en>Sets the background image shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (null permitted).</en>
   */
  public void setBackgroundImageShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_BACKGROUND_IMAGE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundImageShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>Sets the border shape.</en>
   *
   * @param borderShape IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (null permitted).</en>
   */
  public void setBorderShape(IChartShape borderShape) {
    if (borderShape != null) {
      borderShape.setId(shapeIdSequence++) ;
      borderShape.setShapeDrawType(ChartShapeDrawType.RENDER_BORDER_SHAPE) ;
      borderShape.addChangeListener(this) ;
    }

    this.borderShape = borderShape ;
    fireChartShapeChanged(borderShape) ;
  }

  /**
   * <zh>ر߿ͼ.</zh>
   * <en>Returns the border shape.</en>
   *
   * @return IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (maybe null).</en>
   */
  public IChartShape getBorderShape() {
    return this.borderShape ;
  }

  /**
   * <zh>һdomainͼ.</zh>
   * <en>Add a domain grid line shape.</en>
   *
   * @param gridlineShape IChartShape <zh>domainߵͼ (Ϊnull).</zh>
   * <en>A domain grid line shape (null permitted).</en>
   */
  public void addDomainGridLineShape(IChartShape gridlineShape) {
    if (gridlineShape != null) {
      gridlineShape.setId(shapeIdSequence++) ;
      gridlineShape.setShapeDrawType(ChartShapeDrawType.RENDER_DOMAIN_GRIDLINE_SHAPE) ;
      gridlineShape.addChangeListener(this) ;
    }

    this.domainGridLines.add(gridlineShape) ;
  }

  /**
   * <zh>еdomainߵͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of domain grid line shapes and is unmodifiable.</en>
   *
   * @return List <zh>domainߵͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of domain grid line shapes and is unmodifiable.</en>
   */
  public List getDomainGridLineShapes() {
    return Collections.unmodifiableList(this.domainGridLines) ;
  }

  /**
   * <zh>ɾеdomainͼ.</zh>
   * <en>Remove all domain grid line shapes.</en>
   */
  public void clearDomainGridLineShapes() {
    removeShapesListener(this.domainGridLines, this) ;
    this.domainGridLines.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һvalueͼ.</zh>
   * <en>Add a value grid line shape.</en>
   *
   * @param gridlineShape IChartShape <zh>valueߵͼ (Ϊnull).</zh>
   * <en>A value grid line shape (null permitted).</en>
   */
  public void addValueGridLineShape(IChartShape gridlineShape) {
    if (gridlineShape != null) {
      gridlineShape.setId(shapeIdSequence++) ;
      gridlineShape.setShapeDrawType(ChartShapeDrawType.RENDER_VALUE_GRIDLINE_SHAPE) ;
      gridlineShape.addChangeListener(this) ;
    }

    this.valueGridLines.add(gridlineShape) ;
    fireChartShapeChanged(gridlineShape) ;
  }

  /**
   * <zh>еvalueߵͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of value grid line shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>valueߵͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of value grid line shapes and is unmodifiable.</en>
   */
  public List getValueGridLineShapes() {
    return Collections.unmodifiableList(this.valueGridLines) ;
  }

  /**
   * <zh>ɾеvalueͼ.</zh>
   * <en>Remove all value grid line shapes.</en>
   */
  public void clearValueGridLineShapes() {
    removeShapesListener(this.valueGridLines, this) ;
    this.valueGridLines.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һvalueͼ.</zh>
   * <en>Add a value grid band shape.</en>
   *
   * @param gridBandShape IChartShape <zh>valueͼ (Ϊnull).</zh>
   * <en>A value grid band shape (null permitted).</en>
   */
  public void addValueGridBandShape(IChartShape gridBandShape) {
    if (gridBandShape != null) {
      gridBandShape.setId(shapeIdSequence++) ;
      gridBandShape.setShapeDrawType(ChartShapeDrawType.RENDER_VALUE_GRIDBAND_SHAPE) ;
      gridBandShape.addChangeListener(this) ;
    }

    this.valueGridBands.add(gridBandShape) ;
    fireChartShapeChanged(gridBandShape) ;
  }

  /**
   * <zh>еvalueͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of value grid band shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>valueͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of value grid band shapes and is unmodifiable.</en>
   */
  public List getValueGridBandShapes() {
    return Collections.unmodifiableList(this.valueGridBands) ;
  }

  /**
   * <zh>ɾеvalueͼ.</zh>
   * <en>Remove all value grid band shapes.</en>
   */
  public void clearValueGridBandShapes() {
    removeShapesListener(this.valueGridBands, this) ;
    this.valueGridBands.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һֵͼ.</zh>
   * <en>Add a value line shape.</en>
   *
   * @param valueLineShape IChartShape <zh>ֵͼ (Ϊnull).</zh>
   * <en>A value line shape (null permitted).</en>
   */
  public void addValueLineShape(IChartShape valueLineShape) {
    if (valueLineShape != null) {
      valueLineShape.setId(shapeIdSequence++) ;
      valueLineShape.setShapeDrawType(ChartShapeDrawType.RENDER_VALUE_LINE_SHAPE) ;
      valueLineShape.addChangeListener(this) ;
    }

    this.valueLineShapes.add(valueLineShape) ;
    fireChartShapeChanged(valueLineShape) ;
  }

  /**
   * <zh>еֵߵͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of value line shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ֵߵͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of value line shapes and is unmodifiable.</en>
   */
  public List getValueLineShapes() {
    return Collections.unmodifiableList(this.valueLineShapes) ;
  }

  /**
   * <zh>ɾеֵͼ.</zh>
   * <en>Remove all value line shapes.</en>
   */
  public void clearValueLineShapes() {
    removeShapesListener(this.valueLineShapes, this) ;
    this.valueLineShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һmarkerͼ.</zh>
   * <en>Add a background value marker shape.</en>
   *
   * @param markerShape IChartShape <zh>markerͼ (Ϊnull).</zh>
   * <en>A marker shape (null permitted).</en>
   */
  public void addBackgroundMarkerShape(IChartShape markerShape) {
    if (markerShape != null) {
      markerShape.setId(shapeIdSequence++) ;
      markerShape.setShapeDrawType(ChartShapeDrawType.RENDER_BACKGROUND_MARKER_SHAPE) ;
      markerShape.addChangeListener(this) ;
    }

    this.backgroundMarkerShapes.add(markerShape) ;
    fireChartShapeChanged(markerShape) ;
  }

  /**
   * <zh>еıͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of background value marker shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ǰͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of background marker shapes and is unmodifiable.</en>
   */
  public List getBackgroundMarkerShapes() {
    return Collections.unmodifiableList(this.backgroundMarkerShapes) ;
  }

  /**
   * <zh>ɾеıvalue markerͼ.</zh>
   * <en>Remove all background value marker shapes.</en>
   */
  public void clearBackgroundMarkerShapes() {
    removeShapesListener(this.backgroundMarkerShapes, this) ;
    this.backgroundMarkerShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һǰvalue markerͼ.</zh>
   * <en>Add a foreground value marker shape.</en>
   *
   * @param markerShape IChartShape <zh>value markerͼ (Ϊnull)</zh>
   * <en>A marker shape (null permitted).</en>
   */
  public void addForegroundMarkerShape(IChartShape markerShape) {
    if (markerShape != null) {
      markerShape.setId(shapeIdSequence++) ;
      markerShape.setShapeDrawType(ChartShapeDrawType.RENDER_FOREGROUND_MARKER_SHAPE) ;
      markerShape.addChangeListener(this) ;
    }

    this.foregroundMarkerShapes.add(markerShape) ;
    fireChartShapeChanged(markerShape) ;
  }

  /**
   * <zh>еǰͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of foreground shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ǰͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of foreground value marker shapes and is unmodifiable.</en>
   */
  public List getForegroundMarkerShapes() {
    return Collections.unmodifiableList(this.foregroundMarkerShapes) ;
  }

  /**
   * <zh>ɾеǰvalue markerͼ.</zh>
   * <en>Remove all foreground value marker shapes.</en>
   */
  public void clearForegroundMarkerShapes() {
    removeShapesListener(this.foregroundMarkerShapes, this) ;
    this.foregroundMarkerShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ3Dͼ.</zh>
   * <en>Add a line shape for close render area of 3D chart.</en>
   *
   * @param chart3DCloseLines IChartShape <zh>3Dͼ.</zh>
   * <en>A line shape for close render area of 3D chart.</en>
   */
  public void addChart3DCloseLinesShape(IChartShape chart3DCloseLines) {
    if (chart3DCloseLines != null) {
      chart3DCloseLines.setId(shapeIdSequence++) ;
      chart3DCloseLines.setShapeDrawType(ChartShapeDrawType.RENDER_3D_CLOSE_LINE_SHAPE) ;
      chart3DCloseLines.addChangeListener(this) ;
    }

    this.chart3DCloseLines.add(chart3DCloseLines) ;
    fireChartShapeChanged(chart3DCloseLines) ;
  }

  /**
   * <zh>еķ3Dͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of line shapes for close render area of 3D chart and is unmodifiable.</en>
   *
   * @return IChartShape <zh>3Dͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of line shapes for close render area of 3D chart and is unmodifiable.</en>
   */
  public List getChart3DCloseLineShapes() {
    return Collections.unmodifiableList(this.chart3DCloseLines) ;
  }

  /**
   * <zh>ɾеķ3Dͼ.</zh>
   * <en>Remove all line shapes for close render area of 3D chart.</en>
   */
  public void clearChart3DCloseLineShapes() {
    removeShapesListener(this.chart3DCloseLines, this) ;
    this.chart3DCloseLines.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һinterval markerͼ.</zh>
   * <en>Add a background interval value marker shape.</en>
   *
   * @param markerShape IChartShape <zh>interval markerͼ (Ϊnull).</zh>
   * <en>A interval marker shape (null permitted).</en>
   */
  public void addBackgroundIntervalMarkerShape(IChartShape markerShape) {
    if (markerShape != null) {
      markerShape.setId(shapeIdSequence++) ;
      markerShape.setShapeDrawType(ChartShapeDrawType.RENDER_BACKGROUND_INTERVAL_MARKER_SHAPE) ;
      markerShape.addChangeListener(this) ;
    }

    this.backgroundIntervalMarkerShapes.add(markerShape) ;
    fireChartShapeChanged(markerShape) ;
  }

  /**
   * <zh>еıͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of background interval value marker shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ǰͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of background interval marker shapes and is unmodifiable.</en>
   */
  public List getBackgroundIntervalMarkerShapes() {
    return Collections.unmodifiableList(this.backgroundIntervalMarkerShapes) ;
  }

  /**
   * <zh>ɾеıinterval value markerͼ.</zh>
   * <en>Remove all background interval value marker shapes.</en>
   */
  public void clearBackgroundIntervalMarkerShapes() {
    removeShapesListener(this.backgroundIntervalMarkerShapes, this) ;
    this.backgroundIntervalMarkerShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һǰinterval value markerͼ.</zh>
   * <en>Add a foreground interval value marker shape.</en>
   *
   * @param markerShape IChartShape <zh>markerͼ (Ϊnull)</zh>
   * <en>A marker shape (null permitted).</en>
   */
  public void addForegroundIntervalMarkerShape(IChartShape markerShape) {
    if (markerShape != null) {
      markerShape.setId(shapeIdSequence++) ;
      markerShape.setShapeDrawType(ChartShapeDrawType.RENDER_FOREGROUND_INTERVAL_MARKER_SHAPE) ;
      markerShape.addChangeListener(this) ;
    }

    this.foregroundIntervalMarkerShapes.add(markerShape) ;
    fireChartShapeChanged(markerShape) ;
  }

  /**
   * <zh>еǰͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of foreground interval marker shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ǰͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of foreground interval value marker shapes and is unmodifiable.</en>
   */
  public List getForegroundIntervalMarkerShapes() {
    return Collections.unmodifiableList(this.foregroundIntervalMarkerShapes) ;
  }

  /**
   * <zh>ɾеǰinterval value markerͼ.</zh>
   * <en>Remove all foreground interval value marker shapes.</en>
   */
  public void clearForegroundIntervalMarkerShapes() {
    removeShapesListener(this.foregroundIntervalMarkerShapes, this) ;
    this.foregroundIntervalMarkerShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һƫͼ.</zh>
   * <en>Add a deviation shape.</en>
   *
   * @param shape IChartShape <zh>ƫͼ (Ϊnull).</zh>
   * <en>A deviation shape (null permitted).</en>
   */
  public void addDeviationShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_DEVIATION_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.deviationShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ƫͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of deviation shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ƫͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of deviation shapes and is unmodifiable.</en>
   */
  public List getDeviationShapes() {
    return Collections.unmodifiableList(this.deviationShapes) ;
  }

  /**
   * <zh>ɾеƫͼ.</zh>
   * <en>Remove all deviation shapes.</en>
   */
  public void clearDeviationShapes() {
    removeShapesListener(this.deviationShapes, this) ;
    this.deviationShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ(difference)ͼεıɫ.</zh>
   * <en>Add a background shape of difference shape.</en>
   *
   * @param shape IChartShape <zh>(difference)ͼεıɫͼ (Ϊnull).</zh>
   * <en>A background of difference shape (null permitted).</en>
   */
  public void addDifferenceBackgroundShapes(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_DIFFERENCE_BACKGROUND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.differenceBackgroundShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ز(difference)ͼεıɫͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of background shapes of difference shape and is unmodifiable.</en>
   *
   * @return IChartShape <zh>(difference)ͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of shapes and is unmodifiable.</en>
   */
  public List getDifferenceBackgroundShapes() {
    return Collections.unmodifiableList(this.differenceBackgroundShapes) ;
  }

  /**
   * <zh>ɾеĲ(difference)ͼεıɫͼ.</zh>
   * <en>Remove all background shapes of difference shape.</en>
   */
  public void clearDifferenceBackgroundShapes() {
    removeShapesListener(this.differenceBackgroundShapes, this) ;
    this.differenceBackgroundShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ݵ㱳ͼλ.</zh>
   * <en>Returns the draw type of data background shape.</en>
   *
   * @return DataBackgroundShapeDrawType <zh>Ϊnull.</zh>
   * <en>never null.</en>
   */
  public DataBackgroundShapeDrawType getDataBackgroundShapeDrawType() {
    return this.dataBackgroundShapeDrawType ;
  }

  /**
   * <zh>ݵ㱳ͼλ.</zh>
   * <en>Sets the draw type of data background shape.</en>
   *
   * @param type DataBackgroundShapeDrawType <zh> (Ϊnull).</zh>
   * <en>The draw type (null not permitted).</en>
   */
  public void setDataBackgroundShapeDrawType(DataBackgroundShapeDrawType type) {
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument") ;
    }

    this.dataBackgroundShapeDrawType = type ;
  }

  /**
   * <zh>һݵӦͼεıͼ.</zh>
   * <en>Add a data item background shape.</en>
   *
   * @param shape IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>A background shape (null permitted).</en>
   */
  public void addDataItemBackgroundShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_DATA_ITEM_BACKGROUND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.dataItemBackgroundShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ÿݵӦͼεıͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of data item background shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of shapes and is unmodifiable.</en>
   */
  public List getDataItemBackgroundShapes() {
    return Collections.unmodifiableList(this.dataItemBackgroundShapes) ;
  }

  /**
   * <zh>ɾеݵӦͼεıͼ.</zh>
   * <en>Remove all data item background shapes.</en>
   */
  public void clearDataItemBackgroundShapes() {
    removeShapesListener(this.dataItemBackgroundShapes, this) ;
    this.dataItemBackgroundShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һСֵֵͼ.</zh>
   * <en>Add a min-max value shape.</en>
   *
   * @param shape IChartShape <zh>ͼζ (Ϊnull).</zh>
   * <en>A min-max shape (null permitted).</en>
   */
  public void addMinMaxShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.minMaxValueShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>Сֵֵͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of min-max value shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>Сֵֵͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of min-max value shapes and is unmodifiable.</en>
   */
  public List getMinMaxValueShapes() {
    return Collections.unmodifiableList(this.minMaxValueShapes) ;
  }

  /**
   * <zh>ɾеСֵֵͼ.</zh>
   * <en>Remove all min-max value shapes.</en>
   */
  public void clearMinMaxValueShapes() {
    removeShapesListener(this.minMaxValueShapes, this) ;
    this.minMaxValueShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һֵͼ</zh>
   * <en>Add a value-line shape.</en>
   *
   * @param shape IChartShape <zh>ͼζ (Ϊnull).</zh>
   * <en>A value-line shape (null permitted).</en>
   */
  public void addMinMaxValueLineShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_LINE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.minMaxValueLineShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ֵͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of value-line shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ֵͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of value-line shapes and is unmodifiable.</en>
   */
  public List getMinMaxValueLineShapes() {
    return Collections.unmodifiableList(this.minMaxValueLineShapes) ;
  }

  /**
   * <zh>ɾеֵͼ.</zh>
   * <en>Remove all value-line shapes.</en>
   */
  public void clearMinMaxValueLineShapes() {
    removeShapesListener(this.minMaxValueLineShapes, this) ;
    this.minMaxValueLineShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һͼ.</zh>
   * <en>Add a join line shape.</en>
   *
   * @param shape IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>A join line shape (null permitted).</en>
   */
  public void addJoinLineShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_JOIN_LINE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.joinLineShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of join line shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of join line shapes and is unmodifiable.</en>
   */
  public List getJoinLineShapes() {
    return Collections.unmodifiableList(this.joinLineShapes) ;
  }

  /**
   * <zh>ɾеͼ.</zh>
   * <en>Remove all join line shapes.</en>
   */
  public void clearJoinLineShapes() {
    removeShapesListener(this.joinLineShapes, this) ;
    this.joinLineShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һͼ.</zh>
   * <en>Add a guid line shape.</en>
   *
   * @param shape IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>A guid line shape (null permitted).</en>
   */
  public void addGuideLineShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_GUIDE_LINE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.guideLineShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of guid line shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of guid line shapes and is unmodifiable.</en>
   */
  public List getGuideLineShapes() {
    return Collections.unmodifiableList(this.guideLineShapes) ;
  }

  /**
   * <zh>ɾеͼ.</zh>
   * <en>Remove all guid line shapes.</en>
   */
  public void clearGuideLineShapes() {
    removeShapesListener(this.guideLineShapes, this) ;
    this.guideLineShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һݵӦͼ.</zh>
   * <en>Add a data item shape.</en>
   *
   * @param shape IChartShape <zh>ݵӦͼ (Ϊnull).</zh>
   * <en>A data item shape (null permitted).</en>
   */
  public void addDataItemShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.dataItemShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ݵӦͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of data item shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ݵӦͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of data item shapes and is unmodifiable.</en>
   */
  public List getDataItemShapes() {
    return Collections.unmodifiableList(this.dataItemShapes) ;
  }

  /**
   * <zh>ɾеݵͼ.</zh>
   * <en>Remove all data item shapes.</en>
   */
  public void clearDataItemShapes() {
    removeShapesListener(this.dataItemShapes, this) ;
    this.dataItemShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ̶ֵͼ.</zh>
   * <en>Add a tick line shape.</en>
   *
   * @param tickLineShape IChartShape <zh>̶ֵͼ (Ϊnull).</zh>
   * <en>A tick line shape (null permitted).</en>
   */
  public void addTickLineShape(IChartShape tickLineShape) {
    if (tickLineShape != null) {
      tickLineShape.setId(shapeIdSequence++) ;
      tickLineShape.setShapeDrawType(ChartShapeDrawType.RENDER_TICK_LINE_SHAPE) ;
      tickLineShape.addChangeListener(this) ;
    }

    this.tickLineShapes.add(tickLineShape) ;
    fireChartShapeChanged(tickLineShape) ;
  }

  /**
   * <zh>ȫ̶ֵͼ.</zh>
   * <en>Add all tick line shapes.</en>
   *
   * @param tickLineShapes List <zh>̶ֵͼμ (Ϊnull).</zh>
   * <en>A collection of tick line shapes (null permitted).</en>
   */
  public void addAllTickLineShape(List tickLineShapes) {
    if (tickLineShapes != null) {
      for (int i = 0 ; i < tickLineShapes.size() ; i++) {
        IChartShape shape = (IChartShape) tickLineShapes.get(i) ;
        if (shape != null) {
          shape.setId(shapeIdSequence++) ;
          shape.setShapeDrawType(ChartShapeDrawType.RENDER_TICK_LINE_SHAPE) ;
          shape.addChangeListener(this) ;
        }
      }

      this.tickLineShapes.addAll(tickLineShapes) ;
      fireChartShapeChanged(null) ;
    }
  }

  /**
   * <zh>п̶ͼεͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of tick line shapes and is unmodifiable.</en>
   *
   * @return List <zh>̶ͼεͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of tick line shapes and is unmodifiable.</en>
   */
  public List getTickLineShapes() {
    return Collections.unmodifiableList(this.tickLineShapes) ;
  }

  /**
   * <zh>ɾеĿ̶ͼ.</zh>
   * <en>Remove all tick line shapes.</en>
   */
  public void clearTickLineShapes() {
    removeShapesListener(this.tickLineShapes, this) ;
    this.tickLineShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ̶ȱǩͼ.</zh>
   * <en>Add a tick label shape.</en>
   *
   * @param tickLabelShape IChartShape <zh>̶ȱǩͼ (Ϊnull).</zh>
   * <en>A tick label shape (null permitted).</en>
   */
  public void addTickLabelShape(IChartShape tickLabelShape) {
    if (tickLabelShape != null) {
      tickLabelShape.setId(shapeIdSequence++) ;
      tickLabelShape.setShapeDrawType(ChartShapeDrawType.RENDER_TICK_LABEL_SHAPE) ;
      tickLabelShape.addChangeListener(this) ;
    }

    this.tickLabelShapes.add(tickLabelShape) ;
    fireChartShapeChanged(tickLabelShape) ;
  }

  /**
   * <zh>ȫ̶ȱǩͼ.</zh>
   * <en>Add all tick label shapes.</en>
   *
   * @param tickLabelShapes List <zh>̶ȱǩͼμ (Ϊnull).</zh>
   * <en>A collection of tick label shapes (null permitted).</en>
   */
  public void addAllTickLabelShape(List tickLabelShapes) {
    if (tickLabelShapes != null) {
      for (int i = 0 ; i < tickLabelShapes.size() ; i++) {
        IChartShape shape = (IChartShape) tickLabelShapes.get(i) ;
        if (shape != null) {
          shape.setId(shapeIdSequence++) ;
          shape.setShapeDrawType(ChartShapeDrawType.RENDER_TICK_LABEL_SHAPE) ;
          shape.addChangeListener(this) ;
        }
      }

      this.tickLabelShapes.addAll(tickLabelShapes) ;
      fireChartShapeChanged(null) ;
    }
  }

  /**
   * <zh>п̶ȱǩͼεͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of tick label shapes and is unmodifiable.</en>
   *
   * @return List <zh>̶ȱǩͼεͼ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of tick label shapes and is unmodifiable.</en>
   */
  public List getTickLabelShapes() {
    return Collections.unmodifiableList(this.tickLabelShapes) ;
  }

  /**
   * <zh>ɾеĿ̶ȱǩͼ.</zh>
   * <en>Remove all tick label shapes.</en>
   */
  public void clearTickLabelShapes() {
    removeShapesListener(this.tickLabelShapes, this) ;
    this.tickLabelShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>Ǳָͼ.</zh>
   * <en>Returns the needle shape of meter chart.</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (maybe null).</en>
   */
  public IChartShape getMeterNeedleShape() {
    return this.meterNeedleShape ;
  }

  /**
   * <zh>Ǳָͼ.</zh>
   * <en>Sets the needle shape of meter chart.</en>
   *
   * @param meterNeedleShape IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (maybe null).</en>
   */
  public void setMeterNeedleShape(IChartShape meterNeedleShape) {
    if (meterNeedleShape != null) {
      meterNeedleShape.setId(shapeIdSequence++) ;
      meterNeedleShape.setShapeDrawType(ChartShapeDrawType.RENDER_METER_NEEDLE_SHAPE) ;
      meterNeedleShape.addChangeListener(this) ;
    }

    this.meterNeedleShape = meterNeedleShape ;
    fireChartShapeChanged(meterNeedleShape) ;
  }

  /**
   * <zh>Ǳ̵λͼ.</zh>
   * <en>Returns the unit shape of meter chart.</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (maybe null).</en>
   */
  public IChartShape getMeterUnitShape() {
    return this.meterUnitShape ;
  }

  /**
   * <zh>Ǳ̵λͼ.</zh>
   * <en>Sets the unit shape of meter chart.</en>
   *
   * @param meterUnitShape IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (maybe null).</en>
   */
  public void setMeterUnitShape(IChartShape meterUnitShape) {
    if (meterUnitShape != null) {
      meterUnitShape.setId(shapeIdSequence++) ;
      meterUnitShape.setShapeDrawType(ChartShapeDrawType.RENDER_METER_UNIT_SHAPE) ;
      meterUnitShape.addChangeListener(this) ;
    }

    this.meterUnitShape = meterUnitShape ;
    fireChartShapeChanged(meterUnitShape) ;
  }

  /**
   * <zh>ǱͼεĿ̶ֵǩ,˼ϲ޸.</zh>
   * <en>The list of meter arc tick labels and is unmodifiable.</en>
   *
   * @return List <zh>ͼμ.</zh>
   * <en>The list of meter arc tick labels and is unmodifiable.</en>
   */
  public List getMeterArcTickLabels() {
    return this.meterArcTickLabels ;
  }

  /**
   * <zh>ǱͼεĿ̶,˼ϲ޸.</zh>
   * <en>The list of meter arc tick lines and is unmodifiable.</en>
   *
   * @return List <zh>ͼμ.</zh>
   * <en>The list of meter arc tick lines and is unmodifiable.</en>
   */
  public List getMeterArcTickLines() {
    return this.meterArcTickLines ;
  }

  /**
   * <zh>һӰͼ.</zh>
   * <en>Add a shadow shape.</en>
   *
   * @param shape IChartShape <zh>Ӱͼ (Ϊnull).</zh>
   * <en>A shadow shape (null permitted).</en>
   */
  public void addShadowShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_SHADOW_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.shadowShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>Ӱͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of shadow shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>Ӱͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of shadow shapes and is unmodifiable.</en>
   */
  public List getShadowShapes() {
    return Collections.unmodifiableList(this.shadowShapes) ;
  }

  /**
   * <zh>ɾеӰͼ.</zh>
   * <en>Remove all shadow shapes.</en>
   */
  public void clearShadowShapes() {
    removeShapesListener(this.shadowShapes, this) ;
    this.shadowShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һͼǩͼ.</zh>
   * <en>Add a pie label shape.</en>
   *
   * @param shape IChartShape <zh>ͼǩͼ (Ϊnull).</zh>
   * <en>A pie label shape (null permitted).</en>
   */
  public void addPieLabelShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_PIE_LABEL_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.pieLabelShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>رͼǩͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of pie label shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼǩͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of pie label shapes and is unmodifiable.</en>
   */
  public List getPieLabelShapes() {
    return Collections.unmodifiableList(this.pieLabelShapes) ;
  }

  /**
   * <zh>ɾеıͼǩͼ.</zh>
   * <en>Remove all pie label shapes.</en>
   */
  public void clearPieLabelShapes() {
    removeShapesListener(this.pieLabelShapes, this) ;
    this.pieLabelShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ3DЧıͼͼ.</zh>
   * <en>Add a pie 3D effect shape.</en>
   *
   * @param shape IChartShape <zh>3DЧıͼ (Ϊnull).</zh>
   * <en>A pie 3D effect shape (null permitted).</en>
   */
  public void addPie3DEffectShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_PIE_3D_EFFECT_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.pie3DEffectShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ػ3DЧıͼͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of pie 3D effect shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼǩͼεļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of pie 3D effect shapes and is unmodifiable.</en>
   */
  public List getPie3DEffectShapes() {
    return Collections.unmodifiableList(this.pie3DEffectShapes) ;
  }

  /**
   * <zh>ɾеĻ3DЧıͼͼ.</zh>
   * <en>Remove all pie 3D effect shapes.</en>
   */
  public void clearPie3DEffectShapes() {
    removeShapesListener(this.pie3DEffectShapes, this) ;
    this.pie3DEffectShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һ뾶.</zh>
   * <en>Add a radius gridline shape.</en>
   *
   * @param shape IChartShape <zh> (Ϊnull).</zh>
   * <en>A radius gridline shape (null permitted).</en>
   */
  public void addRadiusGridlineShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_RADIUS_GRIDLINE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.radiusGridlines.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ذ뾶ߵļ,˼ϲ޸.</zh>
   * <en>Returns the list of radius gridline shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ߵļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of radius gridline shapes and is unmodifiable.</en>
   */
  public List getRadiusGridlines() {
    return Collections.unmodifiableList(this.radiusGridlines) ;
  }

  /**
   * <zh>ɾеİ뾶ͼ.</zh>
   * <en>Remove all radius gridline shapes.</en>
   */
  public void clearRadiusGridlines() {
    removeShapesListener(this.radiusGridlines, this) ;
    this.radiusGridlines.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>һǶȱ.</zh>
   * <en>Add an angle gridline shape.</en>
   *
   * @param shape IChartShape <zh> (Ϊnull).</zh>
   * <en>An angle gridline shape (null permitted).</en>
   */
  public void addAngleGridlineShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.RENDER_ANGLE_GRIDLINE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.angleGridlines.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ؽǶȱߵļ,˼ϲ޸.</zh>
   * <en>Returns the list of angle gridline shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ߵļ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of angle gridline shapes and is unmodifiable.</en>
   */
  public List getAngleGridlines() {
    return Collections.unmodifiableList(this.angleGridlines) ;
  }

  /**
   * <zh>ɾеĽǶȱͼ.</zh>
   * <en>Remove all angle gridline shapes.</en>
   */
  public void clearAngleGridlines() {
    removeShapesListener(this.angleGridlines, this) ;
    this.angleGridlines.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Returns the dataset.</en>
   *
   * @return IMatrixDataset <zh>ݼ.</zh>
   * <en>The dataset.</en>
   */
  public IMatrixDataset getDataset() {
    return this.dataset ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Sets the dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public void setDataset(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    this.dataset = dataset ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    super.clip(g2) ;

    if (this.backgroundShape != null) {
      this.backgroundShape.draw(g2) ;
    }
    for (int i = 0 ; i < this.quadrantBackgroundShapes.size() ; i++) {
      IChartShape shape = (IChartShape)this.quadrantBackgroundShapes.get(i) ;
      if (shape != null) {
        shape.draw(g2) ;
      }
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.draw(g2) ;
    }
    if (this.borderShape != null) {
      this.borderShape.draw(g2) ;
    }

    draw(g2, this.domainGridLines) ;
    draw(g2, this.valueGridBands) ;
    draw(g2, this.valueGridLines) ;
    draw(g2, this.backgroundMarkerShapes) ;
    draw(g2, this.backgroundIntervalMarkerShapes) ;
    draw(g2, this.valueLineShapes) ;
    draw(g2, this.shadowShapes) ;
    draw(g2, this.pie3DEffectShapes) ;
    draw(g2, this.deviationShapes) ;
    draw(g2, this.differenceBackgroundShapes) ;
    draw(g2, this.radiusGridlines) ;
    draw(g2, this.angleGridlines) ;
    draw(g2, this.minMaxValueShapes) ;
    draw(g2, this.minMaxValueLineShapes) ;
    draw(g2, this.joinLineShapes) ;
    draw(g2, this.guideLineShapes) ;

    if (this.dataBackgroundShapeDrawType.equals(DataBackgroundShapeDrawType.ALL_BY_ALL)) {
      draw(g2, this.dataItemBackgroundShapes) ;
      for (int i = 0 ; i < this.dataItemShapes.size() ; i++) {
        drawDataItemShape(g2, i, (IChartShape)this.dataItemShapes.get(i)) ;
      }
    } else if (this.dataBackgroundShapeDrawType.equals(DataBackgroundShapeDrawType.ONE_BY_ONE)) {
      draw(g2, this.dataItemBackgroundShapes) ;
      for (int i = 0 ; i < this.dataItemShapes.size() ; i++) {
        if (i < this.dataItemBackgroundShapes.size()) {
          IChartShape bgShape = (IChartShape)this.dataItemBackgroundShapes.get(i) ;
          if (bgShape != null) {
            bgShape.draw(g2) ;
          }
        }
        drawDataItemShape(g2, i, (IChartShape)this.dataItemShapes.get(i)) ;
      }
    } else if (this.dataBackgroundShapeDrawType.equals(DataBackgroundShapeDrawType.ONE_BY_ROW)) {
      int cnt = 0 ;
      int rows = this.dataset.getRowCount() ;
      for (int i = 0 ; i < rows ; i++) {
        if (i < this.dataItemBackgroundShapes.size()) {
          IChartShape bgShape = (IChartShape)this.dataItemBackgroundShapes.get(i) ;
          if (bgShape != null) {
            bgShape.draw(g2) ;
          }
        }

        int currRowIdx = this.dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowIdx ; j++) {
          if (cnt < this.dataItemShapes.size()) {
            drawDataItemShape(g2, cnt, (IChartShape)this.dataItemShapes.get(cnt)) ;
            cnt++ ;
          }
        }
      }
    } else if (this.dataBackgroundShapeDrawType.equals(DataBackgroundShapeDrawType.ONE_BY_TWO_ROW)) {
      int cnt = 0 ;
      int rows = this.dataset.getRowCount() ;
      int bgShapeCnt = 0 ;
      for (int i = 0 ; i < rows ; i++) {
        if (bgShapeCnt < this.dataItemBackgroundShapes.size() && i % 2 == 0) {
          IChartShape bgShape = (IChartShape)this.dataItemBackgroundShapes.get(bgShapeCnt) ;
          if (bgShape != null) {
            bgShape.draw(g2) ;
          }
          bgShapeCnt++ ;
        }

        int currRowIdx = this.dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowIdx ; j++) {
          if (cnt < this.dataItemShapes.size()) {
            drawDataItemShape(g2, cnt, (IChartShape)this.dataItemShapes.get(cnt)) ;
            cnt++ ;
          }
        }
      }
    }

    draw(g2, this.tickLabelShapes) ;
    draw(g2, this.chart3DCloseLines) ;
    draw(g2, this.foregroundMarkerShapes) ;
    draw(g2, this.foregroundIntervalMarkerShapes) ;
    draw(g2, this.meterArcTickLabels) ;
    draw(g2, this.meterArcTickLines) ;

    if (this.meterNeedleShape != null) {
      this.meterNeedleShape.draw(g2) ;
    }
    if (this.meterUnitShape != null) {
      this.meterUnitShape.draw(g2) ;
    }

    draw(g2, this.pieLabelShapes) ;
    super.restoreClip(g2) ;
  }

  /**
   * <zh>ݵͼ.</zh>
   * <en>Draw data item shape.</en>
   *
   * @param g2 Graphics2D
   * @param index int
   * @param shape IChartShape
   */
  private void drawDataItemShape(Graphics2D g2, int index, IChartShape shape) {
    if (shape == null) {
      return ;
    }
    if (shape instanceof MeterArcChartShape) {
      if (index == 0) {
        this.meterArcTickLabels.clear() ;
        this.meterArcTickLines.clear() ;
      }
      shape.draw(g2) ;

      Collection arcTickLabelShapes = ((MeterArcChartShape) shape).getTickLabelShapes() ;
      Iterator iter = arcTickLabelShapes.iterator() ;
      while (iter.hasNext()) {
        IChartShape tmpShape = (IChartShape) iter.next() ;
        if (tmpShape != null) {
          tmpShape.setShapeDrawType(ChartShapeDrawType.RENDER_METER_ARC_TICK_LABEL_SHAPE) ;
        }
      }
      this.meterArcTickLabels.addAll(arcTickLabelShapes) ;

      Collection arcTickLineShapes = ((MeterArcChartShape) shape).getTickLineShapes() ;
      iter = arcTickLineShapes.iterator() ;
      while (iter.hasNext()) {
        IChartShape tmpShape = (IChartShape) iter.next() ;
        if (tmpShape != null) {
          tmpShape.setShapeDrawType(ChartShapeDrawType.RENDER_METER_ARC_TICK_LINE_SHAPE) ;
        }
      }
      this.meterArcTickLines.addAll(arcTickLineShapes) ;
    } else {
      shape.draw(g2) ;
    }
  }

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    if (this.borderShape != null) {
      this.borderShape.translate(x, y) ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.translate(x, y) ;
    }
    for (int i = 0 ; i < this.quadrantBackgroundShapes.size() ; i++) {
      IChartShape shape = (IChartShape)this.quadrantBackgroundShapes.get(i) ;
      if (shape != null) {
        shape.translate(x, y) ;
      }
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.translate(x, y) ;
    }

    translate(x, y, this.domainGridLines) ;
    translate(x, y, this.valueGridBands) ;
    translate(x, y, this.valueGridLines) ;
    translate(x, y, this.backgroundMarkerShapes) ;
    translate(x, y, this.backgroundIntervalMarkerShapes) ;
    translate(x, y, this.valueLineShapes) ;
    translate(x, y, this.deviationShapes) ;
    translate(x, y, this.differenceBackgroundShapes) ;
    translate(x, y, this.dataItemBackgroundShapes) ;
    translate(x, y, this.minMaxValueShapes) ;
    translate(x, y, this.minMaxValueLineShapes) ;
    translate(x, y, this.joinLineShapes) ;
    translate(x, y, this.guideLineShapes) ;
    translate(x, y, this.dataItemShapes) ;
    translate(x, y, this.chart3DCloseLines) ;
    translate(x, y, this.foregroundMarkerShapes) ;
    translate(x, y, this.foregroundIntervalMarkerShapes) ;

    if (this.meterUnitShape != null) {
      this.meterUnitShape.translate(x, y) ;
    }
    if (this.meterNeedleShape != null) {
      this.meterNeedleShape.translate(x, y) ;
    }

    translate(x, y, this.meterArcTickLabels) ;
    translate(x, y, this.meterArcTickLines) ;
    translate(x, y, this.shadowShapes) ;
    translate(x, y, this.pieLabelShapes) ;
    translate(x, y, this.pie3DEffectShapes) ;
    translate(x, y, this.radiusGridlines) ;
    translate(x, y, this.angleGridlines) ;
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    if (this.borderShape != null) {
      this.borderShape.removeAllChangeListener() ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.removeAllChangeListener() ;
    }
    clearShapesListener(this.quadrantBackgroundShapes) ;
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.removeAllChangeListener() ;
    }

    clearShapesListener(this.domainGridLines) ;
    clearShapesListener(this.valueGridBands) ;
    clearShapesListener(this.valueGridLines) ;
    clearShapesListener(this.backgroundMarkerShapes) ;
    clearShapesListener(this.backgroundIntervalMarkerShapes) ;
    clearShapesListener(this.valueLineShapes) ;
    clearShapesListener(this.deviationShapes) ;
    clearShapesListener(this.differenceBackgroundShapes) ;
    clearShapesListener(this.dataItemBackgroundShapes) ;
    clearShapesListener(this.minMaxValueShapes) ;
    clearShapesListener(this.minMaxValueLineShapes) ;
    clearShapesListener(this.joinLineShapes) ;
    clearShapesListener(this.guideLineShapes) ;
    clearShapesListener(this.dataItemShapes) ;
    clearShapesListener(this.chart3DCloseLines) ;
    clearShapesListener(this.foregroundMarkerShapes) ;
    clearShapesListener(this.foregroundIntervalMarkerShapes) ;

    if (this.meterUnitShape != null) {
      this.meterUnitShape.removeAllChangeListener() ;
    }
    if (this.meterNeedleShape != null) {
      this.meterNeedleShape.removeAllChangeListener() ;
    }

    clearShapesListener(this.meterArcTickLabels) ;
    clearShapesListener(this.meterArcTickLines) ;
    clearShapesListener(this.shadowShapes) ;
    clearShapesListener(this.pieLabelShapes) ;
    clearShapesListener(this.pie3DEffectShapes) ;
    clearShapesListener(this.radiusGridlines) ;
    clearShapesListener(this.angleGridlines) ;
  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.borderShape) ;
    clearSelectedShape(this.backgroundShape) ;
    clearSelectedShape(this.quadrantBackgroundShapes) ;
    clearSelectedShape(this.backgroundImageShape) ;

    clearSelectedShape(this.domainGridLines) ;
    clearSelectedShape(this.valueGridBands) ;
    clearSelectedShape(this.valueGridLines) ;
    clearSelectedShape(this.backgroundMarkerShapes) ;
    clearSelectedShape(this.backgroundIntervalMarkerShapes) ;
    clearSelectedShape(this.valueLineShapes) ;
    clearSelectedShape(this.deviationShapes) ;
    clearSelectedShape(this.differenceBackgroundShapes) ;
    clearSelectedShape(this.dataItemBackgroundShapes) ;
    clearSelectedShape(this.minMaxValueShapes) ;
    clearSelectedShape(this.minMaxValueLineShapes) ;
    clearSelectedShape(this.joinLineShapes) ;
    clearSelectedShape(this.guideLineShapes) ;
    clearSelectedShape(this.dataItemShapes) ;
    clearSelectedShape(this.chart3DCloseLines) ;
    clearSelectedShape(this.foregroundMarkerShapes) ;
    clearSelectedShape(this.foregroundIntervalMarkerShapes) ;

    clearSelectedShape(this.meterUnitShape) ;
    clearSelectedShape(this.meterNeedleShape) ;

    clearSelectedShape(this.meterArcTickLabels) ;
    clearSelectedShape(this.meterArcTickLines) ;
    clearSelectedShape(this.shadowShapes) ;
    clearSelectedShape(this.pieLabelShapes) ;
    clearSelectedShape(this.pie3DEffectShapes) ;
    clearSelectedShape(this.radiusGridlines) ;
    clearSelectedShape(this.angleGridlines) ;
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    IChartShape shape = null ;
    if (flag || drawType.equals(ChartShapeDrawType.RENDER_METER_ARC_TICK_LINE_SHAPE)) {
      shape = findShape(this.meterArcTickLines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_METER_ARC_TICK_LABEL_SHAPE)) {
      shape = findShape(this.meterArcTickLabels, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_METER_NEEDLE_SHAPE)) {
      if (this.meterNeedleShape != null && this.meterNeedleShape.contains(point)) {
        return this.meterNeedleShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_METER_UNIT_SHAPE)) {
      if (this.meterUnitShape != null && this.meterUnitShape.contains(point)) {
        return this.meterUnitShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_FOREGROUND_INTERVAL_MARKER_SHAPE)) {
      shape = findShape(this.foregroundIntervalMarkerShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_FOREGROUND_MARKER_SHAPE)) {
      shape = findShape(this.foregroundMarkerShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_3D_CLOSE_LINE_SHAPE)) {
      shape = findShape(this.chart3DCloseLines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_DEVIATION_SHAPE)) {
      shape = findShape(this.deviationShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_DIFFERENCE_BACKGROUND_SHAPE)) {
      shape = findShape(this.differenceBackgroundShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_DATA_ITEM_BACKGROUND_SHAPE)) {
      shape = findShape(this.dataItemBackgroundShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_SHAPE)) {
      shape = findShape(this.minMaxValueShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_LINE_SHAPE)) {
      shape = findShape(this.minMaxValueLineShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_JOIN_LINE_SHAPE)) {
      shape = findShape(this.joinLineShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_GUIDE_LINE_SHAPE)) {
      shape = findShape(this.guideLineShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE)) {
      shape = findShape(this.dataItemShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_PIE_LABEL_SHAPE)) {
      shape = findShape(this.pieLabelShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_PIE_3D_EFFECT_SHAPE)) {
      shape = findShape(this.pie3DEffectShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_VALUE_LINE_SHAPE)) {
      shape = findShape(this.valueLineShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_BACKGROUND_INTERVAL_MARKER_SHAPE)) {
      shape = findShape(this.backgroundIntervalMarkerShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_BACKGROUND_MARKER_SHAPE)) {
      shape = findShape(this.backgroundMarkerShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_VALUE_GRIDLINE_SHAPE)) {
      shape = findShape(this.valueGridLines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_VALUE_GRIDBAND_SHAPE)) {
      shape = findShape(this.valueGridBands, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_DOMAIN_GRIDLINE_SHAPE)) {
      shape = findShape(this.domainGridLines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_RADIUS_GRIDLINE_SHAPE)) {
      shape = findShape(this.radiusGridlines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_ANGLE_GRIDLINE_SHAPE)) {
      shape = findShape(this.angleGridlines, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_SHADOW_SHAPE)) {
      shape = findShape(this.shadowShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_BACKGROUND_IMAGE_SHAPE)) {
      if (this.backgroundImageShape != null && this.backgroundImageShape.contains(point)) {
        return this.backgroundImageShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_QUADRANT_BACKGROUND_SHAPE)) {
      shape = findShape(this.quadrantBackgroundShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_BACKGROUND_SHAPE)) {
      if (this.backgroundShape != null && this.backgroundShape.contains(point)) {
        return this.backgroundShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.RENDER_BORDER_SHAPE)) {
      if (this.borderShape != null && this.borderShape.contains(point)) {
        return this.borderShape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    if (drawTypes == null) {
      throw new IllegalArgumentException("Null 'drawTypes' argument.") ;
    }

    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.RENDER_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.backgroundShape, fragment,
                     ChartShapeDrawType.RENDER_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_QUADRANT_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.quadrantBackgroundShapes, fragment,
                     ChartShapeDrawType.RENDER_QUADRANT_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_BACKGROUND_IMAGE_SHAPE)) {
      addHotspotArea(result, this.backgroundImageShape, fragment,
                     ChartShapeDrawType.RENDER_BACKGROUND_IMAGE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_BORDER_SHAPE)) {
      addHotspotArea(result, this.borderShape, fragment, ChartShapeDrawType.RENDER_BORDER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_DOMAIN_GRIDLINE_SHAPE)) {
      addHotspotArea(result, this.domainGridLines, fragment,
                     ChartShapeDrawType.RENDER_DOMAIN_GRIDLINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_VALUE_GRIDBAND_SHAPE)) {
      addHotspotArea(result, this.valueGridBands, fragment,
                     ChartShapeDrawType.RENDER_VALUE_GRIDBAND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_VALUE_GRIDLINE_SHAPE)) {
      addHotspotArea(result, this.valueGridLines, fragment,
                     ChartShapeDrawType.RENDER_VALUE_GRIDLINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_BACKGROUND_MARKER_SHAPE)) {
      addHotspotArea(result, this.backgroundMarkerShapes, fragment,
                     ChartShapeDrawType.RENDER_BACKGROUND_MARKER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_BACKGROUND_INTERVAL_MARKER_SHAPE)) {
      addHotspotArea(result, this.backgroundIntervalMarkerShapes, fragment,
                     ChartShapeDrawType.RENDER_BACKGROUND_INTERVAL_MARKER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_VALUE_LINE_SHAPE)) {
      addHotspotArea(result, this.valueLineShapes, fragment,
                     ChartShapeDrawType.RENDER_VALUE_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_SHADOW_SHAPE)) {
      addHotspotArea(result, this.shadowShapes, fragment, ChartShapeDrawType.RENDER_SHADOW_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_PIE_3D_EFFECT_SHAPE)) {
      addHotspotArea(result, this.pie3DEffectShapes, fragment,
                     ChartShapeDrawType.RENDER_PIE_3D_EFFECT_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_DEVIATION_SHAPE)) {
      addHotspotArea(result, this.deviationShapes, fragment,
                     ChartShapeDrawType.RENDER_DEVIATION_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_DIFFERENCE_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.differenceBackgroundShapes, fragment,
                     ChartShapeDrawType.RENDER_DIFFERENCE_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_RADIUS_GRIDLINE_SHAPE)) {
      addHotspotArea(result, this.radiusGridlines, fragment,
                     ChartShapeDrawType.RENDER_RADIUS_GRIDLINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_ANGLE_GRIDLINE_SHAPE)) {
      addHotspotArea(result, this.angleGridlines, fragment,
                     ChartShapeDrawType.RENDER_ANGLE_GRIDLINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_SHAPE)) {
      addHotspotArea(result, this.minMaxValueShapes, fragment,
                     ChartShapeDrawType.RENDER_MIN_MAX_VALUE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_MIN_MAX_VALUE_LINE_SHAPE)) {
      addHotspotArea(result, this.minMaxValueLineShapes, fragment,
                     ChartShapeDrawType.RENDER_MIN_MAX_VALUE_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_JOIN_LINE_SHAPE)) {
      addHotspotArea(result, this.joinLineShapes, fragment,
                     ChartShapeDrawType.RENDER_JOIN_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_GUIDE_LINE_SHAPE)) {
      addHotspotArea(result, this.guideLineShapes, fragment,
                     ChartShapeDrawType.RENDER_GUIDE_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_DATA_ITEM_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.dataItemBackgroundShapes, fragment,
                     ChartShapeDrawType.RENDER_DATA_ITEM_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE)) {
      addHotspotArea(result, this.dataItemShapes, fragment,
                     ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_TICK_LABEL_SHAPE)) {
      addHotspotArea(result, this.tickLabelShapes, fragment,
                     ChartShapeDrawType.RENDER_TICK_LABEL_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_3D_CLOSE_LINE_SHAPE)) {
      addHotspotArea(result, this.chart3DCloseLines, fragment,
                     ChartShapeDrawType.RENDER_3D_CLOSE_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_FOREGROUND_MARKER_SHAPE)) {
      addHotspotArea(result, this.foregroundMarkerShapes, fragment,
                     ChartShapeDrawType.RENDER_FOREGROUND_MARKER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_FOREGROUND_INTERVAL_MARKER_SHAPE)) {
      addHotspotArea(result, this.foregroundIntervalMarkerShapes, fragment,
                     ChartShapeDrawType.RENDER_FOREGROUND_INTERVAL_MARKER_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_METER_ARC_TICK_LABEL_SHAPE)) {
      addHotspotArea(result, this.meterArcTickLabels, fragment,
                     ChartShapeDrawType.RENDER_METER_ARC_TICK_LABEL_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_METER_ARC_TICK_LINE_SHAPE)) {
      addHotspotArea(result, this.meterArcTickLines, fragment,
                     ChartShapeDrawType.RENDER_METER_ARC_TICK_LINE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_METER_NEEDLE_SHAPE)) {
      addHotspotArea(result, this.meterNeedleShape, fragment,
                     ChartShapeDrawType.RENDER_METER_NEEDLE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_METER_UNIT_SHAPE)) {
      addHotspotArea(result, this.meterUnitShape, fragment,
                     ChartShapeDrawType.RENDER_METER_UNIT_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.RENDER_PIE_LABEL_SHAPE)) {
      addHotspotArea(result, this.pieLabelShapes, fragment,
                     ChartShapeDrawType.RENDER_PIE_LABEL_SHAPE) ;
    }

    return result ;
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.borderShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.quadrantBackgroundShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundImageShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.domainGridLines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueGridLines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueGridBands) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundMarkerShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundIntervalMarkerShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueLineShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.deviationShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.differenceBackgroundShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataItemBackgroundShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.minMaxValueShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.minMaxValueLineShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.joinLineShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataItemShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataBackgroundShapeDrawType) ;
    result = 29 * result + HashcodeUtil.hashCode(this.chart3DCloseLines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.foregroundMarkerShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.foregroundIntervalMarkerShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.meterUnitShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.meterNeedleShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.meterArcTickLabels) ;
    result = 29 * result + HashcodeUtil.hashCode(this.meterArcTickLines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.shadowShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.pieLabelShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.pie3DEffectShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.radiusGridlines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.angleGridlines) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataset) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof RenderShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    RenderShapeCollection that = (RenderShapeCollection) obj ;
    if (!EqualsUtil.equals(this.borderShape, that.borderShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundShape, that.backgroundShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.quadrantBackgroundShapes, that.quadrantBackgroundShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundImageShape, that.backgroundImageShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.domainGridLines, that.domainGridLines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueGridLines, that.valueGridLines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueGridBands, that.valueGridBands)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundMarkerShapes, that.backgroundMarkerShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundIntervalMarkerShapes, that.backgroundIntervalMarkerShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueLineShapes, that.valueLineShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.deviationShapes, that.deviationShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.differenceBackgroundShapes, that.differenceBackgroundShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataItemBackgroundShapes, that.dataItemBackgroundShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.minMaxValueShapes, that.minMaxValueShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.minMaxValueLineShapes, that.minMaxValueLineShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.joinLineShapes, that.joinLineShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataItemShapes, that.dataItemShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataBackgroundShapeDrawType, that.dataBackgroundShapeDrawType)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.chart3DCloseLines, that.chart3DCloseLines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.foregroundMarkerShapes, that.foregroundMarkerShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.foregroundIntervalMarkerShapes, that.foregroundIntervalMarkerShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.meterUnitShape, that.meterUnitShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.meterNeedleShape, that.meterNeedleShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.meterArcTickLabels, that.meterArcTickLabels)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.meterArcTickLines, that.meterArcTickLines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.shadowShapes, that.shadowShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.pieLabelShapes, that.pieLabelShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.pie3DEffectShapes, that.pie3DEffectShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.radiusGridlines, that.radiusGridlines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.angleGridlines, that.angleGridlines)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataset, that.dataset)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[borderShape=" + this.borderShape
      + ",backgroundShape=" + this.backgroundShape
      + ",quadrantBackgroundShapes size=" + this.quadrantBackgroundShapes.size()
      + ",backgroundImageShape=" + this.backgroundImageShape
      + ",domain gridline size=" + this.domainGridLines.size()
      + ",value band size=" + this.valueGridBands.size()
      + ",value gridline size=" + this.valueGridLines.size()
      + ",close3DChart line size=" + this.foregroundMarkerShapes.size()
      + ",foreground marker size=" + this.foregroundMarkerShapes.size()
      + ",foreground interval marker size=" + this.foregroundIntervalMarkerShapes.size()
      + ",valueLineShapes size=" + this.valueLineShapes.size()
      + ",deviationShapes size=" + this.deviationShapes.size()
      + ",differenceBackgroundShapes size=" + this.differenceBackgroundShapes.size()
      + ",dataItemBackgroundShapes size=" + this.dataItemBackgroundShapes.size()
      + ",minMaxValueShapes size=" + this.minMaxValueShapes.size()
      + ",minMaxValueLineShapes size=" + this.minMaxValueLineShapes.size()
      + ",minMaxValueLineShapes size=" + this.minMaxValueLineShapes.size()
      + ",joinLineShapes size=" + this.joinLineShapes.size()
      + ",dataItemShapes size=" + this.dataItemShapes.size()
      + ",dataBackgroundShapeDrawType=" + this.dataBackgroundShapeDrawType
      + ",background marker size=" + this.backgroundMarkerShapes.size()
      + ",background interval marker size=" + this.backgroundIntervalMarkerShapes.size()
      + ",meterUnitShape=" + this.meterUnitShape
      + ",meterNeedleShape=" + this.meterNeedleShape
      + ",meterArcTickLabels size=" + this.meterArcTickLabels.size()
      + ",meterArcTickLines size=" + this.meterArcTickLines.size()
      + ",shadowShapes size=" + this.shadowShapes.size()
      + ",pieLabelShapes size=" + this.pieLabelShapes.size()
      + ",pie3DEffectShapes size=" + this.pie3DEffectShapes.size()
      + ",radiusGridlines size=" + this.radiusGridlines.size()
      + ",angleGridlines size=" + this.angleGridlines.size()
      + "]" ;
  }

}
