package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢ϵͼεļ.</zh>
 * <en>Collection to be used to store the shapes in title area.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TitleShapeCollection extends AbstractShapeCollection implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5979848672019099732L ;

  /**
   * <zh>ͼμ.</zh>
   * <en>The list of title shapes.</en>
   */
  private List titleShapes ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   */
  public TitleShapeCollection() {
    this.titleShapes = new ArrayList() ;
  }

  /**
   * <zh>һͼ.</zh>
   * <en>Add a title shape.</en>
   *
   * @param shape IChartShape <zh>ͼ (Ϊnull)</zh>
   * <en>A title shape (null permitted).</en>
   */
  public void addTitleShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.TITLE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.titleShapes.add(shape) ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>еıͼεļ,˼ϲ޸.</zh>
   * <en>Returns the list of title shapes and is unmodifiable.</en>
   *
   * @return IChartShape <zh>ͼμ(Ϊnull),˼ϲ޸.</zh>
   * <en>The list of title shapes and is unmodifiable.</en>
   */
  public List getTitleShapes() {
    return Collections.unmodifiableList(this.titleShapes) ;
  }

  /**
   * <zh>ɾеıͼ.</zh>
   * <en>Remove all title shapes.</en>
   */
  public void clearTitleShapes() {
    removeShapesListener(this.titleShapes, this) ;
    this.titleShapes.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    super.clip(g2) ;
    draw(g2, this.titleShapes) ;
    super.restoreClip(g2) ;
  }

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    translate(x, y, this.titleShapes) ;
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    clearShapesListener(this.titleShapes) ;
  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.titleShapes) ;
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    IChartShape shape = null ;
    if (flag || drawType.equals(ChartShapeDrawType.TITLE_SHAPE)) {
      shape = findShape(this.titleShapes, point, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    if (drawTypes == null) {
      throw new IllegalArgumentException("Null 'drawTypes' argument.") ;
    }

    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.TITLE_SHAPE)) {
      addHotspotArea(result, this.titleShapes, fragment, ChartShapeDrawType.TITLE_SHAPE) ;
    }

    return result ;
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return HashcodeUtil.hashCode(this.titleShapes) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof RenderShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    TitleShapeCollection that = (TitleShapeCollection) obj ;
    if (!EqualsUtil.equals(this.titleShapes, that.titleShapes)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[title shapes size=" + this.titleShapes.size() + "]" ;
  }

}
