package com.ijchart.xychart.constant ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 * <zh>ڱڶάͼеλ,ǰ֪ͼķ.</zh>
 * <en>Used to indicate the location of an axis on a 2D plot, prior to knowing the
 * orientation of the chart.</en>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class AxisLocation implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 2098541073047896719L ;

  /**
   * <zh>ڶ˻.</zh>
   * <en>Axis at the top or left. </en>
   */
  public static final AxisLocation TOP_OR_LEFT = new AxisLocation("AxisLocation.TOP_OR_LEFT") ;

  /**
   * <zh>ڶ˻ұ</zh>
   * <en>Axis at the top or right.</en>
   */
  public static final AxisLocation TOP_OR_RIGHT = new AxisLocation("AxisLocation.TOP_OR_RIGHT") ;

  /**
   * <zh>ڵ׶˻.</zh>
   * <en>Axis at the bottom or left.</en>
   */
  public static final AxisLocation BOTTOM_OR_LEFT = new AxisLocation("AxisLocation.BOTTOM_OR_LEFT") ;

  /**
   * <zh>ڵ׶˻ұ</zh>
   * <en>Axis at the bottom or right.</en>
   */
  public static final AxisLocation BOTTOM_OR_RIGHT = new AxisLocation(
    "AxisLocation.BOTTOM_OR_RIGHT") ;

  /**
   * <zh>.</zh>
   * <en>The name.</en>
   */
  private String name ;

  /**
   * <zh>˽йֹʵ.</zh>
   * <en>Private constructor to prevent instance.</en>
   *
   * @param name <zh>.</zh>
   * <en>The name.</en>
   */
  private AxisLocation(String name) {
    this.name = name ;
  }

  /**
   * <zh>Եλ.</zh>
   * <en>Returns the location that is opposite to this location.</en>
   *
   * @return The opposite location.
   */
  public AxisLocation getOpposite() {
    return getOpposite(this) ;
  }

  /**
   * <zh>Եλ</zh>
   * <en>Returns the location that is opposite to the supplied location.</en>
   *
   * @param location <zh>AxisLocation (Ϊnull).</zh>
   * <en>The location (null not permitted).</en>
   *
   * @return <zh>Եλ.</zh>
   * <en>The opposite location.</en>
   */
  public static AxisLocation getOpposite(AxisLocation location) {
    if (location == null) {
      throw new IllegalArgumentException("Null 'location' argument.") ;
    }

    AxisLocation result = null ;
    if (location.equals(AxisLocation.TOP_OR_LEFT)) {
      result = AxisLocation.BOTTOM_OR_RIGHT ;
    } else if (location.equals(AxisLocation.TOP_OR_RIGHT)) {
      result = AxisLocation.BOTTOM_OR_LEFT ;
    } else if (location.equals(AxisLocation.BOTTOM_OR_LEFT)) {
      result = AxisLocation.TOP_OR_RIGHT ;
    } else if (location.equals(AxisLocation.BOTTOM_OR_RIGHT)) {
      result = AxisLocation.TOP_OR_LEFT ;
    } else {
      throw new IllegalStateException("AxisLocation not recognised.") ;
    }

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Returns true if this object is equal to the specified
   * object, and false otherwise.</en>
   *
   * @param obj <zh>ҪȽϵĶ.</zh>
   * <en>The other object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AxisLocation)) {
      return false ;
    }

    AxisLocation that = (AxisLocation) obj ;
    if (!this.name.equals(that.name)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>has hcodeֵ.</zh>
   * <en>Returns a hash code value for the object.  </en>
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    return this.name.hashCode() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing the object.</en>
   *
   * @return The string.
   */
  public String toString() {
    return this.name ;
  }
}
