package com.ijchart.xychart.data ;

import java.io.Serializable ;
import java.util.Date ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ʊͼݶ(,̼,ֵ,ֵ,̼,),ұ󲻿ⲿ޸.</zh>
 * <en>An chart data in the form (date,open,high,low,close,volume),and this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class OHLCChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 904308870087722360L ;

  /**
   * <zh>.</zh>
   * <en>The date.</en>
   */
  private Date date ;

  /**
   * <zh>̼.</zh>
   * <en>The open value.</en>
   */
  private double open ;

  /**
   * <zh>̼.</zh>
   * <en>The close value.</en>
   */
  private double close ;

  /**
   * <zh>ֵ.</zh>
   * <en>The high value.</en>
   */
  private double high ;

  /**
   * <zh>ֵ.</zh>
   * <en>The low value.</en>
   */
  private double low ;

  /**
   * <zh>.</zh>
   * <en>The volume value.</en>
   */
  private double volume ;

  /**
   * <zh></zh>
   * <en>constructor</en>
   *
   * @param date Date <zh> (Ϊnull).</zh>
   * <en>The date (null not permitted).</en>
   * @param open double <zh>̼.</zh>
   * <en>the open value.</en>
   * @param close double <zh>ֵ.</zh>
   * <en>the close value.</en>
   * @param high double <zh>ֵ.</zh>
   * <en>the high value.</en>
   * @param low double <zh>̼.</zh>
   * <en>the low value.</en>
   * @param volume double <zh>ɽ.</zh>
   * <en>The volume.</en>
   */
  public OHLCChartData(Date date, double open, double high, double low, double close, double volume) {
    if (date == null) {
      throw new IllegalArgumentException("Null 'date' argument.") ;
    }

    this.date = date ;
    this.open = open ;
    this.close = close ;
    this.high = high ;
    this.low = low ;
    this.volume = volume ;
  }

  /**
   * <zh>.</zh>
   * <en>Returns the date.</en>
   *
   * @return Date <zh> (Ϊnull).</zh>
   * <en>The date (never null).</en>
   */
  public Date getDate() {
    return this.date ;
  }

  /**
   * <zh>ؿ̼.</zh>
   * <en>Returns the open value.</en>
   *
   * @return <zh>̼.</zh>
   * <en>The open value.</en>
   */
  public double getOpen() {
    return this.open ;
  }

  /**
   * <zh>̼.</zh>
   * <en>Returns the close value.</en>
   *
   * @return <zh>̼.</zh>
   * <en>The close value.</en>
   */
  public double getClose() {
    return this.close ;
  }

  /**
   * <zh>ظֵ.</zh>
   * <en>Returns the high value.</en>
   *
   * @return <zh>ֵ.</zh>
   * <en>The high value.</en>
   */
  public double getHigh() {
    return this.high ;
  }

  /**
   * <zh>صֵ.</zh>
   * <en>Returns the low value.</en>
   *
   * @return <zh>ֵ.</zh>
   * <en>The low value.</en>
   */
  public double getLow() {
    return this.low ;
  }

  /**
   * <zh>ؽ.</zh>
   * <en>Returns the volume.</en>
   *
   * @return double <zh>.</zh>
   * <en>The volume.</en>
   */
  public double getVolume() {
    return this.volume ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return "(" + formatDate(this.date) + "," + formatNumber(this.open) + ","
      + formatNumber(this.high) + "," + formatNumber(this.low) + ","
      + formatNumber(this.close) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatDateForFunction(this.date) + "," + formatNumberForFunction(this.open) + ","
      + formatNumberForFunction(this.high) + "," + formatNumberForFunction(this.low) + ","
      + formatNumberForFunction(this.close) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "date=" + formatDate(this.date) + "&open=" + formatNumber(this.open) + "&high="
      + formatNumber(this.high) + "&low=" + formatNumber(this.low) + "&close="
      + formatNumber(this.close) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof OHLCChartData)) {
        throw new ClassCastException("Must be 'OHLCChartData'.") ;
      }

      OHLCChartData other = (OHLCChartData) o ;
      double diff = this.open - other.open ;

      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>ʵָеequals.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ(Ϊnull).</zh>
   * <en>the object (null permitted).</en>
   * @return A boolean
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof OHLCChartData)) {
      return false ;
    }

    OHLCChartData that = (OHLCChartData) obj ;
    if (EqualsUtil.equals(this.date, that.date)) {
      return false ;
    }
    if (this.open != that.open) {
      return false ;
    }
    if (this.close != that.close) {
      return false ;
    }
    if (this.high != that.high) {
      return false ;
    }
    if (this.low != that.low) {
      return false ;
    }
    if (this.volume != that.volume) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.date) ;
    result = 37 * result + (int) (100 * this.open) ;
    result = 37 * result + (int) (100 * this.close) ;
    result = 37 * result + (int) (100 * this.high) ;
    result = 37 * result + (int) (100 * this.low) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[date=" + this.date
      + ",open=" + this.open
      + ",high" + this.high
      + ",low=" + this.low
      + ",close" + this.close
      + ",volume" + this.volume
      + "]" ;
  }
}
