package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>(start,end)ֵͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>The (start,end) Number ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StartEndNumberChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1097817016865389461L ;

  /**
   * <zh>startֵ.</zh>
   * <en>The start value.</en>
   */
  private Number start ;

  /**
   * <zh>endֵ.</zh>
   * <en>The end value.</en>
   */
  private Number end ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param start double <zh>startֵ.</zh>
   * <en>The start value.</en>
   * @param end double <zh>endֵ.</zh>
   * <en>The end value.</en>
   */
  public StartEndNumberChartData(double start, double end) {
    this(new Double(start), new Double(end)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param start Number <zh>startֵ (Ϊnull).</zh>
   * <en>The start value (null not permitted).</en>
   * @param end Number <zh>endֵ (Ϊnull).</zh>
   * <en>The end value (null not permitted).</en>
   */
  public StartEndNumberChartData(Number start, Number end) {
    if (start == null) {
      throw new IllegalArgumentException("Null 'start' argument.") ;
    }
    if (end == null) {
      throw new IllegalArgumentException("Null 'end' argument.") ;
    }

    this.start = start ;
    this.end = end ;
  }

  /**
   * <zh>startֶ.</zh>
   * <en>Returns the start Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getStartData() {
    return this.start ;
  }

  /**
   * <zh>endֶ.</zh>
   * <en>Returns the end Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getEndData() {
    return this.end ;
  }

  /**
   * <zh>õdouble͵startֵ.</zh>
   * <en>Returns the double start value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getStartValue() {
    return this.start.doubleValue() ;
  }

  /**
   * <zh>õdouble͵endֵ.</zh>
   * <en>Returns the double end value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getEndValue() {
    return this.end.doubleValue() ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof StartEndNumberChartData)) {
        throw new ClassCastException("Must be 'StartEndNumberChartData'.") ;
      }

      StartEndNumberChartData other = (StartEndNumberChartData) o ;
      double diff = this.start.doubleValue() - other.getStartValue() ;
      return (diff < 0) ? -1 : ((diff == 0) ? 0 : 1) ;
    }
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (never null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.start) + "," + formatNumber(this.end) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.start) + "," + formatNumberForFunction(this.end) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "start=" + formatNumber(this.start) + "&end=" + formatNumber(this.end) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof StartEndNumberChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StartEndNumberChartData that = (StartEndNumberChartData) obj ;
    if (EqualsUtil.equals(this.start, that.start)) {
      return false ;
    }
    if (EqualsUtil.equals(this.end, that.end)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.start) ;
    result = 37 * result + HashcodeUtil.hashCode(this.end) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[start=" + this.start.doubleValue() + ",end=" + this.end.doubleValue() + "]" ;
  }

}
