package com.ijchart.xychart.dataset ;

import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.Arrays ;

import javax.swing.event.EventListenerList ;

import com.ijchart.xychart.event.DatasetChangeEvent ;
import com.ijchart.xychart.listener.IDatasetChangeListener ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ݼĳʵ,ݼԼ̳д.</zh>
 * <en>The abstract implemention of dataset,concrete dataset class can extend this class.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractDataset implements IDataset, IDatasetChangeListener, Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 2171962141063181247L ;

  /**
   * <zh>,ɱл.</zh>
   * <en>The listener collection,not serializable.</en>
   */
  private transient EventListenerList listenerList ;

  /**
   * <zh>ûвĹ.</zh>
   * <en>No argument constructor.</en>
   */
  protected AbstractDataset() {
    this.listenerList = new EventListenerList() ;
  }

  /**
   * <zh>עݼ仯ļ߶.</zh>
   * <en>Registers an object for notification of changes to the dataset.</en>
   *
   * @param listener <zh>עļ.</zh>
   * <en>the object to register.</en>
   */
  public void addChangeListener(IDatasetChangeListener listener) {
    listenerList.add(IDatasetChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾעݼ仯߶</zh>
   * <en>Deregisters an object for notification of changes to the dataset.</en>
   *
   * @param listener <zh>Ҫɾļ߶</zh>
   * <en>the object to deregister.</en>
   */
  public void removeChangeListener(IDatasetChangeListener listener) {
    listenerList.remove(IDatasetChangeListener.class, listener) ;
  }

  /**
   * <zh>еݼ仯߶</zh>
   * <en>Deregisters all object for notification of changes to the dataset.</en>
   */
  public void removeAllChangeListener() {
    Object[] listeners = this.listenerList.getListenerList() ;

    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      this.listenerList.remove(IDatasetChangeListener.class,
                               (IDatasetChangeListener) listeners[i + 1]) ;
    }
  }

  /**
   * <zh>ָlistenerǷ,ҪڵԪ.</zh>
   * <en>Returns true if the specified object is registered with
   * the dataset as a listener.  Most applications won't need to call this
   * method, it exists mainly for use by unit testing code.</en>
   *
   * @param listener <zh>ҪԵļ.</zh>
   * <en>the listener.</en>
   * @return A boolean.
   */
  public boolean hasListener(IDatasetChangeListener listener) {
    return Arrays.asList(this.listenerList.getListenerList()).contains(listener) ;
  }

  /**
   * <zh>ݼͨݼķıֵ,ͨ˷֪ͨݼı.</zh>
   * <en>If the data object of dataset changed by method out of dataset,set dataset changed by
   * this method.</en>
   */
  public void forceDataSetChanged() {
    notifyListeners(new DatasetChangeEvent(this, this)) ;
  }

  /**
   * <zh>ݼͨݼķıֵ,ͨ˷֪ͨݼı.</zh>
   * <en>If the data object of dataset changed by method out of dataset,set dataset changed by
   * this method.</en>
   *
   * @param event <zh>ݼ仯Ķ</zh>
   * <en>information about the event.</en>
   */
  public void datasetChanged(DatasetChangeEvent event) {
    notifyListeners(event) ;
  }

  /**
   * <zh>ͨмݼ˱仯.</zh>
   * <en>Notifies all registered listeners that the dataset has changed.</en>
   */
  protected void fireDatasetChanged() {
    notifyListeners(new DatasetChangeEvent(this, this)) ;
  }

  /**
   * <zh>֪ͨעļݼ˱仯</zh>
   * <en>Notifies all registered listeners that the dataset has changed.</en>
   *
   * @param event <zh>¼.</zh>
   * <en>contains information about the event that triggered the notification.</en>
   */
  protected void notifyListeners(DatasetChangeEvent event) {
    Object[] listeners = this.listenerList.getListenerList() ;
    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      if (listeners[i] == IDatasetChangeListener.class) {
        ((IDatasetChangeListener) listeners[i + 1]).datasetChanged(event) ;
      }
    }

  }

  /**
   * <zh>Clone,˴ʵֲ¡IDatasetChangeListener,ϱ.</zh>
   * <en>Returns a clone of the dataset. The cloned dataset will NOT include the
   * {@link IDatasetChangeListener} references that have been registered with this dataset.</en>
   *
   * @return <zh>¡Ķ.</zh>
   * <en>A clone. </en>
   * @throws CloneNotSupportedException <zh>ֿ֧¡׳쳣.</zh>
   * <en>if the dataset does not support cloning.</en>
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractDataset clone = (AbstractDataset)super.clone() ;
    clone.listenerList = new EventListenerList() ;

    return clone ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractDataset)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hascodeֵ.</zh>
   * <en>Get hascode.</en>
   * @return int
   */
  public int hashCode() {
    int result = super.hashCode() ;

    return result ;
  }

  /**
   * Handles serialization.
   *
   * @param stream  the output stream.
   *
   * @throws IOException if there is an I/O problem.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
  }

  /**
   * Restores a serialized object.
   *
   * @param stream  the input stream.
   *
   * @throws IOException if there is an I/O problem.
   * @throws ClassNotFoundException if there is a problem loading a class.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.listenerList = new EventListenerList() ;
  }

}
