package com.ijchart.xychart.event ;

import java.io.Serializable ;
import java.util.EventObject ;

import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾͼеͼζı¼.</zh>
 * <en>A change event that encapsulates information about a change to a
 * {@link com.ijchart.xychart.shape.IChartShape}</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ChartShapeChangeEvent extends EventObject implements Serializable {
  /**
   * <zh>仯ͼζ.</zh>
   * <en>The IChartShape that generated the change event.</en>
   */
  private IChartShape chartShape ;

  /**
   * <zh>һµͼζ仯¼.</zh>
   * <en>Constructs a new ChartShape changed event.</en>
   *
   * @param source Object <zh>¼Դ</zh>
   * <en>The source of the event.</en>
   * @param chartShape IChartShape <zh>¼ͼζ (Ϊnull).</zh>
   * <en>The IChartShape that generated the event (null permitted).</en>
   */
  public ChartShapeChangeEvent(Object source, IChartShape chartShape) {
    super(source) ;
    this.chartShape = chartShape ;
  }

  /**
   * <zh>ز¼ͼζ,Ϊnull.</zh>
   * <en>Returns the IChartShape that generated the event.
   * Note that the IChartShape maybe null.</en>
   *
   * @return <zh>¼ͼζ,Ϊnull.</zh>
   * <en>The IChartShape (possibly null).</en>
   */
  public IChartShape getChartShape() {
    return this.chartShape ;
  }

  /**
   * Returns a hash code value for the object.
   * @return int
   */
  public int hashCode() {
    int result = source.hashCode() ;
    result = 29 * result + ((chartShape == null) ? 0 : chartShape.hashCode()) ;

    return result ;
  }

  /**
   * Indicates whether some other object is "equal to" this one.
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof ChartShapeChangeEvent)) {
      return false ;
    }

    ChartShapeChangeEvent that = (ChartShapeChangeEvent) obj ;

    return EqualsUtil.equals(this.source, that.source)
      && EqualsUtil.equals(this.chartShape, that.chartShape) ;
  }

}
