package com.ijchart.xychart.format ;

import java.io.Serializable ;
import java.text.DecimalFormat ;
import java.text.NumberFormat ;
import java.util.List ;

import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.group.NumberGroupChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>({@link NumberGroupChartData}ĸʽ.</zh>
 * <en>A formatter for class for number group chart data ({@link NumberGroupChartData}</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NumberGroupChartDataFormatter extends AbstractChartDataFormatterAdapter implements
  IChartDataFormatter, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -7192012279400974875L ;

  /**
   * <zh>ֻʽ.</zh>
   * <en>The number format object.</en>
   */
  private DecimalFormat format ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param mask String <zh> (Ϊnull).</zh>
   * <en>The number format mask (null not permitted).</en>
   */
  public NumberGroupChartDataFormatter(String mask) {
    if (mask == null) {
      throw new IllegalArgumentException("Null 'mask' argument.") ;
    }

    format = (DecimalFormat) NumberFormat.getInstance() ;
    format.applyPattern(mask) ;
  }

  /**
   * <zh>ֻʽ.</zh>
   * <en>Returns the number format object.</en>
   *
   * @return DecimalFormat <zh>ֻʽ.</zh>
   * <en>The number format object.</en>
   */
  public DecimalFormat getFormat() {
    return this.format ;
  }

  /**
   * <zh>ʽݼĳ{@link IChartData}Ϊַ.</zh>
   * <en>Formatted an {@link IChartData} object to a string.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   * @param index int <zh>еһݵɶ,ֵʾ.</zh>
   * <en>The index of a dataset data.</en>
   *
   * @return String <zh>ַ (Ϊnull).</zh>
   * <en>A string (maybe null).</en>
   */
  public String format(IMatrixDataset dataset, int row, int column, int index) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be 'row'>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be 'column'>=0.") ;
    }

    IChartData data = dataset.getData(row, column) ;
    if (data != null && (data instanceof NumberGroupChartData)) {
      List datas = ((NumberGroupChartData) data).getDatas() ;
      if (datas != null) {
        if (index < datas.size()) {
          Number number = (Number) datas.get(index) ;
          return this.format.format(number.doubleValue()) ;
        }
      }

    }

    return "" ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof NumberChartDataFormatter)) {
      return false ;
    }

    NumberGroupChartDataFormatter that = (NumberGroupChartDataFormatter) obj ;
    if (!EqualsUtil.equals(this.format, that.format)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the <code>Cloneable</code> interface. Subclasses
   *               that override the <code>clone</code> method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    return HashcodeUtil.hashCode(this.format) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[format=" + this.format + "]" ;
  }

}
