package com.ijchart.xychart.marker ;

import java.awt.Paint ;
import java.awt.Stroke ;
import java.io.Serializable ;

import com.ijchart.xychart.event.MarkerChangeEvent ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֵ(value)marker.ڴ˵,ֵˮƽ뷽ʽΪ,ʾߵ,Ϊ
 * ߵұ;ֵĴֱΪϻбʾߵϱ,Ϊߵ±.</zh>
 * <en>A marker that represents a single value.  Markers can be added to plots to highlight specific
 * values. Such, if the text horizontal alignment is left or center, said the text on-line on the
 * left, otherwise to the right side of the text on-line; if the text vertical alignment is top or
 * center, said the text above, otherwise below the line for the text.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ValueMarker extends AbstractMarker implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2163063246091101583L ;

  /**
   * <zh>ֵ.</zh>
   * <en>The value.</en>
   */
  private double value ;

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new marker.</en>
   *
   * @param value double <zh>ֵ.</zh>
   * <en>The value.</en>
   */
  public ValueMarker(double value) {
    this.value = value ;
  }

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new value marker.</en>
   *
   * @param value double <zh>ֵ.</zh>
   * <en>The value.</en>
   * @param fillPaint <zh>ʹõPaint (Ϊnull).</zh>
   * <en>The fill paint (null not permitted).</en>
   */
  public ValueMarker(double value, Paint fillPaint) {
    super(fillPaint) ;
    this.value = value ;
  }

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new value marker.</en>
   *
   * @param value double <zh>ֵ,</zh>
   * <en>The value.</en>
   * @param fillPaint <zh>ʹõpaint (Ϊnull).</zh>
   * <en>The fill paint (null not permitted).</en>
   * @param borderPaint <zh>ʹõPaint (Ϊnull).</zh>
   * <en>The border paint (null permitted).</en>
   * @param borderStroke <zh>ʹõStroke (Ϊnull).</zh>
   * <en>The border stroke (null permitted).</en>
   * @param fillAlpha <zh>䱳alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (must be in the range 0.0f to 1.0f).</en>
   */
  public ValueMarker(double value, Paint fillPaint, Paint borderPaint,
                     Stroke borderStroke, float fillAlpha) {
    super(fillPaint, borderPaint, borderStroke, fillAlpha) ;
    this.value = value ;
  }

  /**
   * <zh>ֵ.</zh>
   * <en>Returns the value.</en>
   *
   * @return double <zh>ֵ.</zh>
   * <en>The value.</en>
   */
  public double getValue() {
    return this.value ;
  }

  /**
   * <zh>ֵ,м{@link MarkerChangeEvent}¼.</zh>
   * <en>Sets the value and sends a {@link MarkerChangeEvent} to all registered listeners.</en>
   *
   * @param value <zh>ֵ.</zh>
   * <en>The value.</en>
   */
  public void setValue(double value) {
    if (this.value != value) {
      this.value = value ;
      notifyListeners(new MarkerChangeEvent(this)) ;
    }
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the double.</en>
   *
   * @return int The hashcode value.
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + (int)this.value ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the marker for equality with an arbitrary double.</en>
   *
   * @param obj  the double (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == null) {
      return false ;
    }
    if (!(obj instanceof ValueMarker)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ValueMarker that = (ValueMarker) obj ;
    if (this.value != that.value) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>Ʊ.</zh>
   * <en>Creates a clone of the marker.</en>
   *
   * @return <zh>ƺĶ</zh>
   * <en>A clone.</en>
   *
   * @throws CloneNotSupportedException never.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.  </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "ValueMarker [value=" + this.value + "]" ;
  }

}
