package com.ijchart.xychart.render ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.GradientPaint ;
import java.awt.Image ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Arrays ;
import java.util.Hashtable ;
import java.util.List ;

import javax.swing.event.EventListenerList ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.TextDirection ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.DatasetChangeEvent ;
import com.ijchart.xychart.event.RenderChangeEvent ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.label.ILabelGenerator ;
import com.ijchart.xychart.listener.IDatasetChangeListener ;
import com.ijchart.xychart.listener.IRenderChangeListener ;
import com.ijchart.xychart.render.util.Shape3DSortUtil ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.shape.AbstractRectangleTextChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LegendItemChartShape ;
import com.ijchart.xychart.shape.LineAndShapeChartShape ;
import com.ijchart.xychart.shape.PaintScaleLegendChartShape ;
import com.ijchart.xychart.shape.RectangleImageChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.tooltip.IToolTipGenerator ;
import com.ijchart.xychart.ui.Align ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.DefaultDrawingSupplier ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩrenderĳʵ.</zh>
 * <en>Base class providing common services for renders.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractRender implements IDatasetChangeListener, IRender, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 2470745792704798948L ;

  /**
   * <zh>ȱʡĻƱ߿paint.</zh>
   * <en>The default border paint.</en>
   */
  public static final Paint DEFAULT_BASE_BORDER_PAINT = Color.lightGray ;

  /**
   * <zh>ȱʡĻƱ߿stroke.</zh>
   * <en>The default border stroke.</en>
   */
  public static final Stroke DEFAULT_BASE_BORDER_STROKE = new BasicStroke(1.0f) ;

  /**
   * <zh>ȱʡıǩǷʾ.</zh>
   * <en>The default value of base item labels visible.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_LABEL_VISIBLE = false ;

  /**
   * <zh>ȱʡıǩ.</zh>
   * <en>The default font of base item labels.</en>
   */
  public static final Font DEFAULT_BASE_ITEM_LABEL_FONT = new Font("SansSerif", Font.PLAIN, 10) ;

  /**
   * <zh>ȱʡıǩpaint.</zh>
   * <en>The default item label paint.</en>
   */
  public static final Paint DEFAULT_BASE_ITEM_LABEL_PAINT = Color.BLACK ;

  /**
   * <zh>ȱʡıǩˮƽ뷽ʽ.</zh>
   * <en>The default item label horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_BASE_ITEM_LABEL_HORIZONTAL_ALIGNMENT =
    HorizontalAlignment.CENTER ;

  /**
   * <zh>ȱʡıǩĴֱ뷽ʽ.</zh>
   * <en>The default item label vertical alignment.</en>
   */
  public static final VerticalAlignment DEFAULT_BASE_ITEM_LABEL_VERTICAL_ALIGNMENT =
    VerticalAlignment.CENTER ;

  /**
   * <zh>ȱʡɫ.</zh>
   * <en>The default background color. </en>
   */
  public static final Paint DEFAULT_BACKGROUND_PAINT = Color.lightGray ;

  /**
   * <zh>ȱʡıͼƬ뷽ʽ.</zh>
   * <en>The default background image alignment.</en>
   */
  public static final Align DEFAULT_BACKGROUND_IMAGE_ALIGNMENT = Align.FIT ;

  /**
   * <zh>ȱʡǷ߿.</zh>
   * <en>A default flag that controls whether or not the render border is drawn.</en>
   */
  public static final boolean DEFAULT_RENDER_AREA_BORDER_VISIBLE = false ;

  /**
   * <zh>ȱʡ߿Stroke.</zh>
   * <en>The default border stroke.</en>
   */
  public static final Stroke DEFAULT_BORDER_STROKE = new BasicStroke(0.5f) ;

  /**
   * <zh>ȱʡ߿Paint.</zh>
   * <en>The default border color.</en>
   */
  public static final Paint DEFAULT_BORDER_PAINT = Color.gray ;

  /**
   * <zh>ȱʡõPaint.</zh>
   * <en>The default fill paint.</en>
   */
  public static final Paint DEFAULT_BASE_ITEM_FILL_PAINT = Color.gray ;

  /**
   * <zh>ȱʡǷƱ߽.</zh>
   * <en>The default value of base item border visible.</en>
   */
  public static final boolean DEFAULT_DRAW_BORDER = false ;

  /**
   * <zh>ȱʡǷԶѡͼεPaint.</zh>
   * <en>A default flag that controls whether or not the fill paint is auto selected.</en>
   */
  public static final boolean DEFAULT_AUTO_SELECT_FILL_PAINT = true ;

  /**
   * <zh>ȱʡǷԶѡͼα߿Paint.</zh>
   * <en>A default flag that controls whether or not the border paint is auto selected</en>
   */
  public static final boolean DEFAULT_AUTO_SELECT_BORDER_PAINT = true ;

  /**
   * <zh>ȱʡǷԶѡͼα߿Stroke.</zh>
   * <en>A default flag that controls whether or not the border stroke is auto selected</en>
   */
  public static final boolean DEFAULT_AUTO_SELECT_BORDER_STROKE = false ;

  /**
   * <zh>ȱʡݹõǷͼڲ.</zh>
   * <en>A default flag that controls whether or not fill all data items shape of a dataset.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_FILL = true ;

  /**
   * <zh>ṩͼҪһϵж.</zh>
   * <en>The drawing supplier for this render.</en>
   */
  public static final IDrawingSupplier DEFAULT_DRAWINT_SUPPLIER = new DefaultDrawingSupplier() ;

  /**
   * <zh>ȱʡıǩ.</zh>
   * <en>The default label generator.</en>
   */
  public static final ILabelGenerator DEFAULT_LABEL_GENERATOR = new DefaultLabelGenerator() ;

  /**
   * <zh>ȱʡͼʾ.</zh>
   * <en>The default tooltip generator.</en>
   */
  public static final IToolTipGenerator DEFAULT_TOOLTIP_GENERATOR = new DefaultTooltipGenerator() ;

  /**
   * <zh>Ƿͼεurlȱʡֵ.</zh>
   * <en>A default flag that controls whether or not generator url of shape.</en>
   */
  public static final boolean DEFAULT_GENERATOR_URL = false ;

  /**
   * <zh>Ƿͼεtooltipȱʡֵ.</zh>
   * <en>A default flag that controls whether or not generator tooltip of shape.</en>
   */
  public static final boolean DEFAULT_GENERATOR_TOOLTIP = false ;

  /**
   * <zh>ȱʡͼڲķʽ.</zh>
   * <en>The default type of gradient paint transform.</en>
   */
  public static final GradientPaintTransformType DEFAULT_BASE_ITEM_FILL_PAINT_TRANSFORMER =
    GradientPaintTransformType.CENTER_HORIZONTAL ;

  /**
   * <zh>ڿݼеݼǷʾ.</zh>
   * <en>Storage for item visible of a dataset.</en>
   */
  private Hashtable itemVisible ;

  /**
   * <zh>ڿݼеһǷʾ.</zh>
   * <en>Storage for item visible of a dataset column.</en>
   */
  private Hashtable itemColumnVisible ;

  /**
   * <zh>ڿݼеһǷʾ.</zh>
   * <en>Storage for item visible of a dataset row.</en>
   */
  private Hashtable itemRowVisible ;

  /**
   * <zh>ݼÿݵpaint.</zh>
   * <en>Storage the paint used to fill data item.</en>
   */
  private transient Hashtable itemFillPaint ;

  /**
   * <zh>ݼеһݵpaint.</zh>
   * <en>Storage the paint used to fill data items of a dataset column.</en>
   */
  private transient Hashtable itemColumnFillPaint ;

  /**
   * <zh>ݼеһݵpaint.</zh>
   * <en>Storage the paint used to fill data items of a dataset row.</en>
   */
  private transient Hashtable itemRowFillPaint ;

  /**
   * <zh>ݹõpaint.</zh>
   * <en>The base paint used to fill all data items of a dataset.</en>
   */
  private transient Paint baseItemFillPaint ;

  /**
   * <zh>ڿǷԶѡͼεPaint.</zh>
   * <en>A flag that controls whether or not the fill paint is auto selected.</en>
   */
  private boolean autoSelectFillPaint ;

  /**
   * <zh>ݼÿݵǷͼڲ.</zh>
   * <en>Storage whether or not to fill shape of a data item.</en>
   */
  private Hashtable itemFill ;

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Storage whether or not to fill data items shape of a dataset column.</en>
   */
  private Hashtable itemColumnFill ;

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Storage whether or not to fill data items shape of a dataset row.</en>
   */
  private Hashtable itemRowFill ;

  /**
   * <zh>ݹõǷͼڲ.</zh>
   * <en>A flag that controls whether or not fill all data items shape of a dataset.</en>
   */
  private boolean baseItemFill ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ͼڲķʽ.</zh>
   * <en>Storage the optional class used to transform gradient paint object to fit item.</en>
   */
  private Hashtable itemFillPaintTransformer ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһݵͼڲķʽ.</zh>
   * <en>Storage the optional class used to transform gradient paint object to fit items of a
   * dataset column.</en>
   */
  private Hashtable itemColumnFillPaintTransformer ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһݵͼڲķʽ.</zh>
   * <en>Storage the optional class used to transform gradient paint object to fit items of a
   * dataset row.</en>
   */
  private Hashtable itemRowFillPaintTransformer ;

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼͼڲķʽ.</zh>
   * <en>Storage the optional class used to transform gradient paint object to fit items of a
   * dataset.</en>
   */
  private GradientPaintTransformType baseItemFillPaintTransformer ;

  /**
   * <zh>ݼеÿݵǷƱ߽.</zh>
   * <en>Storage for border item of a dataset </en>
   */
  private Hashtable itemDrawBorder ;

  /**
   * <zh>ݼеһǷƱ߽.</zh>
   * <en>Storage for border item of a dataset column.</en>
   */
  private Hashtable itemColumnDrawBorder ;

  /**
   * <zh>ݼеһǷƱ߽.</zh>
   * <en>Storage for border item of a dataset row.</en>
   */
  private Hashtable itemRowDrawBorder ;

  /**
   * <zh>ǷƱ߽.</zh>
   * <en>Whether or not draw border.</en>
   */
  private boolean baseItemDrawBorder ;

  /**
   * <zh>ݼÿݵı߿paint.</zh>
   * <en>Storage the paint used to outline data item.</en>
   */
  private transient Hashtable itemBorderPaint ;

  /**
   * <zh>ݼеһݵı߿paint.</zh>
   * <en>Storage the paint used to outline data items of a dataset column.</en>
   */
  private transient Hashtable itemColumnBorderPaint ;

  /**
   * <zh>ݼеһݵı߿paint.</zh>
   * <en>Storage the paint used to outline data items of a dataset row.</en>
   */
  private transient Hashtable itemRowBorderPaint ;

  /**
   * <zh>ݹõı߿paint.</zh>
   * <en>The base paint used to outline all data items of a dataset.</en>
   */
  private transient Paint baseItemBorderPaint ;

  /**
   * <zh>ڿǷԶѡͼα߿Paint.</zh>
   * <en>A flag that controls whether or not the border paint is auto selected</en>
   */
  private boolean autoSelectBorderPaint ;

  /**
   * <zh>ݼÿݵı߿stroke.</zh>
   * <en>Storage the stroke used to outline data item.</en>
   */
  private transient Hashtable itemBorderStroke ;

  /**
   * <zh>ݼеһݵı߿stroke.</zh>
   * <en>Storage the stroke used to outline data items of a dataset column.</en>
   */
  private transient Hashtable itemColumnBorderStroke ;

  /**
   * <zh>ݼеһݵı߿stroke.</zh>
   * <en>Storage the stroke used to outline data items of a dataset row.</en>
   */
  private transient Hashtable itemRowBorderStroke ;

  /**
   * <zh>ݹõı߿stroke.</zh>
   * <en>The base stroke used to outline all data items of a dataset.</en>
   */
  private transient Stroke baseItemBorderStroke ;

  /**
   * <zh>ڿǷԶѡͼα߿Stroke.</zh>
   * <en>A flag that controls whether or not the border stroke is auto selected</en>
   */
  private boolean autoSelectBorderStroke ;

  /**
   * <zh>ڿÿıǩǷʾ.</zh>
   * <en>Storage for item label visible of a dataset.</en>
   */
  private Hashtable itemLabelVisible ;

  /**
   * <zh>ڿݼеһǷʾ.</zh>
   * <en>Storage for item label visible of a dataset column.</en>
   */
  private Hashtable itemColumnLabelVisible ;

  /**
   * <zh>ڿݼеһǷʾ.</zh>
   * <en>Storage for item label visible of a dataset row.</en>
   */
  private Hashtable itemRowLabelVisible ;

  /**
   * <zh>ڿıǩǷʾ.</zh>
   * <en>The base item labels visible.</en>
   */
  private boolean baseItemLabelVisible ;

  /**
   * <zh>ڿÿıǩ.</zh>
   * <en>Storage for item label font of a dataset.</en>
   */
  private Hashtable itemLabelFont ;

  /**
   * <zh>ڿݼеһݵıǩ.</zh>
   * <en>Storage for item label font of a dataset column.</en>
   */
  private Hashtable itemColumnLabelFont ;

  /**
   * <zh>ڿݼеһݵıǩ.</zh>
   * <en>Storage for item label font of a dataset row.</en>
   */
  private Hashtable itemRowLabelFont ;

  /**
   * <zh>ıǩ.</zh>
   * <en>The base item label font.</en>
   */
  private Font baseItemLabelFont ;

  /**
   * <zh>ݼÿݵıǩpaint.</zh>
   * <en>Storage the label paint of a data item.</en>
   */
  private transient Hashtable itemLabelPaint ;

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Storage the label paint of a dataset column.</en>
   */
  private transient Hashtable itemColumnLabelPaint ;

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Storage the label paint of a dataset row.</en>
   */
  private transient Hashtable itemRowLabelPaint ;

  /**
   * <zh>ݹõıǩpaint.</zh>
   * <en>The base label paint.</en>
   */
  private transient Paint baseItemLabelPaint ;

  /**
   * <zh>ݼÿݵıǩλ.</zh>
   * <en>Storage the label position of a data item.</en>
   */
  private Hashtable itemLabelPosition ;

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Storage the label position of a dataset column.</en>
   */
  private Hashtable itemColumnLabelPosition ;

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Storage the label position of a dataset row.</en>
   */
  private Hashtable itemRowLabelPosition ;

  /**
   * <zh>ݹõıǩλ.</zh>
   * <en>The base label position.</en>
   */
  private TextPosition baseItemLabelPosition ;

  /**
   * <zh>ǩˮƽ뷽ʽ.</zh>
   * <en>The item label horizontal alignment.</en>
   */
  private Hashtable itemLabelHorizontalAlignment ;

  /**
   *  <zh>ݼһеݵıǩˮƽ뷽ʽ.</zh>
   * <en>Strorage the label horizontal alignment of a dataset column.</en>
   */
  private Hashtable itemColumnLabelHorizontalAlignment ;

  /**
   *  <zh>ݼһеݵıǩˮƽ뷽ʽ.</zh>
   * <en>Strorage the label horizontal alignment of a dataset row.</en>
   */
  private Hashtable itemRowLabelHorizontalAlignment ;

  /**
   * <zh>ֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>The label horizontal alignment of negative value items.</en>
   */
  private HorizontalAlignment negativeItemLabelHorizontalAlignment ;

  /**
   * <zh>ֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>The label horizontal alignment of positive value items of a dataset.</en>
   */
  private HorizontalAlignment positiveItemLabelHorizontalAlignment ;

  /**
   * <zh>ıǩˮƽ뷽ʽ.</zh>
   * <en>The label horizontal alignment of all items of a dataset.</en>
   */
  private HorizontalAlignment baseItemLabelHorizontalAlignment ;

  /**
   * <zh>ǩĴֱ뷽ʽ.</zh>
   * <en>The item label vertical alignment.</en>
   */
  private Hashtable itemLabelVerticalAlignment ;

  /**
   *  <zh>ݼһеݵıǩĴֱ뷽ʽ.</zh>
   * <en>Strorage the label vertical alignment of a dataset column.</en>
   */
  private Hashtable itemColumnLabelVerticalAlignment ;

  /**
   *  <zh>ݼһеݵıǩĴֱ뷽ʽ.</zh>
   * <en>Strorage the label vertical alignment of a dataset row.</en>
   */
  private Hashtable itemRowLabelVerticalAlignment ;

  /**
   * <zh>ֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>The label vertical alignment of negative value items.</en>
   */
  private VerticalAlignment negativeItemLabelVerticalAlignment ;

  /**
   * <zh>ֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>The label vertical alignment of positive value items of a dataset.</en>
   */
  private VerticalAlignment positiveItemLabelVerticalAlignment ;

  /**
   * <zh>ıǩĴֱ뷽ʽ.</zh>
   * <en>The label vertical alignment of all items of a dataset.</en>
   */
  private VerticalAlignment baseItemLabelVerticalAlignment ;

  /**
   * <zh>ݼÿݵıǩķ.</zh>
   * <en>Storage the label direction of a data item.</en>
   */
  private Hashtable itemLabelDirection ;

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Storage the label direction of a dataset column.</en>
   */
  private Hashtable itemColumnLabelDirection ;

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Storage the label direction of a dataset row.</en>
   */
  private Hashtable itemRowLabelDirection ;

  /**
   * <zh>ݹõıǩ.</zh>
   * <en>The base label direction.</en>
   */
  private TextDirection baseItemLabelDirection ;

  /**
   * <zh>ƱPaint.</zh>
   * <en>An optional color used to fill the render background. </en>
   */
  private transient Paint backgroundPaint ;

  /**
   * <zh>ͼƬ.</zh>
   * <en>An optional image for the render background. </en>
   */
  private transient Image backgroundImage ; // not currently serialized

  /**
   * <zh>ͼƬ뷽ʽ.</zh>
   * <en>The alignment for the background image.</en>
   */
  private Align backgroundImageAlignment = null ;

  /**
   * <zh>ͼƬ͸alphaֵ.</zh>
   * <en>The alpha value used to draw the background image.</en>
   */
  private float backgroundImageAlpha ;

  /**
   * <zh>Ƿrender߿.</zh>
   * <en>A flag that controls whether or not the render area border is drawn.</en>
   */
  private boolean renderAreaBorderVisible ;

  /**
   * <zh>render߿Stroke.</zh>
   * <en>The Stroke used to draw a border around the render area.</en>
   */
  private transient Stroke renderAreaBorderStroke ;

  /**
   * <zh>render߿Paint.</zh>
   * <en>The Paint used to draw a border around the render area.</en>
   */
  private transient Paint renderAreaBorderPaint ;

  /**
   * <zh>ͼαǩĶ.</zh>
   * <en>The item label generator for all items of a dataset.</en>
   */
  private ILabelGenerator labelGenerator ;

  /**
   * <zh>ͼεtooltipĶ.</zh>
   * <en>The tooltip generator for all items of a dataset.</en>
   */
  private IToolTipGenerator tooltipGenerator ;

  /**
   * <zh>ݼ.</zh>
   * <en>The dataset.</en>
   */
  private IMatrixDataset dataset ;

  /**
   * <zh>Զͼ.</zh>
   * <en>The custom legend type.</en>
   */
  private LegendType customLegendType ;

  /**
   * <zh>,ɱл.</zh>
   * <en>The listener collection,not serializable.</en>
   */
  private transient EventListenerList listenerList ;

  /**
   * <zh>ṩͼҪһϵж.</zh>
   * <en>The drawing supplier for this render.</en>
   */
  private IDrawingSupplier drawingSupplier ;

  /**
   * <zh>ͼõPaint.</zh>
   * <en>The paint array used to fill shape.</en>
   */
  protected transient Paint[] fillPaints ;

  /**
   * <zh>ͼα߿õPaint.</zh>
   * <en>The paint array used to outline shape.</en>
   */
  protected transient Paint[] borderPaints ;

  /**
   * <zh>ͼα߿õStroke.</zh>
   * <en>The Stroke array used to outline shape.</en>
   */
  protected transient Stroke[] borderStrokes ;

  /**
   * <zh>Ƿͼεurl.</zh>
   * <en>Whether or not generator url of shape.</en>
   */
  private boolean generatorUrl ;

  /**
   * <zh>Ƿͼεtooltip.</zh>
   * <en>Whether or not generator tooltip of shape.</en>
   */
  private boolean generatorTooltip ;

  /**
   * <zh>ͼ.</zh>
   * <en>The chart area.</en>
   */
  private Rectangle2D chartArea ;

  /**
   * <zh>ͼλ.</zh>
   * <en>The legend position.</en>
   */
  private RectangleEdge legendPositon ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  protected AbstractRender(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }

    this.itemVisible = new Hashtable() ;
    this.itemColumnVisible = new Hashtable() ;
    this.itemRowVisible = new Hashtable() ;

    this.itemFillPaint = new Hashtable() ;
    this.itemColumnFillPaint = new Hashtable() ;
    this.itemRowFillPaint = new Hashtable() ;
    this.baseItemFillPaint = DEFAULT_BASE_ITEM_FILL_PAINT ;
    this.autoSelectFillPaint = DEFAULT_AUTO_SELECT_FILL_PAINT ;

    this.itemFill = new Hashtable() ;
    this.itemColumnFill = new Hashtable() ;
    this.itemRowFill = new Hashtable() ;
    this.baseItemFill = DEFAULT_BASE_ITEM_FILL ;

    this.itemFillPaintTransformer = new Hashtable() ;
    this.itemColumnFillPaintTransformer = new Hashtable() ;
    this.itemRowFillPaintTransformer = new Hashtable() ;
    this.baseItemFillPaintTransformer = DEFAULT_BASE_ITEM_FILL_PAINT_TRANSFORMER ;

    this.itemDrawBorder = new Hashtable() ;
    this.itemColumnDrawBorder = new Hashtable() ;
    this.itemRowDrawBorder = new Hashtable() ;
    this.baseItemDrawBorder = DEFAULT_DRAW_BORDER ;
    this.autoSelectBorderPaint = DEFAULT_AUTO_SELECT_BORDER_PAINT ;

    this.itemBorderPaint = new Hashtable() ;
    this.itemColumnBorderPaint = new Hashtable() ;
    this.itemRowBorderPaint = new Hashtable() ;
    this.baseItemBorderPaint = DEFAULT_BASE_BORDER_PAINT ;

    this.itemBorderStroke = new Hashtable() ;
    this.itemColumnBorderStroke = new Hashtable() ;
    this.itemRowBorderStroke = new Hashtable() ;
    this.baseItemBorderStroke = DEFAULT_BASE_BORDER_STROKE ;
    this.autoSelectBorderStroke = DEFAULT_AUTO_SELECT_BORDER_STROKE ;

    this.itemLabelVisible = new Hashtable() ;
    this.itemColumnLabelVisible = new Hashtable() ;
    this.itemRowLabelVisible = new Hashtable() ;
    this.baseItemLabelVisible = DEFAULT_BASE_ITEM_LABEL_VISIBLE ;

    this.itemLabelFont = new Hashtable() ;
    this.itemColumnLabelFont = new Hashtable() ;
    this.itemRowLabelFont = new Hashtable() ;
    this.baseItemLabelFont = DEFAULT_BASE_ITEM_LABEL_FONT ;

    this.itemLabelPaint = new Hashtable() ;
    this.itemColumnLabelPaint = new Hashtable() ;
    this.itemRowLabelPaint = new Hashtable() ;
    this.baseItemLabelPaint = DEFAULT_BASE_ITEM_LABEL_PAINT ;

    this.itemLabelPosition = new Hashtable() ;
    this.itemColumnLabelPosition = new Hashtable() ;
    this.itemRowLabelPosition = new Hashtable() ;
    this.baseItemLabelPosition = TextPosition.INSIDE ;

    this.itemLabelDirection = new Hashtable() ;
    this.itemColumnLabelDirection = new Hashtable() ;
    this.itemRowLabelDirection = new Hashtable() ;
    this.baseItemLabelDirection = TextDirection.HORIZONTAL ;

    this.itemLabelHorizontalAlignment = new Hashtable() ;
    this.itemColumnLabelHorizontalAlignment = new Hashtable() ;
    this.itemRowLabelHorizontalAlignment = new Hashtable() ;
    this.negativeItemLabelHorizontalAlignment = null ;
    this.positiveItemLabelHorizontalAlignment = null ;
    this.baseItemLabelHorizontalAlignment = DEFAULT_BASE_ITEM_LABEL_HORIZONTAL_ALIGNMENT ;

    this.itemLabelVerticalAlignment = new Hashtable() ;
    this.itemColumnLabelVerticalAlignment = new Hashtable() ;
    this.itemRowLabelVerticalAlignment = new Hashtable() ;
    this.negativeItemLabelVerticalAlignment = null ;
    this.positiveItemLabelVerticalAlignment = null ;
    this.baseItemLabelVerticalAlignment = DEFAULT_BASE_ITEM_LABEL_VERTICAL_ALIGNMENT ;

    this.backgroundPaint = DEFAULT_BACKGROUND_PAINT ;
    this.backgroundImage = null ;
    this.backgroundImageAlignment = DEFAULT_BACKGROUND_IMAGE_ALIGNMENT ;
    this.backgroundImageAlpha = 0.5F ;

    this.renderAreaBorderVisible = DEFAULT_RENDER_AREA_BORDER_VISIBLE ;
    this.renderAreaBorderStroke = DEFAULT_BORDER_STROKE ;
    this.renderAreaBorderPaint = DEFAULT_BORDER_PAINT ;

    this.labelGenerator = DEFAULT_LABEL_GENERATOR ;
    this.tooltipGenerator = DEFAULT_TOOLTIP_GENERATOR ;

    this.generatorUrl = DEFAULT_GENERATOR_URL ;
    this.generatorTooltip = DEFAULT_GENERATOR_TOOLTIP ;

    this.dataset = dataset ;
    this.listenerList = new EventListenerList() ;
    this.drawingSupplier = DEFAULT_DRAWINT_SUPPLIER ;
    this.chartArea = null ;
    this.legendPositon = null ;
    this.customLegendType = null ;

    this.dataset.addChangeListener(this) ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Returns the dataset of this render.</en>
   *
   * @return IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (never null).</en>
   */
  public IMatrixDataset getDataset() {
    return this.dataset ;
  }

  /**
   * <zh>ıݼ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the dataset of this render,and sends a {@link RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public void setDataset(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }

    this.dataset = dataset ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ĳĳеǷʾ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified item should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  public boolean isItemVisible(int row, int column) {
    checkRowAndColumn(row, column) ;

    //Ƿ񵥶
    Object value = this.itemVisible.get(intToKey(row, column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }
    //ֵ
    value = this.itemColumnVisible.get(String.valueOf(column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }
    //ֵ
    value = this.itemRowVisible.get(String.valueOf(row)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    return true ;
  }

  /**
   * <zh>ĳĳеǷʾ,м{@link RenderChangeEvent}¼.
   * ͨ˷ݼĳĳеǷʾ.</zh>
   * <en>Sets a boolean that indicates whether or not the specified item should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setItemVisible(int row, int column, boolean visible) {
    checkRowAndColumn(row, column) ;
    this.itemVisible.put(intToKey(row, column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼһеǷȫɼ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified column should be drawn.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isColumnItemVisible(int column) {
    checkColumnValid(column) ;

    return (Boolean)this.itemColumnVisible.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһǷȫʾ,м{@link RenderChangeEvent}¼.˷ڿݼ
   * еһǷʾ.</zh>
   * <en>Sets the flag that controls whether a column is visible and sends a {@link RenderChangeEvent}
   *  to all registered listeners.This is typically used to hide an entire column data.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The boolean flag.</en>
   */
  public void setColumnItemVisible(int column, boolean visible) {
    checkColumnValid(column) ;
    this.itemColumnVisible.put(String.valueOf(column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼһеǷȫɼ.</zh>
   * <en>Returns a boolean that indicates whether or not the specified row should be drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ʾtrue,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isRowItemVisible(int row) {
    checkRowValid(row) ;

    return (Boolean)this.itemRowVisible.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһǷȫʾ,м{@link RenderChangeEvent}¼.˷ڿݼ
   * еһǷʾ.</zh>
   * <en>Sets the flag that controls whether a row is visible and sends a {@link RenderChangeEvent}
   *  to all registered listeners.This is typically used to hide an entire row data.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The boolean flag.</en>
   */
  public void setRowItemVisible(int row, boolean visible) {
    checkRowValid(row) ;
    this.itemRowVisible.put(String.valueOf(row), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>Ƿͼڲ.</zh>
   * <en>Returns whether or not fill shape.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isItemFill(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemFill.get(intToKey(row, column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    value = this.itemColumnFill.get(String.valueOf(column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }
    value = this.itemRowFill.get(String.valueOf(row)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    return this.baseItemFill ;
  }

  /**
   * <zh>Ƿͼڲ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not fill shape and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setItemFill(int row, int column, boolean flag) {
    checkRowAndColumn(row, column) ;
    this.itemFill.put(intToKey(row, column), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Returns whether or not to fill all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ֵ.</zh>
   * <en>The Boolean object.</en>
   */
  public Boolean isColumnItemFill(int column) {
    checkColumnValid(column) ;

    return (Boolean)this.itemColumnFill.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһǷͼڲ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to fill all items of a dataset column and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setColumnItemFill(int column, boolean flag) {
    checkColumnValid(column) ;
    this.itemColumnFill.put(String.valueOf(column), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһǷͼڲ.</zh>
   * <en>Returns whether or not to fill all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ֵ.</zh>
   * <en>The Boolean object.</en>
   */
  public Boolean isRowItemFill(int row) {
    checkRowValid(row) ;

    return (Boolean)this.itemRowFill.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼеһǷͼڲ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to fill all items of a dataset row and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setRowItemFill(int row, boolean flag) {
    checkRowValid(row) ;
    this.itemRowFill.put(String.valueOf(row), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>Ƿͼڲ.</zh>
   * <en>Returns whether or not fill all items of a dataset.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isBaseItemFill() {
    return this.baseItemFill ;
  }

  /**
   * <zh>Ƿͼڲ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not fill all items of a dataset and sends a {@link RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setBaseItemFill(boolean flag) {
    if (this.baseItemFill != flag) {
      this.baseItemFill = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ͼڲpaint.</zh>
   * <en>Returns the paint used to fill data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getItemFillPaint(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemFillPaint.get(intToKey(row, column)) ;
    if (value != null) {
      return (Paint) value ;
    }

    value = this.itemColumnFillPaint.get(String.valueOf(column)) ;
    if (value != null) {
      return (Paint) value ;
    }
    value = this.itemRowFillPaint.get(String.valueOf(row)) ;
    if (value != null) {
      return (Paint) value ;
    }

    return this.baseItemFillPaint ;
  }

  /**
   * <zh>ͼڲpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill data items as they are drawn and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  public void setItemFillPaint(int row, int column, Paint paint) {
    checkRowAndColumn(row, column) ;
    this.itemFillPaint.put(intToKey(row, column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõpaint.</zh>
   * <en>Returns the paint used to fill all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getColumnItemFillPaint(int column) {
    checkColumnValid(column) ;

    return (Paint)this.itemColumnFillPaint.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһõpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill all items of a dataset column and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  public void setColumnItemFillPaint(int column, Paint paint) {
    checkColumnValid(column) ;
    this.itemColumnFillPaint.put(String.valueOf(column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõpaint.</zh>
   * <en>Returns the paint used to fill all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getRowItemFillPaint(int row) {
    checkRowValid(row) ;

    return (Paint)this.itemRowFillPaint.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼеһõpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill all items of a dataset row and sends
   * a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  public void setRowItemFillPaint(int row, Paint paint) {
    checkRowValid(row) ;
    this.itemRowFillPaint.put(String.valueOf(row), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>õpaint.</zh>
   * <en>Returns the base fill paint.</en>
   *
   * @return Paint <zh>õpaint (Ϊnull).</zh>
   * <en>The base fill paint (null permitted).</en>
   */
  public Paint getBaseItemFillPaint() {
    return this.baseItemFillPaint ;
  }

  /**
   * <zh>õpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the base paint to fill all items of a dataset and sends a {@link RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The base fill paint (null permitted).</en>
   */
  public void setBaseItemFillPaint(Paint paint) {
    if (!(this.baseItemFillPaint == null && paint == null)) {
      this.baseItemFillPaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ǷԶѡͼεPaint.</zh>
   * <en>Returns whether or not the fill paint is auto selected.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isAutoSelectFillPaint() {
    return this.autoSelectFillPaint ;
  }

  /**
   * <zh>ǷԶѡͼεPaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>A flag that controls whether or not the fill paint is auto selected,and sends a
   * {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setAutoSelectFillPaint(boolean flag) {
    if (this.autoSelectFillPaint != flag) {
      this.autoSelectFillPaint = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,򷵻ͼڲķʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit an item.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (maybe null).</en>
   */
  public GradientPaintTransformType getItemFillPaintTransformer(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemFillPaintTransformer.get(intToKey(row, column)) ;
    if (value != null) {
      return (GradientPaintTransformType) value ;
    }

    value = this.itemColumnFillPaintTransformer.get(String.valueOf(column)) ;
    if (value != null) {
      return (GradientPaintTransformType) value ;
    }
    value = this.itemRowFillPaintTransformer.get(String.valueOf(row)) ;
    if (value != null) {
      return (GradientPaintTransformType) value ;
    }

    return this.baseItemFillPaintTransformer ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ͼڲķʽ.
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit an item,and sends a
   * {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType
   * <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (null permitted).</en>
   */
  public void setItemFillPaintTransformer(int row, int column,
                                          GradientPaintTransformType transformer) {
    checkRowAndColumn(row, column) ;
    this.itemFillPaint.put(intToKey(row, column), transformer) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (possibly null).</en>
   */
  public GradientPaintTransformType getColumnItemFillPaintTransformer(int column) {
    checkColumnValid(column) ;

    return (GradientPaintTransformType)this.itemColumnFillPaintTransformer.
      get(String.valueOf(column)) ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ.
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset column,and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType <zh>paint (Ϊnull).</zh>
   * <en>The GradientPaintTransformType (null permitted).</en>
   */
  public void setColumnItemFillPaintTransformer(int column, GradientPaintTransformType transformer) {
    checkColumnValid(column) ;

    this.itemColumnFillPaintTransformer.put(String.valueOf(column), transformer) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return GradientPaintTransformType <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The GradientPaintTransformType object (possibly null).</en>
   */
  public GradientPaintTransformType getRowItemFillPaintTransformer(int row) {
    checkRowValid(row) ;

    return (GradientPaintTransformType)this.itemRowFillPaintTransformer.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼһõͼڲķʽ.
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset row,and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param transformer GradientPaintTransformType <zh>paint (Ϊnull).</zh>
   * <en>The GradientPaintTransformType (null permitted).</en>
   */
  public void setRowItemFillPaintTransformer(int row, GradientPaintTransformType transformer) {
    checkRowValid(row) ;
    this.itemRowFillPaintTransformer.put(String.valueOf(row), transformer) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼݹõͼڲķʽ,
   * ䷽ʽ.</zh>
   * <en>Returns an optional class used to transform gradient paint object to fit all items of a
   * dataset.</en>
   *
   * @return GradientPaintTransformType <zh>õpaint (Ϊnull).</zh>
   * <en>The base GradientPaintTransformType object (possibly null).</en>
   */
  public GradientPaintTransformType getBaseItemFillPaintTransformer() {
    return this.baseItemFillPaintTransformer ;
  }

  /**
   * <zh>ͼڲpaintΪGradientPaint,ݼõͼڲķʽ.
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets an optional class used to transform gradient paint object to fit all items of a
   * dataset,and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param transformer GradientPaintTransformType
   * <zh>GradientPaintTransformType (Ϊnull).</zh>
   * <en>The base GradientPaintTransformType object (null permitted).</en>
   */
  public void setBaseItemFillPaintTransformer(GradientPaintTransformType transformer) {
    this.baseItemFillPaintTransformer = transformer ;
    fireRenderChanged() ;
  }

  /**
   * <zh>Ƿͼα߿.</zh>
   * <en>Returns true if an item border is visible,and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>Ʊ߽򷵻true,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  public boolean isItemDrawBorder(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemDrawBorder.get(intToKey(row, column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    value = this.itemColumnDrawBorder.get(String.valueOf(column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }
    value = this.itemRowDrawBorder.get(String.valueOf(row)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    return this.baseItemDrawBorder ;
  }

  /**
   * <zh>Ƿͼα߿,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the visibility of an item border and sends a {@link RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setItemDrawBorder(int row, int column, boolean visible) {
    checkColumnValid(row) ;
    this.itemDrawBorder.put(intToKey(row, column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset column .</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>Ʊ߽򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isColumnItemDrawBorder(int column) {
    checkColumnValid(column) ;

    return (Boolean)this.itemColumnDrawBorder.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһǷƱ߿,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset column drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setColumnItemDrawBorder(int column, boolean visible) {
    checkColumnValid(column) ;
    this.itemColumnDrawBorder.put(String.valueOf(column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset row .</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>Ʊ߽򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isRowItemDrawBorder(int row) {
    checkRowValid(row) ;

    return (Boolean)this.itemRowDrawBorder.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼеһǷƱ߿,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset row drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setRowItemDrawBorder(int row, boolean visible) {
    checkRowValid(row) ;
    this.itemRowDrawBorder.put(String.valueOf(row), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеݵı߿Ƿɼ.</zh>
   * <en>Returns whether or not outline all items of a dataset.</en>
   *
   * @return boolean <zh>Ʊ߽򷵻true,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  public boolean isBaseItemDrawBorder() {
    return this.baseItemDrawBorder ;
  }

  /**
   * <zh>ݼеǷƱ߿,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets whether or not to outline all items of a dataset drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setBaseItemDrawBorder(boolean visible) {
    if (this.baseItemDrawBorder != visible) {
      this.baseItemDrawBorder = visible ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ڻͼα߿paint.</zh>
   * <en>Returns the paint used to outline data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getItemBorderPaint(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemBorderPaint.get(intToKey(row, column)) ;
    if (value != null) {
      return (Paint) value ;
    }

    value = this.itemColumnBorderPaint.get(String.valueOf(column)) ;
    if (value != null) {
      return (Paint) value ;
    }
    value = this.itemRowBorderPaint.get(String.valueOf(row)) ;
    if (value != null) {
      return (Paint) value ;
    }

    return this.baseItemBorderPaint ;
  }

  /**
   * <zh>ڻͼα߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline data items as they are drawn and sends a
   *  {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setItemBorderPaint(int row, int column, Paint paint) {
    checkColumnValid(row) ;
    this.itemBorderPaint.put(intToKey(row, column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the paint used to outline all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getColumnItemBorderPaint(int column) {
    checkColumnValid(column) ;

    return (Paint)this.itemColumnBorderPaint.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһõı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline all items of a dataset column drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public void setColumnItemBorderPaint(int column, Paint paint) {
    checkColumnValid(column) ;
    this.itemColumnBorderPaint.put(String.valueOf(column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the paint used to outline all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getRowItemBorderPaint(int row) {
    checkRowValid(row) ;

    return (Paint)this.itemRowBorderPaint.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼеһõı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to outline all items of a dataset row drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public void setRowItemBorderPaint(int row, Paint paint) {
    checkRowValid(row) ;
    this.itemRowBorderPaint.put(String.valueOf(row), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>õı߿paint.</zh>
   * <en>Returns the base paint to outline all items of a dataset.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getBaseItemBorderPaint() {
    return this.baseItemBorderPaint ;
  }

  /**
   * <zh>õı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the base border paint and sends a {@link RenderChangeEvent} to
   * all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public void setBaseItemBorderPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.baseItemBorderPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ǷԶѡͼα߿Paint.</zh>
   * <en>Returns whether or not the border paint is auto selected</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isAutoSelectBorderPaint() {
    return this.autoSelectBorderPaint ;
  }

  /**
   * <zh>ǷԶѡͼα߿Paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Returns whether or not the border paint is auto selected,and sends a
   *  {@link RenderChangeEvent} to all registered listeners</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setAutoSelectBorderPaint(boolean flag) {
    if (this.autoSelectBorderPaint != flag) {
      this.autoSelectBorderPaint = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ڻͼα߿stroke.</zh>
   * <en>Returns the stroke used to outline data items as they are drawn.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getItemBorderStroke(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemBorderStroke.get(intToKey(row, column)) ;
    if (value != null) {
      return (Stroke) value ;
    }

    value = this.itemColumnBorderStroke.get(String.valueOf(column)) ;
    if (value != null) {
      return (Stroke) value ;
    }
    value = this.itemRowBorderStroke.get(String.valueOf(row)) ;
    if (value != null) {
      return (Stroke) value ;
    }

    return this.baseItemBorderStroke ;
  }

  /**
   * <zh>ڻͼα߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline data items as they are drawn and sends a
   *  {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (null permitted).</en>
   */
  public void setItemBorderStroke(int row, int column, Stroke stroke) {
    checkColumnValid(row) ;
    this.itemBorderStroke.put(intToKey(row, column), stroke) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the stroke used to outline all items of a dataset column drawn by the render.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  public Stroke getColumnItemBorderStroke(int column) {
    checkColumnValid(column) ;

    return (Stroke)this.itemColumnBorderStroke.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһõı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline all items of a dataset column drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  public void setColumnItemBorderStroke(int column, Stroke stroke) {
    checkColumnValid(column) ;
    this.itemColumnBorderStroke.put(String.valueOf(column), stroke) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһõı߿paint.</zh>
   * <en>Returns the stroke used to outline all items of a dataset row drawn by the render.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  public Stroke getRowItemBorderStroke(int row) {
    checkRowValid(row) ;

    Object value = this.itemRowBorderStroke.get(String.valueOf(row)) ;
    if (value != null) {
      return (Stroke) value ;
    }

    return null ;
  }

  /**
   * <zh>ݼеһõı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline all items of a dataset row drawn by the render
   * and sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (possibly null).</en>
   */
  public void setRowItemBorderStroke(int row, Stroke stroke) {
    checkRowValid(row) ;
    this.itemRowBorderStroke.put(String.valueOf(row), stroke) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>õı߿paint.</zh>
   * <en>Returns the base stroke to outline all items of a dataset.</en>
   *
   * @return Stroke <zh>paint (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getBaseItemBorderStroke() {
    return this.baseItemBorderStroke ;
  }

  /**
   * <zh>õı߿paint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the base border stroke and sends a {@link RenderChangeEvent} to
   * all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public void setBaseItemBorderStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.baseItemBorderStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ǷԶѡͼα߿Stroke.</zh>
   * <en>Returns whether or not the border stroke is auto selected</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isAutoSelectBorderStroke() {
    return this.autoSelectBorderStroke ;
  }

  /**
   * <zh>ǷԶѡͼα߿Stroke,м{@link RenderChangeEvent}¼.</zh>
   * <en>Returns whether or not the border stroke is auto selected,and sends a
   *  {@link RenderChangeEvent} to all registered listeners</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public void setAutoSelectBorderStroke(boolean flag) {
    if (this.autoSelectBorderStroke != flag) {
      this.autoSelectBorderStroke = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ĳıǩǷʾ.</zh>
   * <en>Returns true if an item label is visible,and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return <zh>ɼtrue,򷵻false.</zh>
   * <en>A boolean.</en>
   */
  public boolean isItemLabelVisible(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelVisible.get(intToKey(row, column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    value = this.itemColumnLabelVisible.get(String.valueOf(column)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }
    value = this.itemRowLabelVisible.get(String.valueOf(row)) ;
    if (value != null) {
      return ((Boolean) value).booleanValue() ;
    }

    return this.baseItemLabelVisible ;
  }

  /**
   * <zh>ĳıǩǷʾ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the visibility of an item label and sends a {@link RenderChangeEvent}
   * to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setItemLabelVisible(int row, int column, boolean visible) {
    checkRowAndColumn(row, column) ;
    this.itemLabelVisible.put(intToKey(row, column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩǷʾ.</zh>
   * <en>Returns true if the item labels for a column are visible,
   * and false otherwise.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ʾǩ򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isColumnItemLabelVisible(int column) {
    checkColumnValid(column) ;

    return (Boolean)this.itemColumnLabelVisible.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼеһݵıǩǷʾ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that controls the visibility of the item labels for a column and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setColumnItemLabelVisible(int column, boolean visible) {
    checkColumnValid(column) ;
    this.itemColumnLabelVisible.put(String.valueOf(column), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩǷʾ.</zh>
   * <en>Returns true if the item labels for a row are visible,
   * and false otherwise.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Boolean <zh>ʾǩ򷵻true,򷵻false (Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public Boolean isRowItemLabelVisible(int row) {
    checkRowValid(row) ;

    return (Boolean)this.itemRowLabelVisible.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼеһݵıǩǷʾ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that controls the visibility of the item labels for a row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param visible boolean <zh>ֵ.</zh>
   * <en>The visible flag.</en>
   */
  public void setRowItemLabelVisible(int row, boolean visible) {
    checkColumnValid(row) ;
    this.itemRowLabelVisible.put(String.valueOf(row), Boolean.valueOf(visible)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеݵıǩǷʾ.</zh>
   * <en>Returns the base setting for item label visibility.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A flag (possibly null).</en>
   */
  public boolean getBaseItemLabelVisible() {
    return this.baseItemLabelVisible ;
  }

  /**
   * <zh>ݼеݵıǩǷʾ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the base flag that controls whether or not item labels are visible and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param visible <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemLabelVisible(boolean visible) {
    if (this.baseItemLabelVisible != visible) {
      this.baseItemLabelVisible = visible ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the font for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getItemLabelFont(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelFont.get(intToKey(row, column)) ;
    if (value != null) {
      return (Font) value ;
    }

    value = this.itemColumnLabelFont.get(String.valueOf(column)) ;
    if (value != null) {
      return (Font) value ;
    }
    value = this.itemRowLabelFont.get(String.valueOf(row)) ;
    if (value != null) {
      return (Font) value ;
    }

    return this.baseItemLabelFont ;
  }

  /**
   * <zh>ñǩ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the font for an item label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  public void setItemLabelFont(int row, int column, Font font) {
    checkRowAndColumn(row, column) ;
    this.itemLabelFont.put(intToKey(row, column), font) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Returns the font for all items label of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (maybe null).</en>
   */
  public Font getColumnItemLabelFont(int column) {
    checkColumnValid(column) ;

    return (Font)this.itemColumnLabelFont.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  public void setColumnItemLabelFont(int column, Font font) {
    checkColumnValid(column) ;
    this.itemColumnLabelFont.put(String.valueOf(column), font) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩ.</zh>
   * <en>Returns the font for all items label of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (maybe null).</en>
   */
  public Font getRowItemLabelFont(int row) {
    checkRowValid(row) ;

    return (Font)this.itemRowLabelFont.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  public void setRowItemLabelFont(int row, Font font) {
    checkRowValid(row) ;
    this.itemRowLabelFont.put(String.valueOf(row), font) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩ.</zh>
   * <en>Returns the label font for all items of a dataset.</en>
   *
   * @return Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getBaseItemLabelFont() {
    return this.baseItemLabelFont ;
  }

  /**
   * <zh>ݼݵıǩ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label font for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setBaseItemLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    if (!this.baseItemLabelFont.equals(font)) {
      this.baseItemLabelFont = font ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩpaint.</zh>
   * <en>Returns the paint for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getItemLabelPaint(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelPaint.get(intToKey(row, column)) ;
    if (value != null) {
      return (Paint) value ;
    }

    value = this.itemColumnLabelPaint.get(String.valueOf(column)) ;
    if (value != null) {
      return (Paint) value ;
    }
    value = this.itemRowLabelPaint.get(String.valueOf(row)) ;
    if (value != null) {
      return (Paint) value ;
    }

    return this.baseItemLabelPaint ;
  }

  /**
   * <zh>ñǩpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the paint for an item label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setItemLabelPaint(int row, int column, Paint paint) {
    checkRowAndColumn(row, column) ;
    this.itemLabelPaint.put(intToKey(row, column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getColumnItemLabelPaint(int column) {
    checkColumnValid(column) ;

    return (Paint)this.itemColumnLabelPaint.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setColumnItemLabelPaint(int column, Paint paint) {
    checkColumnValid(column) ;
    this.itemColumnLabelPaint.put(String.valueOf(column), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getRowItemLabelPaint(int row) {
    checkRowValid(row) ;

    return (Paint)this.itemRowLabelPaint.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setRowItemLabelPaint(int row, Paint paint) {
    checkRowValid(row) ;
    this.itemRowLabelPaint.put(String.valueOf(row), paint) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩpaint.</zh>
   * <en>Returns the label paint for all items of a dataset.</en>
   *
   * @return Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getBaseItemLabelPaint() {
    return this.baseItemLabelPaint ;
  }

  /**
   * <zh>ݼݵıǩpaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label paint for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>ǩpaint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setBaseItemLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.baseItemLabelPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>رǩλ.</zh>
   * <en>Returns the label position.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  public TextPosition getItemLabelPosition(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelPosition.get(intToKey(row, column)) ;
    if (value != null) {
      return (TextPosition) value ;
    }

    value = this.itemColumnLabelPosition.get(String.valueOf(column)) ;
    if (value != null) {
      return (TextPosition) value ;
    }
    value = this.itemRowLabelPosition.get(String.valueOf(row)) ;
    if (value != null) {
      return (TextPosition) value ;
    }

    return this.baseItemLabelPosition ;
  }

  /**
   * <zh>ñǩλ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label position and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  public void setItemLabelPosition(int row, int column, TextPosition position) {
    checkRowAndColumn(row, column) ;
    this.itemLabelPosition.put(intToKey(row, column), position) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  public TextPosition getColumnItemLabelPosition(int column) {
    checkColumnValid(column) ;

    return (TextPosition)this.itemColumnLabelPosition.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩλ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  public void setColumnItemLabelPosition(int column, TextPosition position) {
    checkColumnValid(column) ;
    this.itemColumnLabelPosition.put(String.valueOf(column), position) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  public TextPosition getRowItemLabelPosition(int row) {
    checkRowValid(row) ;

    return (TextPosition)this.itemRowLabelPosition.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩλ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null permitted).</en>
   */
  public void setRowItemLabelPosition(int row, TextPosition position) {
    checkRowValid(row) ;
    this.itemRowLabelPosition.put(String.valueOf(row), position) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩλ.</zh>
   * <en>Returns the label position for all items of a dataset.</en>
   *
   * @return LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (never null).</en>
   */
  public TextPosition getBaseItemLabelPosition() {
    return this.baseItemLabelPosition ;
  }

  /**
   * <zh>ݼݵıǩλ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label position for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param position LabelPosition <zh>ǩλ (Ϊnull).</zh>
   * <en>The label position (null not permitted).</en>
   */
  public void setBaseItemLabelPosition(TextPosition position) {
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument.") ;
    }

    if (!this.baseItemLabelPosition.equals(position)) {
      this.baseItemLabelPosition = position ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩķ.</zh>
   * <en>Returns the label direction.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (never null).</en>
   */
  public TextDirection getItemLabelDirection(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelDirection.get(intToKey(row, column)) ;
    if (value != null) {
      return (TextDirection) value ;
    }

    value = this.itemColumnLabelDirection.get(String.valueOf(column)) ;
    if (value != null) {
      return (TextDirection) value ;
    }
    value = this.itemRowLabelDirection.get(String.valueOf(row)) ;
    if (value != null) {
      return (TextDirection) value ;
    }

    return this.baseItemLabelDirection ;
  }

  /**
   * <zh>ñǩķ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label direction and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param direction TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (null permitted).</en>
   */
  public void setItemLabelDirection(int row, int column, TextDirection direction) {
    checkRowAndColumn(row, column) ;
    this.itemLabelDirection.put(intToKey(row, column), direction) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩķ.</zh>
   * <en>Returns the label direction for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (never null).</en>
   */
  public TextDirection getColumnItemLabelDirection(int column) {
    checkColumnValid(column) ;

    return (TextDirection)this.itemColumnLabelDirection.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩķ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label direction for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param direction TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (null permitted).</en>
   */
  public void setColumnItemLabelDirection(int column, TextDirection direction) {
    checkColumnValid(column) ;
    this.itemColumnLabelDirection.put(String.valueOf(column), direction) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩķ.</zh>
   * <en>Returns the label direction for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (never null).</en>
   */
  public TextDirection getRowItemLabelDirection(int row) {
    checkRowValid(row) ;

    return (TextDirection)this.itemRowLabelDirection.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩķ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label direction for all items of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param direction TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (null permitted).</en>
   */
  public void setRowItemLabelDirection(int row, TextDirection direction) {
    checkRowValid(row) ;
    this.itemRowLabelDirection.put(String.valueOf(row), direction) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩķ.</zh>
   * <en>Returns the label direction for all items of a dataset.</en>
   *
   * @return TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (never null).</en>
   */
  public TextDirection getBaseItemLabelDirection() {
    return this.baseItemLabelDirection ;
  }

  /**
   * <zh>ݼݵıǩķ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label direction for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param direction TextDirection <zh>ǩ (Ϊnull).</zh>
   * <en>The label direction (null not permitted).</en>
   */
  public void setBaseItemLabelDirection(TextDirection direction) {
    if (direction == null) {
      throw new IllegalArgumentException("Null 'direction' argument.") ;
    }

    if (!this.baseItemLabelDirection.equals(direction)) {
      this.baseItemLabelDirection = direction ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (never null).</en>
   */
  public HorizontalAlignment getItemLabelHorizontalAlignment(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelHorizontalAlignment.get(intToKey(row, column)) ;
    if (value != null) {
      return (HorizontalAlignment) value ;
    }

    //ֵ
    IChartData data = this.dataset.getData(row, column) ;
    if (data != null && data instanceof NumberChartData) {
      NumberChartData numberData = (NumberChartData) data ;
      if (numberData.getValue() >= 0 && this.positiveItemLabelHorizontalAlignment != null) {
        return this.positiveItemLabelHorizontalAlignment ;
      }
      if (numberData.getValue() < 0 && this.negativeItemLabelHorizontalAlignment != null) {
        return this.negativeItemLabelHorizontalAlignment ;
      }
    }

    value = this.itemColumnLabelHorizontalAlignment.get(String.valueOf(column)) ;
    if (value != null) {
      return (HorizontalAlignment) value ;
    }
    value = this.itemRowLabelHorizontalAlignment.get(String.valueOf(row)) ;
    if (value != null) {
      return (HorizontalAlignment) value ;
    }

    return this.baseItemLabelHorizontalAlignment ;
  }

  /**
   * <zh>ñǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for an item label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  public void setItemLabelHorizontalAlignment(int row, int column, HorizontalAlignment align) {
    checkRowAndColumn(row, column) ;
    this.itemLabelHorizontalAlignment.put(intToKey(row, column), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the label horizontal alignment for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  public HorizontalAlignment getColumnItemLabelHorizontalAlignment(int column) {
    checkColumnValid(column) ;

    return (HorizontalAlignment)this.itemColumnLabelHorizontalAlignment.
      get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all values items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  public void setColumnItemLabelHorizontalAlignment(int column, HorizontalAlignment align) {
    checkColumnValid(column) ;
    this.itemColumnLabelHorizontalAlignment.put(String.valueOf(column), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the label horizontal alignment for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  public HorizontalAlignment getRowItemLabelHorizontalAlignment(int row) {
    checkRowValid(row) ;

    return (HorizontalAlignment)this.itemRowLabelHorizontalAlignment.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all value items label of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  public void setRowItemLabelHorizontalAlignment(int row, HorizontalAlignment align) {
    checkRowValid(row) ;
    this.itemRowLabelHorizontalAlignment.put(String.valueOf(row), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all negative value items label of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  public HorizontalAlignment getNegativeItemLabelHorizontalAlignment() {
    return this.negativeItemLabelHorizontalAlignment ;
  }

  /**
   * <zh>ݼֵΪıǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all negative value items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  public void setNegativeItemLabelHorizontalAlignment(HorizontalAlignment align) {
    this.negativeItemLabelHorizontalAlignment = align ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеֵΪıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment for all positive value items label of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (maybe null).</en>
   */
  public HorizontalAlignment getPositiveItemLabelHorizontalAlignment() {
    return this.positiveItemLabelHorizontalAlignment ;
  }

  /**
   * <zh>ݼֵΪıǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for all positive value items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null permitted).</en>
   */
  public void setPositiveItemLabelHorizontalAlignment(HorizontalAlignment align) {
    this.positiveItemLabelHorizontalAlignment = align ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩˮƽ뷽ʽ.</zh>
   * <en>Returns the label horizontal alignment for all items of a dataset.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (never null).</en>
   */
  public HorizontalAlignment getBaseItemLabelHorizontalAlignment() {
    return this.baseItemLabelHorizontalAlignment ;
  }

  /**
   * <zh>ݼݵıǩˮƽ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label horizontal alignment for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null not permitted).</en>
   */
  public void setBaseItemLabelHorizontalAlignment(HorizontalAlignment align) {
    if (align == null) {
      throw new IllegalArgumentException("Null 'align' argument.") ;
    }

    if (!this.baseItemLabelHorizontalAlignment.equals(align)) {
      this.baseItemLabelHorizontalAlignment = align ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for an item label.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (never null).</en>
   */
  public VerticalAlignment getItemLabelVerticalAlignment(int row, int column) {
    checkRowAndColumn(row, column) ;

    Object value = this.itemLabelVerticalAlignment.get(intToKey(row, column)) ;
    if (value != null) {
      return (VerticalAlignment) value ;
    }

    //ֵ
    IChartData data = this.dataset.getData(row, column) ;
    if (data != null && data instanceof NumberChartData) {
      NumberChartData numberData = (NumberChartData) data ;
      if (numberData.getValue() >= 0 && this.positiveItemLabelVerticalAlignment != null) {
        return this.positiveItemLabelVerticalAlignment ;
      }
      if (numberData.getValue() < 0 && this.negativeItemLabelVerticalAlignment != null) {
        return this.negativeItemLabelVerticalAlignment ;
      }
    }

    value = this.itemColumnLabelVerticalAlignment.get(String.valueOf(column)) ;
    if (value != null) {
      return (VerticalAlignment) value ;
    }
    value = this.itemRowLabelVerticalAlignment.get(String.valueOf(row)) ;
    if (value != null) {
      return (VerticalAlignment) value ;
    }

    return this.baseItemLabelVerticalAlignment ;
  }

  /**
   * <zh>ñǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for an item label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  public void setItemLabelVerticalAlignment(int row, int column, VerticalAlignment align) {
    checkRowAndColumn(row, column) ;
    this.itemLabelVerticalAlignment.put(intToKey(row, column), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset column.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  public VerticalAlignment getColumnItemLabelVerticalAlignment(int column) {
    checkColumnValid(column) ;

    return (VerticalAlignment)this.itemColumnLabelVerticalAlignment.get(String.valueOf(column)) ;
  }

  /**
   * <zh>ݼһݵıǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all values items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column num,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  public void setColumnItemLabelVerticalAlignment(int column, VerticalAlignment align) {
    checkColumnValid(column) ;
    this.itemColumnLabelVerticalAlignment.put(String.valueOf(column), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеһݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset row.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  public VerticalAlignment getRowItemLabelVerticalAlignment(int row) {
    checkRowValid(row) ;

    return (VerticalAlignment)this.itemRowLabelVerticalAlignment.get(String.valueOf(row)) ;
  }

  /**
   * <zh>ݼһݵıǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all value items label of a dataset row and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row num,must be greater than or equal to 0.</en>
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  public void setRowItemLabelVerticalAlignment(int row, VerticalAlignment align) {
    checkRowValid(row) ;
    this.itemRowLabelVerticalAlignment.put(String.valueOf(row), align) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for all negative value items label of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  public VerticalAlignment getNegativeItemLabelVerticalAlignment() {
    return this.negativeItemLabelVerticalAlignment ;
  }

  /**
   * <zh>ݼֵΪıǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all negative value items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  public void setNegativeItemLabelVerticalAlignment(VerticalAlignment align) {
    this.negativeItemLabelVerticalAlignment = align ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼеֵΪıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment for all positive value items label of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (maybe null).</en>
   */
  public VerticalAlignment getPositiveItemLabelVerticalAlignment() {
    return this.positiveItemLabelVerticalAlignment ;
  }

  /**
   * <zh>ݼֵΪıǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for all positive value items label and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null permitted).</en>
   */
  public void setPositiveItemLabelVerticalAlignment(VerticalAlignment align) {
    this.positiveItemLabelVerticalAlignment = align ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ݼݵıǩĴֱ뷽ʽ.</zh>
   * <en>Returns the label vertical alignment for all items of a dataset.</en>
   *
   * @return VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (never null).</en>
   */
  public VerticalAlignment getBaseItemLabelVerticalAlignment() {
    return this.baseItemLabelVerticalAlignment ;
  }

  /**
   * <zh>ݼݵıǩĴֱ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the label vertical alignment for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param align VerticalAlignment <zh>ǩĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (null not permitted).</en>
   */
  public void setBaseItemLabelVerticalAlignment(VerticalAlignment align) {
    if (align == null) {
      throw new IllegalArgumentException("Null 'align' argument.") ;
    }

    if (!this.baseItemLabelVerticalAlignment.equals(align)) {
      this.baseItemLabelVerticalAlignment = align ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱPaint.</zh>
   * <en>Returns the background color of the render area.</en>
   *
   * @return <zh>Paint (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getBackgroundPaint() {
    return this.backgroundPaint ;
  }

  /**
   * <zh>ûƱPaint,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the background color of the render area and sends a
   * {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setBackgroundPaint(Paint paint) {
    if (!(this.backgroundPaint == null && paint == null)) {
      this.backgroundPaint = paint ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رͼƬ.</zh>
   * <en>Returns the background image that is used to fill the render's background area.</en>
   *
   * @return <zh>ͼƬ (Ϊnull).</zh>
   * <en>The image (possibly null).</en>
   */
  public Image getBackgroundImage() {
    return this.backgroundImage ;
  }

  /**
   * <zh>ñͼƬ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the background image for the render and sends a
   * {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param image <zh>ͼƬ(Ϊnull).</zh>
   * <en>The image (null permitted).</en>
   */
  public void setBackgroundImage(Image image) {
    if (!(this.backgroundImage == null && image == null)) {
      this.backgroundImage = image ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>رͼƬĶ뷽ʽ.</zh>
   * <en>Returns the background image alignment. Alignment constants are defined
   * in the {@link Align} class.</en>
   *
   * @return <zh>뷽ʽ (Ϊnull).</zh>
   * <en>The alignment (never null).</en>
   */
  public Align getBackgroundImageAlignment() {
    return this.backgroundImageAlignment ;
  }

  /**
   * <zh>ñͼƬĶ뷽ʽ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the alignment for the background image and sends a
   * {@link RenderChangeEvent} to all registered listeners.  Alignment options
   * are defined by the {@link Align} class.</en>
   *
   * @param alignment Align <zh>뷽ʽ (Ϊnull).</zh>
   * <en>The alignment (null not permitted).</en>
   */
  public void setBackgroundImageAlignment(Align alignment) {
    if (alignment == null) {
      throw new IllegalArgumentException("Null 'alignment' argument.") ;
    }

    if (!this.backgroundImageAlignment.equals(alignment)) {
      this.backgroundImageAlignment = alignment ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƱͼƬalphaֵ.</zh>
   * <en>Returns the alpha transparency used to draw the background image.  This
   * is a value in the range 0.0f to 1.0f, where 0.0f is fully transparent
   * and 1.0f is fully opaque.</en>
   *
   * @return <zh>͸ȵalphaֵ.</zh>
   * <en>The alpha transparency.</en>
   */
  public float getBackgroundImageAlpha() {
    return this.backgroundImageAlpha ;
  }

  /**
   * <zh>ûƱͼƬalphaֵ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the alpha transparency used when drawing the background image.</en>
   *
   * @param alpha <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void setBackgroundImageAlpha(float alpha) {
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException(
        "The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    if (this.backgroundImageAlpha != alpha) {
      this.backgroundImageAlpha = alpha ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>render߿Ƿɼ.</zh>
   * <en>Returns the flag that controls whether or not the render area border is
   * drawn.  The default value is true. </en>
   *
   * @return <zh>booleanֵ.</zh>
   * <en>The border visibility flag.</en>
   */
  public boolean isRenderAreaBorderVisible() {
    return this.renderAreaBorderVisible ;
  }

  /**
   * <zh>render߿Ƿɼ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not the render's area border is
   * drawn, and sends a {@link com.ijchart.xychart.event.PlotChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param visible <zh>booleanֵ.</zh>
   * <en>The new flag value.</en>
   */
  public void setRenderAreaBorderVisible(boolean visible) {
    if (this.renderAreaBorderVisible != visible) {
      this.renderAreaBorderVisible = visible ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>render߿Stroke.</zh>
   * <en>Returns the stroke used to outline the render area.</en>
   *
   * @return <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getRenderAreaBorderStroke() {
    return this.renderAreaBorderStroke ;
  }

  /**
   * <zh>render߿Stroke,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the stroke used to outline the render area and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners. </en>
   *
   * @param stroke <zh>Stroke(Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setRenderAreaBorderStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.renderAreaBorderStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>render߿Paint.</zh>
   * <en>Returns the color used to draw the border of the render area.</en>
   *
   * @return <zh>Paint(Ϊnull).</zh>
   * <en>The color (never null).</en>
   */
  public Paint getRenderAreaBorderPaint() {
    return this.renderAreaBorderPaint ;
  }

  /**
   * <zh>render߿Paint,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the border of the render area and sends a
   * {@link com.ijchart.xychart.event.PlotChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setRenderAreaBorderPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.renderAreaBorderPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ͼαǩ{@link ILabelGenerator}.</zh>
   * <en>Returns the item label generator for all items of a dataset.</en>
   *
   * @return ILabelGenerator <zh>ǩɶ (Ϊnull).</zh>
   * <en>The generator (possibly null).</en>
   */
  public ILabelGenerator getLabelGenerator() {
    return this.labelGenerator ;
  }

  /**
   * <zh>ͼαǩ{@link ILabelGenerator},
   * м{@link RenderChangeEvent}¼</zh>
   * <en>Sets the item label generator for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param generator ILabelGenerator <zh>ǩɶ (Ϊnull).</zh>
   * <en>The generator (null permitted).</en>
   */
  public void setLabelGenerator(ILabelGenerator generator) {
    if (!(this.labelGenerator == null && generator == null)) {
      this.labelGenerator = generator ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ͼεtooltip{@link IToolTipGenerator}.</zh>
   * <en>Returns the tool tip generator that should be used for all items of a dataset.</en>
   *
   * @return IToolTipGenerator <zh>tooltipɶ (Ϊnull).</zh>
   * <en>The generator (possibly null).</en>
   */
  public IToolTipGenerator getToolTipGenerator() {
    return this.tooltipGenerator ;
  }

  /**
   * <zh>ͼεtooltip{@link IToolTipGenerator},
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the tool tip generator that should be used for all items of a dataset and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param generator IToolTipGenerator <zh>tooltipɶ (Ϊnull).</zh>
   * <en>The generator (null permitted).</en>
   */
  public void setToolTipGenerator(IToolTipGenerator generator) {
    if (!(this.tooltipGenerator == null && generator == null)) {
      this.tooltipGenerator = generator ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿͼεurl.</zh>
   * <en>Returns whether or not generator url of shape.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isGeneratorUrl() {
    return this.generatorUrl ;
  }

  /**
   * <zh>Ƿͼεurl,м{@link RenderChangeEvent}¼.</zh>
   * <en>Whether or not generator url of shape and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setGeneratorUrl(boolean flag) {
    if (this.generatorUrl != flag) {
      this.generatorUrl = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿͼεtooltip.</zh>
   * <en>Returns whether or not generator tooltip of shape.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isGeneratorTooltip() {
    return this.generatorTooltip ;
  }

  /**
   * <zh>Ƿͼεtooltip,м{@link RenderChangeEvent}¼.</zh>
   * <en>Whether or not generator tooltip of shape and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setGeneratorTooltip(boolean flag) {
    if (this.generatorTooltip != flag) {
      this.generatorTooltip = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>{@link IDrawingSupplier},ṩͼҪһϵж.</zh>
   * <en>Returns the drawing supplier for this render.</en>
   *
   * @return IDrawingSupplier <zh>IDrawingSupplier (Ϊnull).</zh>
   * <en>The drawing supplier for this render (never null).</en>
   */
  public IDrawingSupplier getDrawingSupplier() {
    return this.drawingSupplier ;
  }

  /**
   * <zh>{@link IDrawingSupplier},ṩͼҪһϵж,
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the drawing supplier for this render,and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param supplier IDrawingSupplier <zh>IDrawingSupplier (Ϊnull).</zh>
   * <en>The drawing supplier for this render (null not permitted).</en>
   */
  public void setDrawingSupplier(IDrawingSupplier supplier) {
    if (supplier == null) {
      throw new IllegalArgumentException("Null 'supplier' argument.") ;
    }

    this.drawingSupplier = supplier ;
    fireRenderChanged() ;
  }

  /**
   * <zh>{@link IDrawingSupplier},ṩͼҪһϵж,
   * м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the drawing supplier for this render and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param supplier IDrawingSupplier <zh>IDrawingSupplier (Ϊnull).</zh>
   * <en>The drawing supplier for this render (null not permitted).</en>
   * @param notify boolean <zh>Ƿ֪ͨ¼.</zh>
   * <en>Whether or not notify listeners.</en>
   */
  public void setDrawingSupplier(IDrawingSupplier supplier, boolean notify) {
    if (supplier == null) {
      throw new IllegalArgumentException("Null 'supplier' argument.") ;
    }

    this.drawingSupplier = supplier ;
    if (notify) {
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Returns the chart area.</en>
   *
   * @return Rectangle2D <zh>ͼ.</zh>
   * <en>The chart area.</en>
   */
  public Rectangle2D getChartArea() {
    return this.chartArea ;
  }

  /**
   * <zh>ͼ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the chart area and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param area Rectangle2D <zh>ͼ (Ϊnull).</zh>
   * <en>The chart area (null permitted).</en>
   */
  public void setChartArea(Rectangle2D area) {
    if (!EqualsUtil.equals(this.chartArea, area)) {
      this.chartArea = area ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ͼλ.</zh>
   * <en>Returns the legend position.</en>
   *
   * @return RectangleEdge <zh>ͼλ.</zh>
   * <en>The legend position.</en>
   */
  public RectangleEdge getLegendPositon() {
    return this.legendPositon ;
  }

  /**
   * <zh>ͼλ,м{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the legend position and
   * sends a {@link RenderChangeEvent} to all registered listeners.</en>
   *
   * @param positon RectangleEdge <zh>ͼλ.</zh>
   * <en>The legend position.</en>
   */
  public void setLegendPositon(RectangleEdge positon) {
    if (!EqualsUtil.equals(this.legendPositon, positon)) {
      this.legendPositon = positon ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ʊɫ.</zh>
   * <en>Draws a background for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼƷ (Ϊnull).</zh>
   * <en>The chart draw orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawBackground(Rectangle2D dataArea, ChartOrientation orientation,
                             RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    if (this.backgroundPaint == null) {
      return ;
    }

    Paint p = this.backgroundPaint ;
    if (p instanceof GradientPaint) {
      GradientPaint gp = (GradientPaint) p ;
      if (orientation.equals(ChartOrientation.VERTICAL)) {
        p = new GradientPaint((float) dataArea.getCenterX(),
                              (float) dataArea.getMaxY(), gp.getColor1(),
                              (float) dataArea.getCenterX(), (float) dataArea.getMinY(),
                              gp.getColor2()) ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        p = new GradientPaint((float) dataArea.getMinX(),
                              (float) dataArea.getCenterY(), gp.getColor1(),
                              (float) dataArea.getMaxX(), (float) dataArea.getCenterY(),
                              gp.getColor2()) ;
      }
    }

    //ɫͼ
    RectangleTextChartShape shape = new RectangleTextChartShape(dataArea, null) ;
    shape.setDrawBorder(false) ;
    shape.setFillPaint(p) ;
    shape.setFillShape(true) ;

    renderShapes.setBackgroundShape(shape) ;
  }

  /**
   * <zh>ƱͼƬ.</zh>
   * <en>Draws a background image for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawBackgroundImage(Rectangle2D dataArea, RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    if (this.backgroundImage != null) {
      Rectangle2D dest = new Rectangle2D.Double(0.0, 0.0, this.backgroundImage.getWidth(null),
                                                this.backgroundImage.getHeight(null)) ;
      Rectangle2D imageRect = Align.align(dest, dataArea, this.backgroundImageAlignment) ;
      imageRect.setRect(imageRect.getX(), imageRect.getY(), imageRect.getWidth() + 1,
                        imageRect.getHeight() + 1) ;

      //ͼƬͼ
      RectangleImageChartShape shape = new RectangleImageChartShape(imageRect, this.backgroundImage) ;
      shape.setDrawBorder(false) ;
      shape.setFillShape(false) ; //ͼ
      shape.setImageAlpha(this.backgroundImageAlpha) ;

      renderShapes.setBackgroundImageShape(shape) ;
    }

  }

  /**
   * <zh>Ʊ߿.</zh>
   * <en>Draws a border for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawDrawAreaBorder(Rectangle2D dataArea, RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    if (this.renderAreaBorderVisible && this.renderAreaBorderStroke != null
        && this.renderAreaBorderPaint != null) {
      //ͼƬͼ
      RectangleTextChartShape shape = new RectangleTextChartShape(dataArea, null) ;
      shape.setDrawBorder(true) ;
      shape.setBorderPaint(this.renderAreaBorderPaint) ;
      shape.setBorderStroke(this.renderAreaBorderStroke) ;
      shape.setFillShape(false) ;

      renderShapes.setBorderShape(shape) ;
    }
  }

  /**
   * <zh>յݼ仯֪ͨ.</zh>
   * <en>Receives notification of an dataset change event.</en>
   *
   * @param event DatasetChangeEvent <zh>ݼ仯Ķ</zh>
   * <en>Information about the event.</en>
   */
  public void datasetChanged(DatasetChangeEvent event) {
    notifyListeners(new RenderChangeEvent(this)) ;
  }

  /**
   * <zh>õPaintİ.</zh>
   * <en>Helper method for returns the fill paint.</en>
   *
   * @param autoFillPaint Paint <zh>ԶѡPaint.</zh>
   * <en>The auto selected fill paint.</en>
   * @param row int <zh>ݼ.</zh>
   * <en>The dataset row num.</en>
   * @param column int <zh>ݼ.</zh>
   * <en>The dataset column num.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  protected Paint getItemFillPaintHelper(Paint autoFillPaint, int row, int column) {
    Paint result = null ;

    if (isAutoSelectFillPaint() && autoFillPaint != null) {
      result = autoFillPaint ;
    } else {
      result = getItemFillPaint(row, column) ;
    }

    return result ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param label String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String label, Object appendParam) {
    AbstractRectangleTextChartShape shape = new RectangleTextChartShape(rect, label) ;

    return shape ;
  }

  /**
   * <zh>ͼͼμ.</zh>
   * <en>Calculate the legend item shapes.</en>
   *
   * @return List <zh>ͼͼμ(еÿһ
   * {@link com.ijchart.xychart.shape.LegendItemChartShape}) (Ϊnull).</zh>
   * <en>The legend item shapes (elements in this list must be
   * {@link com.ijchart.xychart.shape.LegendItemChartShape}) (never null).</en>
   */
  public List calculateLegendItemShapes() {
    List result = new ArrayList() ;

    int legendCount = getLegendItemCount() ;
    IChartShape[] shapes = getLegendItemShapes(legendCount) ;
    boolean legendRow = getLegendType().equals(LegendType.ROW) ;
    if (legendCount > 0) {
      final int legendWidth = 8, legendHeight = 8 ; //ͼ,߶
      int legendCnt = getLegendItemCount() ;
      Paint[] localFillPaints = getFillPaints(legendCnt) ;
      Paint[] localBorderPaints = getBorderPaints(legendCnt) ;
      Stroke[] localBorderStrokes = getBorderStrokes(legendCnt) ;
      Paint[] oriLocalFillPaints = new Paint[localFillPaints.length] ;
      System.arraycopy(localFillPaints, 0, oriLocalFillPaints, 0, legendCnt) ;

      if (!this.autoSelectFillPaint) {
        for (int i = 0 ; i < legendCount ; i++) {
          Paint tmpPaint = legendRow ? getRowItemFillPaint(i) : getColumnItemFillPaint(i) ;
          if (tmpPaint != null) {
            localFillPaints[i] = tmpPaint ;
          }
        }
      }

      if (!this.autoSelectBorderPaint) {
        for (int i = 0 ; i < legendCount ; i++) {
          Paint tmpPaint = legendRow ? getRowItemBorderPaint(i) : getColumnItemBorderPaint(i) ;
          if (tmpPaint != null) {
            localBorderPaints[i] = tmpPaint ;
          }
        }
      }

      if (!this.autoSelectBorderStroke) {
        for (int i = 0 ; i < legendCount ; i++) {
          Stroke tmpStroke = legendRow ? getRowItemBorderStroke(i) : getColumnItemBorderStroke(i) ;
          if (tmpStroke != null) {
            localBorderStrokes[i] = tmpStroke ;
          }
        }
      }

      String[] labelArray = getLegendItemLabels() ; //ǩ
      for (int i = 0 ; i < legendCount ; i++) {
        IChartShape tmpLegendShape = null ;
        if (shapes != null && i < shapes.length) {
          tmpLegendShape = shapes[i] ;
        } else {
          Rectangle2D rect = new Rectangle2D.Double(0, 0, legendWidth, legendHeight) ;
          tmpLegendShape = new RectangleTextChartShape(rect, "") ;
          tmpLegendShape.setDrawBorder(false) ;
        }

        if (legendRow) {
          Boolean rowFill = isRowItemFill(i) ;
          if (rowFill != null) {
            tmpLegendShape.setFillShape(rowFill.booleanValue()) ;
          } else {
            tmpLegendShape.setFillShape(this.baseItemFill) ;
          }

          GradientPaintTransformType rowTransform = getRowItemFillPaintTransformer(i) ;
          if (rowTransform != null) {
            tmpLegendShape.setGradientPaintTransformType(rowTransform) ;
          } else {
            tmpLegendShape.setGradientPaintTransformType(this.baseItemFillPaintTransformer) ;
          }
        } else {
          Boolean colFill = isColumnItemFill(i) ;
          if (colFill != null) {
            tmpLegendShape.setFillShape(colFill.booleanValue()) ;
          } else {
            tmpLegendShape.setFillShape(this.baseItemFill) ;
          }

          GradientPaintTransformType colTransform = getColumnItemFillPaintTransformer(i) ;
          if (colTransform != null) {
            tmpLegendShape.setGradientPaintTransformType(colTransform) ;
          } else {
            tmpLegendShape.setGradientPaintTransformType(this.baseItemFillPaintTransformer) ;
          }
        }

        boolean isLineAndShape = tmpLegendShape instanceof LineAndShapeChartShape
                                 || this instanceof XYLineAndShapeRender ;
        if (!(tmpLegendShape instanceof PaintScaleLegendChartShape)) {
          if (localFillPaints != null && i < localFillPaints.length) {
            tmpLegendShape.setFillPaint(localFillPaints[i]) ;
          } else {
            Paint fillPaint = legendRow ? getRowItemFillPaint(i) : getColumnItemFillPaint(i) ;
            if (fillPaint != null) {
              tmpLegendShape.setFillPaint(fillPaint) ;
            } else {
              if (this.baseItemFillPaint != null) {
                tmpLegendShape.setFillPaint(this.baseItemFillPaint) ;
              } else {
                tmpLegendShape.setFillPaint(DEFAULT_BASE_ITEM_FILL_PAINT) ;
              }
            }
          }
        }

        if (!isLineAndShape) {
          if (localBorderPaints != null && i < localBorderPaints.length) {
            tmpLegendShape.setBorderPaint(localBorderPaints[i]) ;
          } else {
            Paint borderPaint = legendRow ? getRowItemBorderPaint(i) : getColumnItemBorderPaint(i) ;
            if (borderPaint != null) {
              tmpLegendShape.setBorderPaint(borderPaint) ;
            } else {
              if (this.baseItemBorderPaint != null) {
                tmpLegendShape.setBorderPaint(this.baseItemBorderPaint) ;
              } else {
                tmpLegendShape.setBorderPaint(DEFAULT_BASE_BORDER_PAINT) ;
              }
            }
          }
        } else {
          if (this.autoSelectFillPaint && localFillPaints != null && i < localFillPaints.length) {
            tmpLegendShape.setFillPaint(localFillPaints[i]) ;
          } else {
            boolean setFill = false ;
            Paint fillPaint = legendRow ? getRowItemFillPaint(i) : getColumnItemFillPaint(i) ;
            if (fillPaint != null) {
              setFill = true ;
              tmpLegendShape.setFillPaint(fillPaint) ;
            }

            if (!setFill) {
              if (this.baseItemFillPaint != null) {
                tmpLegendShape.setFillPaint(this.baseItemFillPaint) ;
              } else {
                tmpLegendShape.setFillPaint(DEFAULT_BASE_ITEM_FILL_PAINT) ;
              }
            }
          }

          if (this.autoSelectBorderPaint && localFillPaints != null && i < localFillPaints.length) {
            tmpLegendShape.setBorderPaint(oriLocalFillPaints[i]) ;
          } else {
            Paint borderPaint = legendRow ? getRowItemBorderPaint(i) : getColumnItemBorderPaint(i) ;
            boolean setBorder = false ;
            if (borderPaint != null) {
              setBorder = true ;
              tmpLegendShape.setBorderPaint(borderPaint) ;
            }

            if (!setBorder) {
              if (this.baseItemBorderPaint != null) {
                tmpLegendShape.setBorderPaint(this.baseItemBorderPaint) ;
              } else {
                tmpLegendShape.setBorderPaint(DEFAULT_BASE_BORDER_PAINT) ;
              }
            }
          }
        }

        if (!(tmpLegendShape instanceof PaintScaleLegendChartShape)) {
          if (localBorderStrokes != null && i < localBorderStrokes.length) {
            tmpLegendShape.setBorderStroke(localBorderStrokes[i]) ;
          } else {
            Stroke borderStroke = legendRow ? getRowItemBorderStroke(i)
                                  : getColumnItemBorderStroke(i) ;
            if (borderStroke != null) {
              tmpLegendShape.setBorderStroke(borderStroke) ;
            } else {
              if (this.baseItemBorderStroke != null) {
                tmpLegendShape.setBorderStroke(this.baseItemBorderStroke) ;
              } else {
                tmpLegendShape.setBorderStroke(DEFAULT_BASE_BORDER_STROKE) ;
              }
            }
          }
        }

        String label = null ;
        if (labelArray != null) {
          if (i < labelArray.length) {
            label = labelArray[i] ;
          }
        }
        label = StringUtil.nullToEmpty(label) ;
        label = label.replaceAll("\r", "") ;
        label = label.replaceAll("\n", "") ;

        if (legendRow) {
          Boolean visible = isRowItemVisible(i) ;
          if (visible == null || visible.booleanValue()) {
            result.add(new LegendItemChartShape(tmpLegendShape, label)) ;
          }
        } else {
          Boolean visible = isColumnItemVisible(i) ;
          if (visible == null || visible.booleanValue()) {
            result.add(new LegendItemChartShape(tmpLegendShape, label)) ;
          }
        }

        if (!tmpLegendShape.isDrawBorder() && !tmpLegendShape.isFillShape()) { //⴦
          tmpLegendShape.setFillShape(true) ;
        }
      }
    }

    return result ;
  }

  /**
   * <zh>ͼͼεıǩ.</zh>
   * <en>Returns the label array for legend item shapes.</en>
   *
   * @return String[] <zh>ǩ</zh>
   * <en>The label array.</en>
   */
  protected String[] getLegendItemLabels() {
    LegendType type = getLegendType() ;
    if (type.equals(LegendType.COLUMN)) {
      return getLegendItemLabelsByColumn() ;
    } else if (type.equals(LegendType.ROW)) {
      return getLegendItemLabelsByRow() ;
    }

    return null ;
  }

  /**
   * <zh>ͼͼεıǩ,ǩȡColKeyֵ.</zh>
   * <en>Returns the label array for legend item shapes,the label comes from ColKeys.</en>
   *
   * @return String[] <zh>ǩ</zh>
   * <en>The label array.</en>
   */
  protected String[] getLegendItemLabelsByColumn() {
    int columnCount = this.dataset.getColumnCount() ;

    String[] result = new String[columnCount] ;
    for (int i = 0 ; i < columnCount ; i++) {
      Object colKey = this.dataset.getColKey(i) ;
      result[i] = (colKey == null) ? null : colKey.toString() ;
    }

    return result ;
  }

  /**
   * <zh>ͼͼεıǩ,ǩȡColKeyֵ.</zh>
   * <en>Returns the label array for legend item shapes,the label comes from ColKeys.</en>
   *
   * @return String[] <zh>ǩ</zh>
   * <en>The label array.</en>
   */
  protected String[] getLegendItemLabelsByRow() {
    int rowCount = this.dataset.getRowCount() ;

    String[] result = new String[rowCount] ;
    for (int i = 0 ; i < rowCount ; i++) {
      Object rowKey = this.dataset.getRowKey(i) ;
      result[i] = (rowKey == null) ? null : rowKey.toString() ;
    }

    return result ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Sets the legend type.</en>
   *
   * @param legendType LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public void setCustomLegendType(LegendType legendType) {
    this.customLegendType = legendType ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.COLUMN ;
  }

  /**
   * <zh>ͼĸ.</zh>
   * <en>Returns the count of legends.</en>
   *
   * @return int <zh>ͼĸ.</zh>
   * <en>The count of legends.</en>
   */
  protected int getLegendItemCount() {
    LegendType type = getLegendType() ;
    if (this.customLegendType != null) {
      type = this.customLegendType ;
    }

    if (type.equals(LegendType.COLUMN)) {
      if (this.dataset != null) {
        return this.dataset.getColumnCount() ;
      }
    } else if (type.equals(LegendType.ROW)) {
      if (this.dataset != null) {
        return this.dataset.getRowCount() ;
      }
    }

    return 0 ;
  }

  /**
   * <zh>ÿͼͼ.</zh>
   * <en>Returns the shape array of legend items.</en>
   *
   * @param legendCount int <zh>ͼĸ.</zh>
   * <en>The legend items count.</en>
   * @return IChartShape[] <zh>ͼ.</zh>
   * <en>The shapes array.</en>
   */
  protected IChartShape[] getLegendItemShapes(int legendCount) {
    if (legendCount > 0) {
      final int legendWidth = 8, legendHeight = 8 ; //ͼ,߶
      IChartShape[] shapes = new IChartShape[legendCount] ;

      for (int i = 0 ; i < legendCount ; i++) {
        Rectangle2D rect = new Rectangle2D.Double(0, 0, legendWidth, legendHeight) ;
        shapes[i] = new RectangleTextChartShape(rect, null) ;
      }

      return shapes ;
    }

    return null ;
  }

  /**
   * <zh>עrender仯ļ߶.</zh>
   * <en>Registers an object for notification of changes to the render object.</en>
   *
   * @param listener IRenderChangeListener <zh> (Ϊnull).</zh>
   * <en>The listener (null not permitted).</en>
   */
  public void addChangeListener(IRenderChangeListener listener) {
    listenerList.add(IRenderChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾעļ.</zh>
   * <en>Removes a change listener.</en>
   *
   * @param listener IRenderChangeListener <zh> (Ϊnull).</zh>
   * <en>The listener (null not permitted).</en>
   */
  public void removeChangeListener(IRenderChangeListener listener) {
    listenerList.remove(IRenderChangeListener.class, listener) ;
  }

  /**
   * <zh>еļ.</zh>
   * <en>Deregisters all object for notification of changes to the render.</en>
   */
  public void removeAllChangeListener() {
    Object[] listeners = this.listenerList.getListenerList() ;

    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      this.listenerList.remove(IRenderChangeListener.class,
                               (IRenderChangeListener) listeners[i + 1]) ;
    }
  }

  /**
   * <zh>ָlistenerǷ,ҪڵԪ.</zh>
   * <en>Returns true if the specified object is registered with
   * the {@link IChartShape} as a listener.  Most applications won't need to call this
   * method, it exists mainly for use by unit testing code.</en>
   *
   * @param listener <zh>ҪԵļ.</zh>
   * <en>the listener.</en>
   * @return A boolean.
   */
  public boolean hasListener(IRenderChangeListener listener) {
    return Arrays.asList(this.listenerList.getListenerList()).contains(listener) ;
  }

  /**
   * <zh>֪ͨעļݼ˱仯</zh>
   * <en>Notifies all registered listeners that the dataset has changed.</en>
   *
   * @param event <zh>¼.</zh>
   * <en>contains information about the event that triggered the notification.</en>
   */
  protected void notifyListeners(RenderChangeEvent event) {
    Object[] listeners = this.listenerList.getListenerList() ;
    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      if (listeners[i] == IRenderChangeListener.class) {
        ((IRenderChangeListener) listeners[i + 1]).renderChanged(event) ;
      }
    }
  }

  /**
   * <zh>ͨмݼ˱仯.</zh>
   * <en>Notifies all registered listeners that the dataset has changed.</en>
   */
  protected void fireRenderChanged() {
    notifyListeners(new RenderChangeEvent(this)) ;
  }

  /**
   * <zh>õPaint.</zh>
   * <en>Returns the fill paint array.</en>
   *
   * @param count int <zh>Paint.</zh>
   * <en>The count of paint.</en>
   * @return Paint[] <zh>Paint(Ϊnull).</zh>
   * <en>The paint array (never null).</en>
   */
  protected Paint[] getFillPaints(int count) {
    if (count < 0) {
      throw new IllegalArgumentException("Must be 'count'>=0.") ;
    }

    Paint[] paints = new Paint[count] ;
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < count ; i++) {
      paints[i] = supplier.getNextFillPaint() ;
    }

    return paints ;
  }

  /**
   * <zh>õƱ߿Paint.</zh>
   * <en>Returns the border paint array.</en>
   *
   * @param count int <zh>Paint.</zh>
   * <en>The count of paint.</en>
   * @return Paint[] <zh>Paint(Ϊnull).</zh>
   * <en>The paint array (never null).</en>
   */
  protected Paint[] getBorderPaints(int count) {
    if (count < 0) {
      throw new IllegalArgumentException("Must be 'count'>=0.") ;
    }

    Paint[] paints = new Paint[count] ;
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < count ; i++) {
      paints[i] = supplier.getNextBorderPaint() ;
    }

    return paints ;
  }

  /**
   * <zh>õƱ߿Stroke.</zh>
   * <en>Returns the border Stroke array.</en>
   *
   * @param count int <zh>Stroke.</zh>
   * <en>The count of Stroke.</en>
   * @return Paint[] <zh>Stroke(Ϊnull).</zh>
   * <en>The Stroke array (never null).</en>
   */
  protected Stroke[] getBorderStrokes(int count) {
    if (count < 0) {
      throw new IllegalArgumentException("Must be 'count'>=0.") ;
    }

    Stroke[] strokes = new Stroke[count] ;
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < count ; i++) {
      strokes[i] = supplier.getNextBorderStroke() ;
    }

    return strokes ;
  }

  /**
   * <zh>3Dͼ.</zh>
   * <en>Add 3d shapes.</en>
   *
   * @param renderShapes RenderShapeCollection
   * @param orientation ChartOrientation
   * @param sortUtil Shape3DSortUtil
   */
  protected void add3DShapes(RenderShapeCollection renderShapes, ChartOrientation orientation,
                             Shape3DSortUtil sortUtil) {
    sortUtil.sort() ;
    List sortShapes = sortUtil.getAllShapes() ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      for (int m = sortShapes.size() - 1 ; m >= 0 ; m--) {
        renderShapes.addDataItemShape((IChartShape) sortShapes.get(m)) ;
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      for (int m = 0 ; m < sortShapes.size() ; m++) {
        renderShapes.addDataItemShape((IChartShape) sortShapes.get(m)) ;
      }
    }
  }

  /**
   * <zh>rowcolumnǷ񶼴ڵ0.</zh>
   * <en>Check whether or not the row and column are greater than or equal to 0.</en>
   * @param row int
   * @param column int
   */
  private static void checkRowAndColumn(int row, int column) {
    checkRowValid(row) ;
    checkColumnValid(column) ;
  }

  /**
   * <zh>жrowǷڵ0.</zh>
   * <en>Check whether or not the row is greater than or equal to 0.</en>
   * @param row int <zh>.</zh>
   * <en>The row value.</en>
   */
  private static void checkRowValid(int row) {
    if (row < 0) {
      throw new IllegalArgumentException("Must be 'row'>=0.") ;
    }
  }

  /**
   * <zh>жcolumnǷڵ0.</zh>
   * <en>Check whether or not the column is greater than or equal to 0.</en>
   * @param column int <zh>.</zh>
   * <en>The column value.</en>
   */
  private static void checkColumnValid(int column) {
    if (column < 0) {
      throw new IllegalArgumentException("Must be 'column'>=0.") ;
    }
  }

  /**
   * <zh>кתΪֵ.</zh>
   * <en>Convert row and column to key value.</en>
   *
   * @param row int
   * @param column int
   * @return String
   */
  private static String intToKey(int row, int column) {
    return row + "," + column ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof AbstractRender)) {
      return false ;
    }

    AbstractRender that = (AbstractRender) obj ;
    if (!EqualsUtil.equals(this.itemVisible, that.itemVisible)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnVisible, that.itemColumnVisible)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowVisible, that.itemRowVisible)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemFillPaint, that.itemFillPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnFillPaint, that.itemColumnFillPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowFillPaint, that.itemRowFillPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemFillPaint, that.baseItemFillPaint)) {
      return false ;
    }
    if (this.autoSelectFillPaint != that.autoSelectFillPaint) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemFillPaintTransformer, that.itemFillPaintTransformer)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnFillPaintTransformer, that.itemColumnFillPaintTransformer)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowFillPaintTransformer, that.itemRowFillPaintTransformer)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemFillPaintTransformer, that.baseItemFillPaintTransformer)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemDrawBorder, that.itemDrawBorder)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnDrawBorder, that.itemColumnDrawBorder)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowDrawBorder, that.itemRowDrawBorder)) {
      return false ;
    }
    if (this.baseItemDrawBorder != that.baseItemDrawBorder) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemBorderPaint, that.itemBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnBorderPaint, that.itemColumnBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowBorderPaint, that.itemRowBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemBorderPaint, that.baseItemBorderPaint)) {
      return false ;
    }
    if (this.autoSelectBorderPaint != that.autoSelectBorderPaint) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemBorderStroke, that.itemBorderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnBorderStroke, that.itemColumnBorderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowBorderStroke, that.itemRowBorderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemBorderStroke, that.baseItemBorderStroke)) {
      return false ;
    }
    if (this.autoSelectBorderStroke != that.autoSelectBorderStroke) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelVisible, that.itemLabelVisible)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelVisible, that.itemColumnLabelVisible)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelVisible, that.itemRowLabelVisible)) {
      return false ;
    }
    if (this.baseItemLabelVisible != that.baseItemLabelVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelFont, that.itemLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelFont, that.itemColumnLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelFont, that.itemRowLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelFont, that.baseItemLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelPaint, that.itemLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelPaint, that.itemColumnLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelPaint, that.itemRowLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelPaint, that.baseItemLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelPosition, that.itemLabelPosition)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelPosition, that.itemColumnLabelPosition)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelPosition, that.itemRowLabelPosition)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelPosition, that.baseItemLabelPosition)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelDirection, that.itemLabelDirection)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelDirection, that.itemColumnLabelDirection)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelDirection, that.itemRowLabelDirection)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelDirection, that.baseItemLabelDirection)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelHorizontalAlignment, that.itemLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelHorizontalAlignment,
                           that.itemColumnLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelHorizontalAlignment,
                           that.itemRowLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.negativeItemLabelHorizontalAlignment,
                           that.negativeItemLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.positiveItemLabelHorizontalAlignment,
                           that.positiveItemLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelHorizontalAlignment,
                           that.baseItemLabelHorizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemLabelVerticalAlignment, that.itemLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemColumnLabelVerticalAlignment,
                           that.itemColumnLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.itemRowLabelVerticalAlignment,
                           that.itemRowLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.negativeItemLabelVerticalAlignment,
                           that.negativeItemLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.positiveItemLabelVerticalAlignment,
                           that.positiveItemLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.baseItemLabelVerticalAlignment,
                           that.baseItemLabelVerticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundPaint, that.backgroundPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundImage, that.backgroundImage)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundImageAlignment, that.backgroundImageAlignment)) {
      return false ;
    }
    if (this.backgroundImageAlpha != that.backgroundImageAlpha) {
      return false ;
    }
    if (this.renderAreaBorderVisible != that.renderAreaBorderVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.renderAreaBorderStroke, that.renderAreaBorderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.renderAreaBorderPaint, that.renderAreaBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.labelGenerator, that.labelGenerator)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tooltipGenerator, that.tooltipGenerator)) {
      return false ;
    }
    if (this.generatorUrl != that.generatorUrl) {
      return false ;
    }
    if (this.generatorTooltip != that.generatorTooltip) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataset, that.dataset)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.drawingSupplier, that.drawingSupplier)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.chartArea, that.chartArea)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.legendPositon, that.legendPositon)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = 37 + HashcodeUtil.hashCode(this.itemVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemFillPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnFillPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowFillPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemFillPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.autoSelectFillPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemFillPaintTransformer) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnFillPaintTransformer) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowFillPaintTransformer) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemFillPaintTransformer) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemDrawBorder) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnDrawBorder) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowDrawBorder) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemDrawBorder) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.autoSelectBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.autoSelectBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelPosition) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelPosition) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelPosition) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelPosition) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelDirection) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelDirection) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelDirection) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelDirection) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.negativeItemLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.positiveItemLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelHorizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemColumnLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.itemRowLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.negativeItemLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.positiveItemLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.baseItemLabelVerticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.backgroundPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.backgroundImage) ;
    result = 37 * result + HashcodeUtil.hashCode(this.backgroundImageAlignment) ;
    result = 37 * result + (int) (100 * this.backgroundImageAlpha) ;
    result = 37 * result + HashcodeUtil.hashCode(this.renderAreaBorderVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.renderAreaBorderStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.renderAreaBorderPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.labelGenerator) ;
    result = 37 * result + HashcodeUtil.hashCode(this.tooltipGenerator) ;
    result = 37 * result + HashcodeUtil.hashCode(this.generatorUrl) ;
    result = 37 * result + HashcodeUtil.hashCode(this.generatorTooltip) ;
    result = 37 * result + HashcodeUtil.hashCode(this.dataset) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawingSupplier) ;
    result = 37 * result + HashcodeUtil.hashCode(this.chartArea) ;
    result = 37 * result + HashcodeUtil.hashCode(this.legendPositon) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractRender clone = (AbstractRender)super.clone() ;
    clone.listenerList = new EventListenerList() ;

    return clone ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractRender" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeHashtablePaint(stream, this.itemFillPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemColumnFillPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemRowFillPaint) ;
    SerialUtil.writePaint(stream, this.baseItemFillPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemBorderPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemColumnBorderPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemRowBorderPaint) ;
    SerialUtil.writePaint(stream, this.baseItemBorderPaint) ;

    SerialUtil.writeHashtableStroke(stream, this.itemBorderStroke) ;
    SerialUtil.writeHashtableStroke(stream, this.itemColumnBorderStroke) ;
    SerialUtil.writeHashtableStroke(stream, this.itemRowBorderStroke) ;
    SerialUtil.writeStroke(stream, this.baseItemBorderStroke) ;
    SerialUtil.writeHashtablePaint(stream, this.itemLabelPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemColumnLabelPaint) ;
    SerialUtil.writeHashtablePaint(stream, this.itemRowLabelPaint) ;
    SerialUtil.writePaint(stream, this.baseItemLabelPaint) ;

    SerialUtil.writePaint(stream, this.backgroundPaint) ;
    SerialUtil.writeStroke(stream, this.renderAreaBorderStroke) ;
    SerialUtil.writePaint(stream, this.renderAreaBorderPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.itemFillPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemColumnFillPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemRowFillPaint = SerialUtil.readHashtablePaint(stream) ;
    this.baseItemFillPaint = SerialUtil.readPaint(stream) ;
    this.itemBorderPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemColumnBorderPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemRowBorderPaint = SerialUtil.readHashtablePaint(stream) ;
    this.baseItemBorderPaint = SerialUtil.readPaint(stream) ;

    this.itemBorderStroke = SerialUtil.readHashtableStroke(stream) ;
    this.itemColumnBorderStroke = SerialUtil.readHashtableStroke(stream) ;
    this.itemRowBorderStroke = SerialUtil.readHashtableStroke(stream) ;
    this.baseItemBorderStroke = SerialUtil.readStroke(stream) ;
    this.itemLabelPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemColumnLabelPaint = SerialUtil.readHashtablePaint(stream) ;
    this.itemRowLabelPaint = SerialUtil.readHashtablePaint(stream) ;
    this.baseItemLabelPaint = SerialUtil.readPaint(stream) ;

    this.backgroundPaint = SerialUtil.readPaint(stream) ;
    this.renderAreaBorderStroke = SerialUtil.readStroke(stream) ;
    this.renderAreaBorderPaint = SerialUtil.readPaint(stream) ;

    this.listenerList = new EventListenerList() ;
  }

}
