package com.ijchart.xychart.render.category ;

import java.awt.Shape ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.Hashtable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineAndShapeChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ʒͼͼ.</zh>
 * <en>A render that draws shapes for each data item, and lines between data
 * items (for use with the {@link com.ijchart.xychart.plot.CategoryPlot} class).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LineAndShapeRender extends AbstractCategoryRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1914794249796802410L ;

  /**
   * <zh>ȱʡеǷʾ.</zh>
   * <en>A default flag indicating whether or not lines are drawn between non-null points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_LINE_VISIBLE = true ;

  /**
   * <zh>ȱʡеͼǷʾ.</zh>
   * <en>A default flag indicating whether or not shapes are drawn for all points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_SHAPE_VISIBLE = true ;

  /**
   * <zh>ȱʡеͼǷ.</zh>
   * <en>A default flag indicating whether or not shapes are drawn for all points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_SHAPE_FILLED = true ;

  /**
   * <zh>ÿϵеǷʾ.</zh>
   * <en>A table of flags that control (per series) whether or not lines are visible.</en>
   */
  private Hashtable rowItemLineVisible ;

  /**
   * <zh>еǷʾ.</zh>
   * <en>A flag indicating whether or not lines are drawn between non-null points.</en>
   */
  private boolean baseItemLineVisible ;

  /**
   * <zh>ÿϵеͼǷʾ.</zh>
   * <en>A table of flags that control (per series) whether or not shapes are visible.</en>
   */
  private Hashtable rowItemShapeVisible ;

  /**
   * <zh>еͼǷʾ.</zh>
   * <en>A flag indicating whether or not shapes are drawn for all points.</en>
   */
  private boolean baseItemShapeVisible ;

  /**
   * <zh>ÿϵеͼǷ.</zh>
   * <en>A table of flags that control (per series) whether or not shapes are filled.</en>
   */
  private Hashtable rowItemShapeFilled ;

  /**
   * <zh>еͼǷ.</zh>
   * <en>A flag indicating whether or not shapes are drawn for all points.</en>
   */
  private boolean baseItemShapeFilled ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public LineAndShapeRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.rowItemLineVisible = new Hashtable() ;
    this.baseItemLineVisible = DEFAULT_BASE_ITEM_LINE_VISIBLE ;
    this.rowItemShapeVisible = new Hashtable() ;
    this.baseItemShapeVisible = DEFAULT_BASE_ITEM_SHAPE_VISIBLE ;
    this.rowItemShapeFilled = new Hashtable() ;
    this.baseItemShapeFilled = DEFAULT_BASE_ITEM_SHAPE_FILLED ;

    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
    setBaseItemLabelPosition(TextPosition.OUTSIDE_TOP) ;
  }

  /**
   * <zh>ÿϵеǷʾ.</zh>
   * <en>Returns the flag used to control whether or not the lines for a series
   * are visible.</en>
   *
   * @param series int <zh>һϵе,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag (possibly null).</en>
   */
  public boolean isRowItemLineVisible(int series) {
    Boolean value = (Boolean)this.rowItemLineVisible.get(intToKey(series)) ;
    if (value != null) {
      return value.booleanValue() ;
    }

    return false ;
  }

  /**
   * <zh>һϵеǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'lines visible' flag for a series and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   * @param flag boolean <zh>ǷɼĲֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemLineVisible(int series, boolean flag) {
    this.rowItemLineVisible.put(intToKey(series), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еǷʾĲֵ.</zh>
   * <en>Returns the base 'lines visible' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemLineVisible() {
    return this.baseItemLineVisible ;
  }

  /**
   * <zh>еǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'lines visible' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemLineVisible(boolean flag) {
    if (this.baseItemLineVisible != flag) {
      this.baseItemLineVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ĳϵеͼǷʾĲֵ.</zh>
   * <en>Returns the flag used to control whether or not the shapes for a series are visible.</en>
   *
   * @param series int <zh>ϵек,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public boolean isRowItemShapeVisible(int series) {
    Boolean value = (Boolean)this.rowItemShapeVisible.get(intToKey(series)) ;
    if (value != null) {
      return value.booleanValue() ;
    }

    return false ;
  }

  /**
   * <zh>һϵеͼǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'shapes visible' flag for a series and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent}.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   * @param flag boolean <zh>ǷɼĲֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemShapeVisible(int series, boolean flag) {
    this.rowItemShapeVisible.put(intToKey(series), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еͼǷʾĲֵ.</zh>
   * <en>Returns the base 'shape visible' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemShapeVisible() {
    return this.baseItemShapeVisible ;
  }

  /**
   * <zh>еͼǷʾĲֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'shapes visible' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemShapeVisible(boolean flag) {
    if (this.baseItemShapeVisible != flag) {
      this.baseItemShapeVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ĳϵеͼǷ.</zh>
   * <en>Returns the flag used to control whether or not the shapes for a series
   * are filled.</en>
   *
   * @param series boolean <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>The boolean flag (maybe null).</en>
   */
  public boolean isRowItemShapeFilled(int series) {
    Boolean value = (Boolean)this.rowItemShapeFilled.get(intToKey(series)) ;
    if (value != null) {
      return value.booleanValue() ;
    }

    return false ;
  }

  /**
   * <zh>ĳϵеͼǷ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'shapes filled' flag for a series and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   * @param filled boolean <zh>ֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemShapeFilled(int series, boolean filled) {
    this.rowItemShapeFilled.put(intToKey(series), Boolean.valueOf(filled)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еͼǷĲֵ.</zh>
   * <en>Returns the base 'shape filled' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemShapeFilled() {
    return this.baseItemShapeFilled ;
  }

  /**
   * <zh>еͼǷĲֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'shapes filled' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemShapeFilled(boolean flag) {
    if (this.baseItemShapeFilled != flag) {
      this.baseItemShapeFilled = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>Ƿͼڲ.</zh>
   * <en>Returns whether or not fill shape.</en>
   *
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The boolean.</en>
   */
  public boolean isItemFill(int row, int column) {
    return isShapeFilled(row) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double categoryMargin = (columns == 1) ? 0 : categoryAxis.getCategoryMargin() ;
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - categoryMargin - axisMargin) / columns ;
      double gapSize = 0 ;
      if (columns > 1) {
        gapSize = dataArea.getWidth() * categoryMargin / (columns - 1) ;
      }

      double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin()
                      + 0.5 * categoryWidth ;

      double lastX = Double.NaN ; //һXֵ
      double lastY = Double.NaN ; //һYֵ
      IDrawingSupplier supplier = getDrawingSupplier() ;
      for (int i = 0 ; i < rows ; i++) {
        boolean lineVisible = isLineVisible(i) ;
        boolean shapeVisible = isShapeVisible(i) ;

        Shape shape = null ;
        if (shapeVisible) {
          shape = supplier.getNextShape() ;
        }

        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          double x = startX + j * (categoryWidth + gapSize) ;

          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double y = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              lastX = Double.NaN ;
              lastY = Double.NaN ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            Line2D line = null ;
            if (j == 0) {
              line = null ;
            } else {
              if (!Double.isNaN(lastX) && !Double.isNaN(lastY) && lineVisible) {
                line = new Line2D.Double(lastX - x, lastY - y, 0, 0) ;
              }
            }

            LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(x, y, shape, line) ;
            IChartShape chartShape = createItemShape(orientation, valueAxis, i, j, null,
                                                     fillPaints[i], borderPaints[i],
                                                     borderStrokes[i], alpha, lineChartShape) ;
            renderShapes.addDataItemShape(chartShape) ;

            lastX = x ;
            lastY = y ;
          } else {
            lastX = Double.NaN ;
            lastY = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double categoryMargin = (columns == 1) ? 0 : categoryAxis.getCategoryMargin() ;
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - categoryMargin - axisMargin) / columns ;
      double gapSize = 0 ;
      if (columns > 1) {
        gapSize = dataArea.getHeight() * categoryMargin / (columns - 1) ;
      }

      double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin()
                      + 0.5 * categoryWidth ;

      double lastX = Double.NaN ; //һXֵ
      double lastY = Double.NaN ; //һYֵ
      IDrawingSupplier supplier = getDrawingSupplier() ;
      for (int i = 0 ; i < rows ; i++) {
        boolean lineVisible = isLineVisible(i) ;
        boolean shapeVisible = isShapeVisible(i) ;

        Shape shape = null ;
        if (shapeVisible) {
          shape = supplier.getNextShape() ;
        }

        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          double y = startY + j * (categoryWidth + gapSize) ;

          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double x = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              lastX = Double.NaN ;
              lastY = Double.NaN ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            Line2D line = null ;
            if (j == 0) {
              line = null ;
            } else {
              if (!Double.isNaN(lastX) && !Double.isNaN(lastY) && lineVisible) {
                line = new Line2D.Double(lastX - x, lastY - y, 0, 0) ;
              }
            }

            LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(x, y, shape, line) ;
            IChartShape chartShape = createItemShape(orientation, valueAxis, i, j, null,
                                                     fillPaints[i], borderPaints[i],
                                                     borderStrokes[i], alpha, lineChartShape) ;
            renderShapes.addDataItemShape(chartShape) ;

            lastX = x ;
            lastY = y ;
          } else {
            lastX = Double.NaN ;
            lastY = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }

  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ÿͼͼ.</zh>
   * <en>Returns the shape array of legend items.</en>
   *
   * @param legendCount int <zh>ͼĸ.</zh>
   * <en>The legend items count.</en>
   * @return IChartShape[] <zh>ͼ.</zh>
   * <en>The shapes array.</en>
   */
  protected IChartShape[] getLegendItemShapes(int legendCount) {
    if (legendCount > 0) {
      IDrawingSupplier supplier = getDrawingSupplier() ;

      IChartShape[] shapes = new IChartShape[legendCount] ;
      for (int i = 0 ; i < legendCount ; i++) {
        Shape shape = supplier.getNextShape() ;
        if (!isShapeVisible(i)) {
          shape = LineAndShapeChartShape.ZERO_RECT ;
        }

        Rectangle2D bound = shape.getBounds() ;
        Line2D line = null ;
        if (isLineVisible(i)) {
          line = new Line2D.Double(bound.getMinX() - 6, 0, bound.getMaxX() + 6, 0) ;
        }

        LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(0, 0, shape, line) ;
        lineChartShape.setDrawBorder(isItemDrawBorder(i, 0)) ;
        lineChartShape.setFillShape(isItemFill(i, 0)) ;
        lineChartShape.setGradientPaintTransformType(getItemFillPaintTransformer(i, 0)) ;

        if (!isAutoSelectBorderPaint()) {
          lineChartShape.setBorderPaint(getItemBorderPaint(i, 0)) ;
        }

        if (!isAutoSelectBorderStroke()) {
          lineChartShape.setBorderStroke(getItemBorderStroke(i, 0)) ;
        }

        shapes[i] = lineChartShape ;
      }

      return shapes ;
    }

    return null ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    if (appendParam instanceof LineAndShapeChartShape) {
      LineAndShapeChartShape lineAndShape = (LineAndShapeChartShape) appendParam ;
      lineAndShape.setText(text) ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>һintֵתΪһ.</zh>
   * <en>Convert a int value to an object.</en>
   *
   * @param value int <zh>ֵ.</zh>
   * <en>A int.</en>
   * @return Object <zh>һ (never null).</zh>
   * <en>An object.</en>
   */
  private static Object intToKey(int value) {
    return String.valueOf(value) ;
  }

  /**
   * <zh>һϵеǷɼ.</zh>
   * <en>Calculate whether or not the line is visible.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isLineVisible(int series) {
    Boolean visible = (Boolean)this.rowItemLineVisible.get(intToKey(series)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemLineVisible ;
    }
  }

  /**
   * <zh>һϵеͼǷɼ.</zh>
   * <en>Calculate whether or not the shape is visible.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isShapeVisible(int series) {
    Boolean visible = (Boolean)this.rowItemShapeVisible.get(intToKey(series)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemShapeVisible ;
    }
  }

  /**
   * <zh>һϵеͼǷ.</zh>
   * <en>Calculate whether or not the shape is filled.</en>
   *
   * @param series int <zh>ϵеֵ,0ʼ.</zh>
   * <en>The series index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isShapeFilled(int series) {
    Boolean visible = (Boolean)this.rowItemShapeFilled.get(intToKey(series)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemShapeFilled ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof LineAndShapeRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    LineAndShapeRender that = (LineAndShapeRender) obj ;
    if (!EqualsUtil.equals(this.rowItemLineVisible, that.rowItemLineVisible)) {
      return false ;
    }
    if (this.baseItemLineVisible != that.baseItemLineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemShapeVisible, that.rowItemShapeVisible)) {
      return false ;
    }
    if (this.baseItemShapeVisible != that.baseItemShapeVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemShapeFilled, that.rowItemShapeFilled)) {
      return false ;
    }
    if (this.baseItemShapeFilled != that.baseItemShapeFilled) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemLineVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemLineVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemShapeVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemShapeVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemShapeFilled) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemShapeFilled) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "LineAndShapeRender" ;
  }
}
