package com.ijchart.xychart.render.category ;

import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.StatisticalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.StatisticalLineChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Statistical()ʽ״ͼ.</zh>
 * <en>Draw statistical bar shapes.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StatisticalLineRender extends AbstractCategoryRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2691860229484187307L ;

  /**
   * <zh>ȱʡĹε.</zh>
   * <en>The default maximum width of I-Shaped shape.</en>
   */
  public static final double DEFAULT_ISHAPED_MAX_WIDTH = 10 ;

  /**
   * <zh>ε.</zh>
   * <en>The maximum width of I-Shaped shape.</en>
   */
  private double iShapedMaxWidth ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public StatisticalLineRender(IMatrixDataset dataset) {
    super(dataset) ;
    this.iShapedMaxWidth = DEFAULT_ISHAPED_MAX_WIDTH ;

    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
  }

  /**
   * <zh>عε.</zh>
   * <en>Returns the maximum width of I-Shaped shape.</en>
   *
   * @return double <zh>.</zh>
   * <en>The maximum width.</en>
   */
  public double getIShapedMaxWidth() {
    return this.iShapedMaxWidth ;
  }

  /**
   * <zh>ùε,
   * м{@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the maximum width of I-Shaped shape and sends a
   *  {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param maxWidth double <zh>.</zh>
   * <en>The maximum width.</en>
   */
  public void setIShapedMaxWidth(double maxWidth) {
    if (this.iShapedMaxWidth != maxWidth) {
      this.iShapedMaxWidth = maxWidth ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    double basePosition = valueAxis.valueToJava2D(dataArea, valueAxisEdge, 0, false) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //״ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - axisMargin) / columns ; //ÿĿ

      for (int i = 0 ; i < rows ; i++) {
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof StatisticalChartData) {
            StatisticalChartData statisData = (StatisticalChartData) data ;
            double mean = statisData.getMeanValue() ;
            double y = valueAxis.valueToJava2D(dataArea, valueAxisEdge, mean, false) ;
            //ֵ᷶Χֵ
            if (y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
            } else {
              double x11 = startX + categoryWidth * (0.5 + j) ;
              //β
              double valueDelta = statisData.getStandDeviationValue() ;
              double lowValue = valueAxis.valueToJava2D(dataArea, valueAxisEdge,
                                                        mean - valueDelta, false) ;
              double highValue = valueAxis.valueToJava2D(dataArea, valueAxisEdge,
                                                         mean + valueDelta, false) ;
              if (lowValue == AbstractValueAxis.INVALID_COORDINATE_VALUE
                  || highValue == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                renderShapes.addDataItemShape(null) ;
                continue ;
              }

              double minValue = Math.max(lowValue, highValue) ;

              double iShapedWidth = Math.min(categoryWidth, iShapedMaxWidth) ;
              if (iShapedWidth == categoryWidth && iShapedWidth >= 4) { //ȱ֤Сͼ>=2
                iShapedWidth -= 2 ;
              }
              Rectangle2D rect = new Rectangle2D.Double(x11 - 0.5 * iShapedWidth,
                                                        minValue, iShapedWidth,
                                                        Math.abs(lowValue - highValue)) ;
              double iShapedX11 = x11 - 0.5 * iShapedWidth ;

              StatisticalLineChartShape statisticalShape =
                new StatisticalLineChartShape(rect, null, iShapedX11, minValue, iShapedWidth,
                                              Math.abs(highValue - lowValue), null, null) ;

              if (j + 1 <= columns - 1) { //һԪ
                IChartData nextData = dataset.getData(i, j + 1) ;
                if (nextData != null) {
                  StatisticalChartData nextStatisData = (StatisticalChartData) nextData ;
                  double nextMean = nextStatisData.getMeanValue() ;
                  double nextY = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextMean, false) ;
                  //ֵ᷶Χֵ
                  if (nextY != AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                    statisticalShape.setLineStartPoint(new Point2D.Double(x11, y)) ;
                    statisticalShape.setLineEndPoint(new Point2D.Double(x11 + categoryWidth, nextY)) ;
                  }
                }
              }

              IChartShape shape = createItemShape(orientation, valueAxis, i, j, rect, fillPaints[i],
                                                  borderPaints[i], borderStrokes[i], alpha,
                                                  statisticalShape) ;
              renderShapes.addDataItemShape(shape) ;
            }

          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - axisMargin) / columns ; //ÿĿ
      for (int i = 0 ; i < rows ; i++) {
        double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof StatisticalChartData) {
            StatisticalChartData statisData = (StatisticalChartData) data ;
            double mean = statisData.getMeanValue() ;
            double x = valueAxis.valueToJava2D(dataArea, valueAxisEdge, mean, false) ;
            //ֵ᷶Χֵ
            if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
            } else {
              //ӹ
              double valueDelta = statisData.getStandDeviationValue() ;
              double lowValue = valueAxis.valueToJava2D(dataArea, valueAxisEdge,
                                                        mean - valueDelta, false) ;
              double highValue = valueAxis.valueToJava2D(dataArea, valueAxisEdge,
                                                         mean + valueDelta, false) ;
              if (lowValue == AbstractValueAxis.INVALID_COORDINATE_VALUE
                  || highValue == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                renderShapes.addDataItemShape(null) ;
                continue ;
              }
              double minValue = Math.min(lowValue, highValue) ;

              double iShapedWidth = Math.min(categoryWidth, iShapedMaxWidth) ;
              if (iShapedWidth == categoryWidth && iShapedWidth >= 4) { //ȱ֤Сͼ>=2
                iShapedWidth -= 2 ;
              }

              double y11 = startY + categoryWidth * (0.5 + j) ;
              double iShapedY11 = y11 - 0.5 * iShapedWidth ;
              double iShapedHeight = Math.abs(highValue - lowValue) ;
              Rectangle2D rect = new Rectangle2D.Double(minValue,
                                                        y11 - 0.5 * iShapedWidth,
                                                        iShapedWidth, iShapedHeight) ;
              StatisticalLineChartShape statisticalShape =
                new StatisticalLineChartShape(rect, null, minValue, iShapedY11, iShapedWidth,
                                              iShapedHeight, null, null) ;
              statisticalShape.setRotate(IAngle.ANGLE_DOWN_90) ;

              if (j + 1 <= columns - 1) { //һԪ
                IChartData nextData = dataset.getData(i, j + 1) ;
                if (nextData != null) {
                  StatisticalChartData nextStatisData = (StatisticalChartData) nextData ;
                  double nextMean = nextStatisData.getMeanValue() ;
                  double nextX = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextMean, false) ;
                  //ֵ᷶Χֵ
                  if (nextX != AbstractValueAxis.INVALID_COORDINATE_VALUE) {
                    double startPointX = minValue + 0.5 * iShapedWidth ;
                    double startPointY = y11 - 0.5 * iShapedWidth - 0.5 * iShapedHeight ;
                    Point2D startPoint = new Point2D.Double(startPointX, startPointY) ;
                    statisticalShape.setLineStartPoint(startPoint) ;
                    double endPointX = startPointX + categoryWidth ;
                    double endPointY = startPointY - (nextX - x) ;
                    statisticalShape.setLineEndPoint(new Point2D.Double(endPointX, endPointY)) ;
                  }
                }
              }

              IChartShape shape = createItemShape(orientation, valueAxis, i, j, rect, fillPaints[i],
                                                  borderPaints[i], borderStrokes[i], alpha,
                                                  statisticalShape) ;
              renderShapes.addDataItemShape(shape) ;
            }
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }

  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof StatisticalLineRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StatisticalLineRender that = (StatisticalLineRender) obj ;
    if (this.iShapedMaxWidth != that.iShapedMaxWidth) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() + (int)this.iShapedMaxWidth ;

    return result ;

  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[iShapedMaxWidth=" + iShapedMaxWidth + "]" ;
  }

}
