package com.ijchart.xychart.render.category.outlier ;

import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Iterator ;
import java.util.List ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ɢ㼯.ÿϺһӦ,ɢ㱻ֳɶ,ÿһɢ,ȡǷص,
 * صĵ㱻һ.</zh>
 * <en>A collection of outlier lists for a box and whisker plot. Each collection is
 * associated with a single box and whisker entity.
 * Outliers are grouped in lists for each entity. Lists contain
 * one or more outliers, determined by whether overlaps have
 * occurred. Overlapping outliers are grouped in the same list.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class OutlierListCollection implements Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1049224604434311852L ;

  /**
   * <zh>ɢ㼯.</zh>
   * <en>Storage for the outlier lists.</en>
   */
  private List outlierLists ;

  /**
   * <zh>ɢĸ2м䲿,ΪԶֵ.</zh>
   * <en>Unbelievably, outliers which are more than 2 * interquartile range are
   * called far outs...  See Tukey EDA  (a classic one of a kind...).</en>
   */
  private boolean highFarOut = false ;

  /**
   * <zh>Ƿ񼯺Զֵֵ.</zh>
   * <en>A flag that indicates whether or not the collection contains low far out values.</en>
   */
  private boolean lowFarOut = false ;

  /**
   * <zh>.</zh>
   * <en>Creates a new empty collection.</en>
   */
  public OutlierListCollection() {
    this.outlierLists = new ArrayList() ;
  }

  /**
   * <zh>ɢĸ2м䲿,ΪԶֵ.ڷظòֵ.</zh>
   * <en>A flag to indicate the presence of one or more far out values at the
   * top end of the range.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isHighFarOut() {
    return this.highFarOut ;
  }

  /**
   * <zh>ɢĸ2м䲿,ΪԶֵ.øòֵ.</zh>
   * <en>Sets the flag that indicates the presence of one or more far out values
   * at the top end of the range.</en>
   *
   * @param farOut boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setHighFarOut(boolean farOut) {
    this.highFarOut = farOut ;
  }

  /**
   * <zh>Ƿ񼯺Զֵֵ.ڷظòֵ.</zh>
   * <en>A flag to indicate the presence of one or more far out values at the
   * bottom end of the range.</en>
   *
   * @return <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isLowFarOut() {
    return this.lowFarOut ;
  }

  /**
   * <zh>Ƿ񼯺Զֵֵ.øòֵ.</zh>
   * <en>Sets the flag that indicates the presence of one or more far out values
   * at the bottom end of the range.</en>
   *
   * @param farOut boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setLowFarOut(boolean farOut) {
    this.lowFarOut = farOut ;
  }

  /**
   * <zh>һOutlier.ûкڵļص,ᴴµļ;׷ӵԭϵĺ.</zh>
   * <en>Appends the specified element as a new OutlierList to the
   * end of this list if it does not overlap an outlier in an existing list.
   * If it does overlap, it is appended to the outlier list which it overlaps
   * and that list is updated.</en>
   *
   * @param outlier Outlier <zh>ҪӵĶ.</zh>
   * <en>Element to be appended to this list.</en>
   *
   * @return true <zh>ͬCollection.add()ķֵ.</zh>
   * <en>As per the general contract of Collection.add().</en>
   */
  public boolean add(Outlier outlier) {
    if (this.outlierLists.isEmpty()) {
      return this.outlierLists.add(new OutlierList(outlier)) ;
    } else {
      boolean updated = false ;
      for (Iterator iterator = this.outlierLists.iterator() ;
                               iterator.hasNext() ; ) {
        OutlierList list = (OutlierList) iterator.next() ;
        if (list.isOverlapped(outlier)) {
          updated = updateOutlierList(list, outlier) ;
        }
      }
      if (!updated) {
        updated = this.outlierLists.add(new OutlierList(outlier)) ;
      }
      return updated ;
    }
  }

  /**
   * <zh>Iterator.</zh>
   * <en>Returns an iterator for the outlier lists.</en>
   *
   * @return Iterator <zh>.</zh>
   * <en>An iterator.</en>
   */
  public Iterator iterator() {
    return this.outlierLists.iterator() ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Updates the outlier list by adding the outlier to the end of the list and
   * setting the averaged outlier to the average x and y coordinnate values
   * of the outliers in the list.</en>
   *
   * @param list OutlierList <zh>OutlierList.</zh>
   * <en>The outlier list to be updated.</en>
   * @param outlier Outlier <zh>ҪӵOutlier.</zh>
   * <en>The outlier to be added.</en>
   *
   * @return true (as per the general contract of Collection.add).
   */
  private static boolean updateOutlierList(OutlierList list, Outlier outlier) {
    boolean result = false ;
    result = list.add(outlier) ;
    list.updateAveragedOutlier() ;
    list.setMultiple(true) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof OutlierListCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    OutlierListCollection that = (OutlierListCollection) obj ;
    if (!EqualsUtil.equals(this.outlierLists, that.outlierLists)) {
      return false ;
    }
    if (this.highFarOut != that.highFarOut) {
      return false ;
    }
    if (this.lowFarOut != that.lowFarOut) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() + HashcodeUtil.hashCode(this.outlierLists) ;
    result = 37 * result + HashcodeUtil.hashCode(this.highFarOut) ;
    result = 37 * result + HashcodeUtil.hashCode(this.lowFarOut) ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a textual representation of the outlier.</en>
   *
   * @return A String representing the outlier.
   */
  public String toString() {
    return "[outlierListCollections size="
      + ((this.outlierLists == null) ? "null" : String.valueOf(this.outlierLists.size()))
      + ",highFarOut=" + this.highFarOut
      + ",lowFarOut=" + this.lowFarOut + "]" ;
  }

}
