package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.axis.value.SymbolValueAxis ;
import com.ijchart.xychart.axis.value.ZNumberValueAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.data.XYZNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.paint.IPaintScale ;
import com.ijchart.xychart.render.paint.LookupPaintScale ;
import com.ijchart.xychart.tickunit.DoubleTickUnit ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڷͼͼͼ.</zh>
 * <en>A legend shape for block chart.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PaintScaleLegendChartShape extends AbstractChartShape implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -252506601961153884L ;

  /**
   * <zh>ȱʡķֿ.</zh>
   * <en>The default number of divisions.</en>
   */
  public static final int DEFAULT_DIVISION = 300 ;

  /**
   * <zh>״ͼεĿ.</zh>
   * <en>The width of strip.</en>
   */
  public static final double DEFAULT_STRIP_WIDTH = 15.0 ;

  /**
   * <zh>ȱʡǷ״ͼεı߿.</zh>
   * <en>A default boolean flag controls whether or not draw strip border.</en>
   */
  public static final boolean DEFAULT_DRAW_STRIP_BORDER = true ;

  /**
   * <zh>ȱʡĻ״ͼεı߿paint.</zh>
   * <en>The default paint used to draw strip border.</en>
   */
  public static final Paint DEFAULT_STRIP_BORDER_PAINT = Color.gray ;

  /**
   * <zh>ȱʡĻ״ͼεı߿stroke.</zh>
   * <en>The default stroke used to draw strip border.</en>
   */
  public static final Stroke DEFAULT_STRIP_BORDER_STROKE = new BasicStroke(0.5f) ;

  /**
   * <zh>ȱʡ״ͼεƫ.</zh>
   * <en>The default offset between the axis and the strip.</en>
   */
  public static final double DEFAULT_AXIS_OFFSET = 0.0 ;

  /**
   * <zh>{@link IPaintScale}.</zh>
   * <en>An {@link IPaintScale} object.</en>
   */
  private IPaintScale paintScale ;

  /**
   * <zh>ͼλ.</zh>
   * <en>The position of legend.</en>
   */
  private RectangleEdge position ;

  /**
   * <zh>.</zh>
   * <en>The drawing area.</en>
   */
  private Rectangle2D drawArea ;

  /**
   * <zh>ֿ.</zh>
   * <en>The number of divisions.</en>
   */
  private int divisions ;

  /**
   * <zh>״ͼεĿ.</zh>
   * <en>The width of strip.</en>
   */
  private double stripWidth ;

  /**
   * <zh>Ƿ״ͼεı߿.</zh>
   * <en>A boolean flag controls whether or not draw strip border.</en>
   */
  private boolean drawStripBorder ;

  /**
   * <zh>״ͼεı߿paint.</zh>
   * <en>The paint used to draw strip border.</en>
   */
  private transient Paint stripBorderPaint ;

  /**
   * <zh>״ͼεı߿stroke.</zh>
   * <en>The stroke used to draw strip border.</en>
   */
  private transient Stroke stripBorderStroke ;

  /**
   * <zh>״ͼεƫ.</zh>
   * <en>The offset between the axis and the strip.</en>
   */
  private double axisOffset ;

  /**
   * <zh>ͼμ.</zh>
   * <en>The axis shapes collection.</en>
   */
  private AxisShapeCollection axisShapes ;

  /**
   * <zh>ǩ.</zh>
   * <en>The label.</en>
   */
  private String label ;

  /**
   * <zh>value.</zh>
   * <en>The value axis.</en>
   */
  private NumberValueAxis valueAxis ;

  /**
   * <zh>ݼ.</zh>
   * <en>The dataset.</en>
   */
  private IMatrixDataset dataset ;

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param paintScale IPaintScale <zh>ʵֽӿ{@link IPaintScale}Ķ (Ϊnull).</zh>
   * <en>An object implements {@link IPaintScale} interface.</en>
   * @param label String <zh>ǩ.</zh>
   * <en>The label.</en>
   * @param position RectangleEdge <zh>ͼλ (Ϊnull).</zh>
   * <en>The position of legend (null not permitted).</en>
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The drawing area (null not permitted).</en>
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public PaintScaleLegendChartShape(IPaintScale paintScale, String label,
                                    RectangleEdge position, Rectangle2D drawArea,
                                    IMatrixDataset dataset) {
    if (paintScale == null) {
      throw new IllegalArgumentException("Null 'paintScale' argument") ;
    }
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument") ;
    }
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument") ;
    }
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    this.paintScale = paintScale ;
    this.label = label ;
    this.position = position ;
    this.drawArea = drawArea ;
    this.divisions = DEFAULT_DIVISION ;

    this.stripWidth = DEFAULT_STRIP_WIDTH ;
    this.drawStripBorder = DEFAULT_DRAW_STRIP_BORDER ;
    this.stripBorderPaint = DEFAULT_STRIP_BORDER_PAINT ;
    this.stripBorderStroke = DEFAULT_STRIP_BORDER_STROKE ;
    this.axisOffset = DEFAULT_AXIS_OFFSET ;

    this.dataset = dataset ;
    createValueAxis() ;
  }

  /**
   * <zh>{@link IPaintScale}.</zh>
   * <en>Returns the {@link IPaintScale} object.</en>
   *
   * @return IPaintScale <zh>{@link IPaintScale} (Ϊnull).</zh>
   * <en>An {@link IPaintScale} object (never null).</en>
   */
  public IPaintScale getPaintScale() {
    return this.paintScale ;
  }

  /**
   * <zh>{@link IPaintScale},м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the {@link IPaintScale} object and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paintScale IPaintScale <zh>{@link IPaintScale} (Ϊnull).</zh>
   * <en>The {@link IPaintScale} object (null not permitted).</en>
   */
  public void setPaintScale(IPaintScale paintScale) {
    if (paintScale == null) {
      throw new IllegalArgumentException("Null 'paintScale' argument") ;
    }

    this.paintScale = paintScale ;
    createValueAxis() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼλ.</zh>
   * <en>Returns the position of legend.</en>
   *
   * @return RectangleEdge <zh>ͼλ (Ϊnull).</zh>
   * <en>The position of legend (never null).</en>
   */
  public RectangleEdge getPosition() {
    return this.position ;
  }

  /**
   * <zh>ͼλ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the position of legend and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param position RectangleEdge <zh>ͼλ (Ϊnull).</zh>
   * <en>The position of legend.</en>
   */
  public void setPosition(RectangleEdge position) {
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument") ;
    }

    if (!this.position.equals(position)) {
      this.position = position ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػ.</zh>
   * <en>Returns the drawing area.</en>
   *
   * @return Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (never null).</en>
   */
  public Rectangle2D getDrawArea() {
    return this.drawArea ;
  }

  /**
   * <zh>û,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the drawing area and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void setDrawArea(Rectangle2D drawArea) {
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument") ;
    }

    this.drawArea = drawArea ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>طֿ.</zh>
   * <en>Returns the number of divisions.</en>
   *
   * @return int <zh>.</zh>
   * <en>The count.</en>
   */
  public int getDivisions() {
    return this.divisions ;
  }

  /**
   * <zh>÷ֿ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the number of divisions and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param divisions int <zh>,0.</zh>
   * <en>The count and must be >0.</en>
   */
  public void setDivisions(int divisions) {
    if (divisions <= 0) {
      throw new IllegalArgumentException("Argument 'divisions' must be >=0.") ;
    }

    if (this.divisions != divisions) {
      this.divisions = divisions ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>״ͼεĿ.</zh>
   * <en>Returns the width of strip.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  public double getStripWidth() {
    return this.stripWidth ;
  }

  /**
   * <zh>״ͼεĿ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the width of strip and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param width double <zh>.</zh>
   * <en>The width.</en>
   */
  public void setStripWidth(double width) {
    if (width <= 0) {
      throw new IllegalArgumentException("Argument 'width' must be >=0.") ;
    }

    if (this.stripWidth != width) {
      this.stripWidth = width ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Ƿ״ͼεı߿.</zh>
   * <en>Returns the boolean flag controls whether or not draw strip border.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawStripBorder() {
    return this.drawStripBorder ;
  }

  /**
   * <zh>Ƿ״ͼεı߿,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the boolean flag controls whether or not draw strip border and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawStripBorder(boolean flag) {
    if (this.drawStripBorder != flag) {
      this.drawStripBorder = flag ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػ״ͼεı߿paint.</zh>
   * <en>Returns the paint used to draw strip border.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getStripBorderPaint() {
    return this.stripBorderPaint ;
  }

  /**
   * <zh>û״ͼεı߿paint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw strip border and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setStripBorderPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.stripBorderPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ػ״ͼεı߿stroke.</zh>
   * <en>Returns the stroke used to draw strip border.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getStripBorderStroke() {
    return this.stripBorderStroke ;
  }

  /**
   * <zh>û״ͼεı߿stroke,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw strip border and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setStripBorderStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.stripBorderStroke = stroke ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>״ͼεƫ.</zh>
   * <en>Returns the offset between the axis and the strip.</en>
   *
   * @return double <zh>ƫ.</zh>
   * <en>The offset.</en>
   */
  public double getAxisOffset() {
    return this.axisOffset ;
  }

  /**
   * <zh>״ͼεƫ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the offset between the axis and the strip and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param offset double <zh>ƫ.</zh>
   * <en>The offset.</en>
   */
  public void setAxisOffset(double offset) {
    if (this.axisOffset != offset) {
      this.axisOffset = offset ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼμ.</zh>
   * <en>Returns the axis shapes collection.</en>
   *
   * @return AxisShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>The axis shapes collection (maybe null).</en>
   */
  public AxisShapeCollection getAxisShapes() {
    return this.axisShapes ;
  }

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the label.</en>
   *
   * @return String <zh>ǩ (Ϊnull).</zh>
   * <en>The label (maybe null).</en>
   */
  public String getLabel() {
    return this.label ;
  }

  /**
   * <zh>ñǩ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the label of the strip and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param label String <zh>ǩ (Ϊnull).</zh>
   * <en>The label (null permitted).</en>
   */
  public void setLabel(String label) {
    if (!EqualsUtil.equals(this.label, label)) {
      this.label = label ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Returns the dataset.</en>
   *
   * @return IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (never null).</en>
   */
  public IMatrixDataset getDataset() {
    return this.dataset ;
  }

  /**
   * <zh>ݼ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the dataset and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public void setDataset(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    this.dataset = dataset ;
    createValueAxis() ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>.</zh>
   * <en>Create a value axis.</en>
   */
  private void createValueAxis() {
    if (this.paintScale instanceof LookupPaintScale) {
      LookupPaintScale lookupScale = (LookupPaintScale)this.paintScale ;
      String[] symbols = lookupScale.getLegendSymbols() ;
      if (symbols == null || symbols.length == 0) {
        this.valueAxis = new ZNumberValueAxis(this.label, this.dataset, false) ;
      } else {
        DefaultMatrixDataset newDataset = new DefaultMatrixDataset() ;
        int rows = this.dataset.getRowCount() ;
        for (int i = 0 ; i < rows ; i++) {
          newDataset.addRowData() ;
          int currRowCols = dataset.getColumnCount(i) ;

          for (int j = 0 ; j < currRowCols ; j++) {
            Object obj = this.dataset.getData(i, j) ;
            if (obj != null && obj instanceof XYZNumberChartData) {
              XYZNumberChartData data = (XYZNumberChartData) obj ;
              newDataset.addData(i, new NumberChartData(data.getZData())) ;
            } else {
              newDataset.addData(i, null) ;
            }
          }
        }

        NumberValueAxis numberAxis = new NumberValueAxis(newDataset) ;
        numberAxis.setTickMarkVisible(false) ;
        this.valueAxis = new SymbolValueAxis(this.label, numberAxis, symbols) ;
      }
    } else {
      this.valueAxis = new ZNumberValueAxis(this.label, this.dataset, false) ;
    }

    this.valueAxis.setMaximumCategoryLabelWidthRatio(1.0F) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull).</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    double drawAreaWidth = this.drawArea.getWidth() ;
    double drawAreaHeight = this.drawArea.getHeight() ;
    Rectangle2D newArea = new Rectangle2D.Double(0, 0, drawAreaWidth, drawAreaHeight) ;
    g2.translate(this.drawArea.getX(), this.drawArea.getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    drawBackground(g2, newArea) ;
    drawBorder(g2, newArea) ;

    this.axisShapes = new AxisShapeCollection() ;
    this.axisShapes.setDrawArea(this.drawArea) ;
    NumberRange range = this.valueAxis.getZoomDatasetRange() ;
    if (range == null) {
      range = this.valueAxis.getDatasetRange() ;
    }

    double base = range.getLowerBound() ;
    double increment = range.getLength() / this.divisions ;

    if (RectangleEdge.isTopOrBottom(this.position)) {
      double oneWidth = drawAreaWidth / this.divisions ;
      double barHeight = Math.min(drawAreaHeight, this.stripWidth) ;
      if (this.position.equals(RectangleEdge.TOP)) {
        for (int i = 0 ; i < this.divisions ; i++) {
          double currX = i * oneWidth ;
          Paint p = this.paintScale.getPaint(base + i * increment) ;
          Rectangle2D rect = new Rectangle2D.Double(currX - 0.5, 0, oneWidth + 1, barHeight) ;
          g2.setPaint(p) ;
          g2.fill(rect) ;
        }

        if (this.drawStripBorder) {
          g2.setPaint(this.stripBorderPaint) ;
          g2.setStroke(this.stripBorderStroke) ;
          g2.draw(new Rectangle2D.Double(0, 0, drawAreaWidth, barHeight)) ;
        }

        this.axisShapes.setEdge(RectangleEdge.BOTTOM) ;
        double axisHeight = valueAxis.draw(g2, drawArea, drawArea, RectangleEdge.BOTTOM,
                                           this.axisShapes, true) ;

        double translateY = drawAreaHeight - barHeight - axisHeight - this.axisOffset ;
        this.axisShapes.translate(0, -translateY) ;
      } else if (this.position.equals(RectangleEdge.BOTTOM)) {
        this.axisShapes.setEdge(RectangleEdge.BOTTOM) ;
        double axisHeight = valueAxis.draw(g2, drawArea, drawArea, RectangleEdge.BOTTOM,
                                           this.axisShapes, true) ;

        double yPos = drawAreaHeight - barHeight - axisHeight ;
        for (int i = 0 ; i < this.divisions ; i++) {
          double currX = i * oneWidth ;
          Paint p = this.paintScale.getPaint(base + i * increment) ;
          Rectangle2D rect = new Rectangle2D.Double(currX - 0.5, yPos, oneWidth + 1, barHeight) ;
          g2.setPaint(p) ;
          g2.fill(rect) ;
        }

        if (this.drawStripBorder) {
          g2.setPaint(this.stripBorderPaint) ;
          g2.setStroke(this.stripBorderStroke) ;
          g2.draw(new Rectangle2D.Double(0, yPos, drawAreaWidth, barHeight)) ;
        }
      }
    } else if (RectangleEdge.isLeftOrRight(this.position)) {
      double oneHeight = drawAreaHeight / this.divisions ;
      double barWidth = Math.min(drawAreaWidth, this.stripWidth) ;
      this.valueAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
      if (this.position.equals(RectangleEdge.LEFT)) {
        this.axisShapes.setEdge(RectangleEdge.LEFT) ;
        double axisWidth = valueAxis.draw(g2, drawArea, drawArea, RectangleEdge.LEFT,
                                          this.axisShapes, true) ;

        double xPos = axisWidth ;
        for (int i = 0 ; i < this.divisions ; i++) {
          double currY = i * oneHeight ;
          Paint p = this.paintScale.getPaint(base + i * increment) ;
          Rectangle2D rect = new Rectangle2D.Double(xPos, currY - 0.5, barWidth, oneHeight + 1) ;
          g2.setPaint(p) ;
          g2.fill(rect) ;
        }

        if (this.drawStripBorder) {
          g2.setPaint(this.stripBorderPaint) ;
          g2.setStroke(this.stripBorderStroke) ;
          g2.draw(new Rectangle2D.Double(xPos, 0, barWidth, drawAreaHeight)) ;
        }
      } else if (this.position.equals(RectangleEdge.RIGHT)) {
        this.axisShapes.setEdge(RectangleEdge.RIGHT) ;
        double axisWidth = valueAxis.draw(g2, drawArea, drawArea, RectangleEdge.RIGHT,
                                          this.axisShapes, true) ;

        double xPos = drawAreaWidth - barWidth - axisWidth ;
        for (int i = 0 ; i < this.divisions ; i++) {
          double currY = i * oneHeight ;
          Paint p = this.paintScale.getPaint(base + i * increment) ;
          Rectangle2D rect = new Rectangle2D.Double(xPos, currY - 0.5, barWidth, oneHeight + 1) ;
          g2.setPaint(p) ;
          g2.fill(rect) ;
        }

        if (this.drawStripBorder) {
          g2.setPaint(this.stripBorderPaint) ;
          g2.setStroke(this.stripBorderStroke) ;
          g2.draw(new Rectangle2D.Double(xPos, 0, barWidth, drawAreaHeight)) ;
        }
      }
    }

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;
      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      g2.translate(this.drawArea.getX(), this.drawArea.getY()) ;
      g2.rotate(getRotate()) ;

      g2.setPaint(getSelectAnchorPaint()) ;
      //ê
      Rectangle2D strip = getStripArea() ;
      strip.setRect(0, 0, strip.getWidth(), strip.getHeight()) ;
      AnchorUtil.drawAnchor(g2, strip, rectWidth) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_BLOCK_CHART_LEGEND ;
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    if (RectangleEdge.TOP.equals(this.position) || RectangleEdge.LEFT.equals(this.position)) {
      return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
    } else if (RectangleEdge.BOTTOM.equals(this.position)) {
      double axisHeight = this.valueAxis.estimateAxisWidth(this.position, getMaximumLengthLabel()) ;
      double yPos = Math.max(this.drawArea.getY(),
                             this.drawArea.getMaxY() - this.stripWidth - axisHeight) ;

      return new Point2D.Double(this.drawArea.getX(), yPos) ;
    } else if (RectangleEdge.RIGHT.equals(this.position)) {
      double axisWidth = this.valueAxis.estimateAxisWidth(this.position, getMaximumLengthLabel()) ;
      double xPos = Math.max(this.drawArea.getX(),
                             this.drawArea.getMaxX() - this.stripWidth - axisWidth) ;

      return new Point2D.Double(xPos, this.drawArea.getY()) ;
    }

    return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.drawArea.setRect(drawArea.getX() + tx, drawArea.getY() + ty,
                            drawArea.getWidth(), drawArea.getHeight()) ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      Rectangle2D strip = getStripArea() ;
      AffineTransform transform = new AffineTransform() ;
      transform.translate(strip.getX(), strip.getY()) ;
      transform.rotate(getRotate()) ;

      Rectangle2D shapeArea = null ;
      double axisWidth = this.valueAxis.estimateAxisWidth(this.position, getMaximumLengthLabel()) ;
      if (RectangleEdge.TOP.equals(this.position)) {
        double height = Math.min(this.drawArea.getHeight(), this.stripWidth + axisWidth) ;
        shapeArea = new Rectangle2D.Double(this.drawArea.getX(), this.drawArea.getY(),
                                           this.drawArea.getWidth(), height) ;
      } else if (RectangleEdge.LEFT.equals(this.position)) {
        double width = Math.min(this.drawArea.getWidth(), this.stripWidth + axisWidth) ;
        shapeArea = new Rectangle2D.Double(this.drawArea.getX(), this.drawArea.getY(),
                                           width, this.drawArea.getHeight()) ;
      } else if (RectangleEdge.BOTTOM.equals(this.position)) {
        double height = Math.min(this.drawArea.getHeight(), this.stripWidth + axisWidth) ;
        shapeArea = new Rectangle2D.Double(this.drawArea.getX(), this.drawArea.getMaxY() - height,
                                           this.drawArea.getWidth(), height) ;
      } else if (RectangleEdge.RIGHT.equals(this.position)) {
        double width = Math.min(this.drawArea.getWidth(), this.stripWidth + axisWidth) ;
        shapeArea = new Rectangle2D.Double(this.drawArea.getMaxX() - width, this.drawArea.getY(),
                                           width, this.drawArea.getHeight()) ;
      }

      transformedShape = transform.createTransformedShape(shapeArea) ;
    }

  }

  /**
   * <zh>strip.</zh>
   * <en>Calculate the strip area.</en>
   *
   * @return Rectangle2D
   */
  private Rectangle2D getStripArea() {
    Rectangle2D result = new Rectangle2D.Double() ;

    double x = drawArea.getX() ;
    double y = drawArea.getY() ;
    double w = this.drawArea.getWidth() ;
    double h = this.drawArea.getHeight() ;

    if (RectangleEdge.isTopOrBottom(this.position)) {
      double barHeight = Math.min(h, this.stripWidth) ;
      if (this.position.equals(RectangleEdge.TOP)) {
        result = new Rectangle2D.Double(x, y, w, barHeight) ;
      } else if (this.position.equals(RectangleEdge.BOTTOM)) {
        result = new Rectangle2D.Double(x, y + (h - barHeight), w, barHeight) ;
      }
    } else if (RectangleEdge.isLeftOrRight(this.position)) {
      double barWidth = Math.min(w, this.stripWidth) ;
      if (this.position.equals(RectangleEdge.LEFT)) {
        result = new Rectangle2D.Double(x, y, barWidth, h) ;
      } else if (this.position.equals(RectangleEdge.RIGHT)) {
        result = new Rectangle2D.Double(x + (w - barWidth), y, barWidth, h) ;
      }
    }

    return result ;
  }

  /**
   * <zh>سıǩ.</zh>
   * <en>Returns the label of maximum length.</en>
   * @return String
   */
  protected String getMaximumLengthLabel() {
    if (this.paintScale instanceof LookupPaintScale) {
      LookupPaintScale lookupScale = (LookupPaintScale)this.paintScale ;
      String[] symbols = lookupScale.getLegendSymbols() ;
      if (symbols != null && symbols.length > 0) {
        return getMaxLengthStr(symbols) ;
      }
    }

    NumberRange range = getDatasetRange() ;
    DoubleTickUnit tickUnit = new DoubleTickUnit() ;
    String[] ticks = tickUnit.getTickUnits(range.getLowerBound(), range.getUpperBound(), 5, false) ;

    return getMaxLengthStr(ticks) ;
  }

  /**
   * <zh>ַ.</zh>
   * <en>Calculate the label of maximum length.</en>
   * @param strs String[]
   * @return String
   */
  private static String getMaxLengthStr(String[] strs) {
    if (strs == null || strs.length == 0) {
      return "" ;
    }

    String result = null ;
    int maxLength = -1 ;
    for (int i = 0 ; i < strs.length ; i++) {
      String str = strs[i] ;
      if (str != null && str.length() > maxLength) {
        result = str ;
        maxLength = str.length() ;
      }
    }

    return StringUtil.nullToEmpty(result) ;
  }

  /**
   * <zh>ݼķΧ.</zh>
   * <en>Calculate the range of dataset.</en>
   * @return NumberRange
   */
  public NumberRange getDatasetRange() {
    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    if (this.dataset != null) {
      int rows = this.dataset.getRowCount() ;

      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = this.dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof XYZNumberChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'XYZNumberChartData'.") ;
            } else {
              XYZNumberChartData numberData = (XYZNumberChartData) data ;
              double value = numberData.getZ() ;
              if (!Double.isNaN(value)) {
                if (Double.isNaN(minValue)) {
                  minValue = value ;
                } else {
                  minValue = Math.min(minValue, value) ;
                }

                if (Double.isNaN(maxValue)) {
                  maxValue = value ;
                } else {
                  maxValue = Math.max(maxValue, value) ;
                }
              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof PaintScaleLegendChartShape)) {
      return false ;
    }

    PaintScaleLegendChartShape that = (PaintScaleLegendChartShape) obj ;
    if (!EqualsUtil.equals(this.paintScale, that.paintScale)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.label, that.label)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.position, that.position)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.drawArea, that.drawArea)) {
      return false ;
    }
    if (this.divisions != that.divisions) {
      return false ;
    }
    if (this.stripWidth != that.stripWidth) {
      return false ;
    }
    if (this.drawStripBorder != that.drawStripBorder) {
      return false ;
    }
    if (!EqualsUtil.equals(this.stripBorderPaint, that.stripBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.stripBorderStroke, that.stripBorderStroke)) {
      return false ;
    }
    if (this.axisOffset != that.axisOffset) {
      return false ;
    }
    if (this.valueAxis != that.valueAxis) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.paintScale) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.label) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.position) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.drawArea) ;
    hashCode = 37 * hashCode + this.divisions ;

    hashCode = 37 * hashCode + this.stripWidth ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.drawStripBorder) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.stripBorderPaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.stripBorderStroke) ;
    hashCode = 37 * hashCode + this.axisOffset ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.valueAxis) ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "PaintScaleLegendChartShape" ;
  }

}
