package com.ijchart.xychart.tickunit ;

import java.io.Serializable ;
import java.text.DecimalFormat ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڼ͵ݵ̶ֵ,̶ֵΪ.</zh>
 * <en>Calculate the tick unit for number data and all values is long integer.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LongTickUnit implements INumberTickUnit, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3148867211254682512L ;

  /**
   * <zh>.</zh>
   * <en>The step.</en>
   */
  private long step = Long.MIN_VALUE ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>A default constructor.</en>
   */
  public LongTickUnit() {

  }

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param step long <zh>.</zh>
   * <en>The step.</en>
   */
  public LongTickUnit(long step) {
    this.step = step ;
  }

  /**
   * <zh>СֵֵݵȷϵĿ̶ֵ,̶ֵǳ.</zh>
   * <en>According to the minimum and maximum data and the largest number of coordinates
   * to determine the scale of axis,all values are long.</en>
   *
   * @param minValue double <zh>Сֵ.</zh>
   * <en>The minimum value.</en>
   * @param maxValue double <zh>ֵ.</zh>
   * <en>The max value.</en>
   * @param maxCount int <zh>̶ֵ,0.</zh>
   * <en>The maximum count of tick units (must great than 0).</en>
   * @param includeZero boolean <zh>ֵ,Ƿ0ֵ.</zh>
   * <en>A boolean flag indicates whether or not include zero value.</en>
   *
   * @return String[] <zh>̶ֵǿ,ֵС˳,ÿԪض,(Ϊnull).</zh>
   * <en>An array of tick units,the values are sorted ascending.
   * All elements in array is number (never null).</en>
   */
  public String[] getTickUnits(double minValue, double maxValue, int maxCount, boolean includeZero) {
    if (minValue > maxValue) {
      throw new IllegalArgumentException("Must be 'minValue'<='maxValue'.") ;
    }
    if (maxCount < 1) {
      throw new IllegalArgumentException("Must be 'maxCount'>0.") ;
    }

    if (includeZero) {
      if (maxValue < 0) {
        maxValue = 0 ;
      } else if (minValue > 0) {
        minValue = 0 ;
      }
    }

    DecimalFormat format = new DecimalFormat("#") ;
    if (minValue == maxValue) {
      if (maxCount > 1) { //ǩֵ1
        double floor = Math.floor(minValue) ;
        double ceil = Math.ceil(minValue) ;
        if (ceil != floor) {
          return new String[] {format.format(floor), format.format(ceil)} ;
        }

        return new String[] {format.format(minValue)} ;
      } else {
        return new String[] {format.format(minValue)} ;
      }
    }

    String[] result = null ;
    if (minValue >= 0) { //minValue>=0 maxValue>=0
      if (includeZero) { //0
        double estimateStep = (maxValue - 0) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          long calcStep = this.step ;
          if (calcStep == Long.MIN_VALUE) {
            calcStep = calculateStep(estimateStep) ;
          }

          int unitCount = (int) (maxValue / calcStep) + ((maxValue % calcStep > 0) ? 1 : 0) + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            for (int i = 0 ; i < unitCount ; i++) {
              result[i] = format.format((i * calcStep < 0) ? Long.MAX_VALUE : i * calcStep) ;
            }
            break ;
          }
        }

      } else { //0
        double minStartValue = minValue ;
        double estimateStep = (maxValue - minStartValue) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          long calcStep = this.step ;
          if (calcStep == Long.MIN_VALUE) {
            calcStep = calculateStep(estimateStep) ;
          }

          double firstValue = Math.floor(minValue / calcStep) * calcStep ;

          int unitCount = (int) ((maxValue - firstValue) / calcStep)
                          + (((maxValue - firstValue) % calcStep > 0) ? 1 : 0) + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            result[0] = format.format(firstValue) ;
            for (int i = 1 ; i < unitCount ; i++) {
              double tmpValue = firstValue + i * calcStep ;
              result[i] = format.format((tmpValue < 0) ? Long.MAX_VALUE : tmpValue) ;
            }
            break ;
          }
        }
      }
    } else if (minValue < 0 && maxValue >= 0) {
      double estimateStep = (maxValue - minValue) / maxCount ;

      int count = 0 ;
      while (true && count++ < 5) {
        long calcStep = this.step ;
        if (calcStep == Long.MIN_VALUE) {
          calcStep = calculateStep(estimateStep) ;
        }

        int postiveCount = (int) (maxValue / calcStep) + ((maxValue % calcStep > 0) ? 1 : 0) + 1 ;
        int negativeCount = (int) ( -minValue / calcStep) + (( -minValue % calcStep > 0) ? 1 : 0) ;
        if (negativeCount + postiveCount > maxCount) {
          estimateStep = calcStep ;
          continue ;
        } else {
          result = new String[negativeCount + postiveCount] ;

          long value = 0 ;
          for (int i = 0 ; i < negativeCount ; i++) { //
            value = -(negativeCount - i) * calcStep ;
            if (value >= 0) {
              value = Long.MIN_VALUE ;
            }
            result[i] = format.format(value) ;
          }
          for (int i = 0 ; i < postiveCount ; i++) { //
            value = (i * calcStep < 0) ? Long.MAX_VALUE : i * calcStep ;
            result[negativeCount + i] = format.format(value) ;
          }

          break ;
        }
      }
    } else if (minValue < 0 && maxValue < 0) {
      if (includeZero) { //0
        double estimateStep = ( -minValue - 0) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          long calcStep = this.step ;
          if (calcStep == Long.MIN_VALUE) {
            calcStep = calculateStep(estimateStep) ;
          }

          int unitCount = (int) ( -minValue / calcStep) + (( -minValue % calcStep > 0) ? 1 : 0) + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            long value = 0 ;
            for (int i = 0 ; i < unitCount ; i++) {
              value = -calcStep * (unitCount - i - 1) ;
              if (i != unitCount - 1 && value >= 0) {
                value = Long.MIN_VALUE ;
              }
              result[i] = format.format(value) ;
            }
            break ;
          }
        }

      } else { //0
        double tmpMinValue = -maxValue ;
        double tmpMaxValue = -minValue ;
        double minStartValue = tmpMinValue ;
        double estimateStep = (tmpMaxValue - minStartValue) / maxCount ;

        int count = 0 ;
        while (true && count++ < 5) {
          long calcStep = this.step ;
          if (calcStep == Long.MIN_VALUE) {
            calcStep = calculateStep(estimateStep) ;
          }

          double firstValue = Math.floor(tmpMinValue / calcStep) * calcStep ;

          int unitCount = (int) ((tmpMaxValue - firstValue) / calcStep)
                          + (((tmpMaxValue - firstValue) % calcStep > 0) ? 1 : 0) + 1 ;
          if (unitCount > maxCount) {
            estimateStep = calcStep ;
            continue ;
          } else {
            result = new String[unitCount] ;

            result[unitCount - 1] = format.format((firstValue != 0) ? -firstValue : firstValue) ;
            for (int i = 1 ; i < unitCount ; i++) {
              double tmpValue = firstValue + i * calcStep ;
              result[unitCount - i - 1] = format.format((tmpValue < 0) ? Long.MIN_VALUE :
                                                        (tmpValue != 0) ? tmpValue * -1 : tmpValue) ;
            }
            break ;
          }
        }
      }
    }

    return result ;
  }

  /**
   * <zh>㲽</zh>
   * <en>Calculate step size.</en>
   *
   * @param nStep double
   * @return int
   */
  private static long calculateStep(double nStep) {
    if (nStep < 1) {
      return 1 ;
    } else if (nStep < 2) {
      return 2 ;
    } else if (nStep < 5) {
      return 5 ;
    } else if (nStep < 10) {
      return 10 ;
    }

    String str = Long.toString((long) nStep) ;
    char[] charArr = str.toCharArray() ;

    String first = "" ;
    char c = charArr[0] ;
    if (c >= '1' && c < '2') {
      first = "2" ;
    } else if (c >= '2' && c < '5') {
      first = "5" ;
    } else {
      first = "10" ;
    }

    for (int i = 1 ; i < charArr.length ; i++) {
      first += "0" ;
    }

    return Long.parseLong(first, 10) ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests the axis for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }

    if (!(obj instanceof LongTickUnit)) {
      return false ;
    }

    return true ;
  }

  /**
   * Returns a clone of the object.
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return this.getClass().getName().hashCode() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "LongTickUnit" ;
  }

}
