package com.ijchart.xychart.title ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.util.Calendar ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֵtitle.</zh>
 * <en>A chart title that displays the date.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DateTitle extends TextTitle implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2554457933483445155L ;

  /**
   * <zh>ڶ.</zh>
   * <en>The date object.</en>
   */
  private Calendar calendar ;

  /**
   * <zh>ʽڵ.</zh>
   * <en>Constant for date format.</en>
   */
  private int style ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param cal Calendar <zh>ڶ (Ϊnull).</zh>
   * <en>The date object (null permitted).</en>
   */
  public DateTitle(Calendar cal) {
    this(cal, DateFormat.DEFAULT) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param cal Calendar <zh>ڶ (Ϊnull).</zh>
   * <en>The date object (null permitted).</en>
   * @param style int <zh>ʽڵ,{@link DateFormat}.</zh>
   * <en>Constant for date format,The date style should be one of:SHORT,MEDIUM, LONG or FULL.</en>
   */
  public DateTitle(Calendar cal, int style) {
    super((cal == null) ? null : DateFormat.getDateInstance(style).format(cal.getTime())) ;

    this.calendar = cal ;
    this.style = style ;
  }

  /**
   * <zh>ڶ.</zh>
   * <en>Returns the date object.</en>
   *
   * @return Calendar <zh>ڶ (Ϊnull).</zh>
   * <en>The date object (maybe null).</en>
   */
  public Calendar getCalendar() {
    return this.calendar ;
  }

  /**
   * <zh>ڶ,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the date object and sends a
   * {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param cal Calendar <zh>ڶ.</zh>
   * <en>The date object.</en>
   */
  public void setCalendar(Calendar cal) {
    if (!EqualsUtil.equals(this.calendar, cal)) {
      this.calendar = cal ;
      setText(formatDate()) ;
    }
  }

  /**
   * <zh>ظʽڵ.</zh>
   * <en>Returns the constant for date format.</en>
   *
   * @return int <zh>ʽڵ.</zh>
   * <en>Constant for date format.</en>
   */
  public int getStyle() {
    return this.style ;
  }

  /**
   * <zh>øʽڵ,м
   * {@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the constant for date format and sends a
   * {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param style int <zh>ʽڵ.</zh>
   * <en>Constant for date format.</en>
   */
  public void setStyle(int style) {
    if (this.style != style) {
      this.style = style ;
      setText(formatDate()) ;
    }
  }

  /**
   * <zh>ʽ.</zh>
   * <en>Format a date object.</en>
   *
   * @return String
   */
  private String formatDate() {
    if (this.calendar != null) {
      DateFormat format = DateFormat.getDateInstance(this.style) ;
      return format.format(this.calendar) ;
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this title for equality with another object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof TextTitle)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    DateTitle that = (DateTitle) obj ;
    if (!EqualsUtil.equals(this.calendar, that.calendar)) {
      return false ;
    }
    if (this.style != that.style) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.calendar) ;
    result = 29 * result + this.style ;

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of this object.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException never.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
