package com.ijchart.xychart.title ;

import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.TitleShapeCollection ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֵı,ֿԸݻԶ.</zh>
 * <en>A chart title that displays a text string with automatic wrapping as required.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TextTitle extends AbstractTitle implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8398763896448424325L ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>The default font.</en>
   */
  public static final Font DEFAULT_FONT = new Font("SansSerif", Font.BOLD, 12) ;

  /**
   * <zh>ȱʡɫ.</zh>
   * <en>The default text color.</en>
   */
  public static final Paint DEFAULT_TEXT_PAINT = Color.black ;

  /**
   * <zh>ȱʡǷչռ.</zh>
   * <en>A default flag that controls whether the title expands to fit the available space.</en>
   */
  public static final boolean DEFAULT_EXPAND_TO_FIT_SPACE = false ;

  /**
   * <zh>ַ.</zh>
   * <en>The title text. </en>
   */
  private String text ;

  /**
   * <zh>.</zh>
   * <en>The font used to display the title.</en>
   */
  private Font textFont ;

  /**
   * <zh>ɫ.</zh>
   * <en>The paint used to display the title text.</en>
   */
  private transient Paint textPaint ;

  /**
   * <zh>Ƿչռ.</zh>
   * <en>A flag that controls whether the title expands to fit the available space.</en>
   */
  private boolean expandToFitSpace ;

  /**
   * <zh>.</zh>
   * <en>Creates a new title, using default attributes where necessary.</en>
   *
   * @param text String <zh>ַ (Ϊnull).</zh>
   * <en>The title text (null not permitted).</en>
   */
  public TextTitle(String text) {
    this(text, DEFAULT_FONT, DEFAULT_TEXT_PAINT, AbstractTitle.DEFAULT_TITLE_POSITION,
         AbstractTitle.DEFAULT_HORIZONTAL_ALIGNMENT, AbstractTitle.DEFAULT_VERTICAL_ALIGNMENT,
         AbstractTitle.DEFAULT_PADDING) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new title, using default attributes where necessary.</en>
   *
   * @param text String <zh>ַ (Ϊnull).</zh>
   * <en>The title text (null not permitted).</en>
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The title font (null not permitted).</en>
   */
  public TextTitle(String text, Font font) {
    this(text, font, DEFAULT_TEXT_PAINT, AbstractTitle.DEFAULT_TITLE_POSITION,
         AbstractTitle.DEFAULT_HORIZONTAL_ALIGNMENT,
         AbstractTitle.DEFAULT_VERTICAL_ALIGNMENT, AbstractTitle.DEFAULT_PADDING) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new title, using default attributes where necessary.</en>
   *
   * @param text String <zh>ַ (Ϊnull).</zh>
   * <en>The title text (null not permitted).</en>
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The title font (null not permitted).</en>
   * @param textPaint Paint <zh>ɫ (Ϊnull).</zh>
   * <en>The title paint (null not permitted).</en>
   * @param position RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The title position (null not permitted).</en>
   * @param horizontalAlignment HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null not permitted).</en>
   * @param verticalAlignment VerticalAlignment <zh>Ĵֱ뷽ʽ.</zh>
   * <en>The vertical alignment (null not permitted).</en>
   * @param padding RectangleInsets <zh>Ŀհ (Ϊnull).</zh>
   * <en>The space to leave around the outside of the title (null not permitted).</en>
   */
  public TextTitle(String text, Font font, Paint textPaint, RectangleEdge position,
                   HorizontalAlignment horizontalAlignment, VerticalAlignment verticalAlignment,
                   RectangleInsets padding) {
    super(position, horizontalAlignment, verticalAlignment, padding) ;

    if (text == null) {
      throw new NullPointerException("Null 'text' argument.") ;
    }
    if (font == null) {
      throw new NullPointerException("Null 'font' argument.") ;
    }
    if (textPaint == null) {
      throw new NullPointerException("Null 'textPaint' argument.") ;
    }

    this.text = text ;
    this.textFont = font ;
    this.textPaint = textPaint ;
    this.expandToFitSpace = DEFAULT_EXPAND_TO_FIT_SPACE ;
  }

  /**
   * <zh>رַ.</zh>
   * <en>Returns the title text.</en>
   *
   * @return The text (never null).
   */
  public String getText() {
    return this.text ;
  }

  /**
   * <zh>ñַ.</zh>
   * <en>Sets the title to the specified text and sends a
   * {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param text String <zh>ַ.</zh>
   * <en>The text (null not permitted).</en>
   */
  public void setText(String text) {
    if (text == null) {
      throw new IllegalArgumentException("Null 'text' argument.") ;
    }

    if (!this.text.equals(text)) {
      this.text = text ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ر.</zh>
   * <en>Returns the font used to display the title string.</en>
   *
   * @return Font <zh> (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getTextFont() {
    return this.textFont ;
  }

  /**
   * <zh>ñ,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the font used to display the title string.  Registered listeners
   * are notified that the title has been modified.</en>
   *
   * @param font Font <zh>.</zh>
   * <en>The new font (null not permitted).</en>
   */
  public void setTextFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    if (!this.textFont.equals(font)) {
      this.textFont = font ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رɫ.</zh>
   * <en>Returns the paint used to display the title string.</en>
   *
   * @return Paint <zh>ɫ (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTextPaint() {
    return this.textPaint ;
  }

  /**
   * <zh>ñɫ,м{@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets the paint used to display the title string.Registered listeners
   * are notified that the title has been modified.</en>
   *
   * @param paint Paint <zh>ɫ.</zh>
   * <en>The new paint (null not permitted).</en>
   */
  public void setTextPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.textPaint = paint ;
    fireTitleChanged() ;
  }

  /**
   * <zh>Ƿչռ.</zh>
   * <en>Returns the flag that controls whether the title expands to fit the available space.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean getExpandToFitSpace() {
    return this.expandToFitSpace ;
  }

  /**
   * <zh>Ƿչռ,м
   * {@link com.ijchart.xychart.event.TitleChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether the title expands to fit the available space and sends a
   * {@link com.ijchart.xychart.event.TitleChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setExpandToFitSpace(boolean flag) {
    if (this.expandToFitSpace != flag) {
      this.expandToFitSpace = flag ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ͼchartAreaƱⲿ.ͼеͼδshapeCollection.</zh>
   * <en>Draw the title on chart area(chartArea).All title shape is stored in shapeCollection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param chartArea Rectangle2D <zh>ͼ.</zh>
   * <en>The area within which the chart should be drawn.</en>
   * @param shapeCollection ChartShapeCollection <zh>ͼͼμ.</zh>
   * <en>Collects information about the plot (null permitted).</en>
   *
   * @return Rectangle <zh>.</zh>
   * <en>The area within which the title should be drawn.</en>
   */
  public Rectangle draw(Graphics2D g2, Rectangle2D chartArea, ChartShapeCollection shapeCollection) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (chartArea == null) {
      throw new IllegalArgumentException("Null 'chartArea' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    if (StringUtil.isEmpty(this.text)) {
      return Rectangle.RECTANGLE_ZERO ;
    }

    //title
    RectangleTextChartShape estimateShape = null ;
    RectangleEdge position = getPosition() ;
    if (RectangleEdge.isTopOrBottom(position)) {
      estimateShape = new RectangleTextChartShape(chartArea, getText()) ;
    } else if (RectangleEdge.isLeftOrRight(position)) {
      Rectangle2D area = new Rectangle2D.Double(0, 0, chartArea.getHeight(), chartArea.getWidth()) ;
      estimateShape = new RectangleTextChartShape(area, getText()) ;
    }

    estimateShape.setForceShowAllText(true) ;
    estimateShape.setTextFont(getTextFont()) ;
    estimateShape.setMargin(getPadding()) ;
    estimateShape.setDrawBorder(isBorderVisible()) ;
    estimateShape.setBorderStroke(getBorderStroke()) ;

    Rectangle titleArea = estimateShape.getMinimumBound(g2) ; //Ƴı
    //ܳchartAreaĴС
    if (RectangleEdge.isTopOrBottom(position)) {
      if (titleArea.getHeight() > chartArea.getHeight()) {
        titleArea = new Rectangle(titleArea.getWidth(), chartArea.getHeight()) ;
      }
    } else if (RectangleEdge.isLeftOrRight(position)) {
      if (titleArea.getHeight() > chartArea.getWidth()) {
        titleArea = new Rectangle(titleArea.getWidth(), chartArea.getWidth()) ;
      }
    }

    Rectangle2D titleRect = null ; //ʵλ
    if (position.equals(RectangleEdge.TOP)) {
      if (this.expandToFitSpace) {
        titleRect = new Rectangle2D.Double(chartArea.getX(), chartArea.getY(),
                                           chartArea.getWidth(), titleArea.getHeight()) ;
      } else {
        titleRect = new Rectangle2D.Double(getTitleXPosition(chartArea, titleArea), chartArea.getY(),
                                           titleArea.getWidth(), titleArea.getHeight()) ;
      }
    } else if (position.equals(RectangleEdge.BOTTOM)) {
      if (this.expandToFitSpace) {
        titleRect = new Rectangle2D.Double(chartArea.getX(),
                                           chartArea.getMaxY() - titleArea.getHeight(),
                                           chartArea.getWidth(), titleArea.getHeight()) ;
      } else {
        titleRect = new Rectangle2D.Double(getTitleXPosition(chartArea, titleArea),
                                           chartArea.getMaxY() - titleArea.getHeight(),
                                           titleArea.getWidth(), titleArea.getHeight()) ;
      }
    } else if (position.equals(RectangleEdge.LEFT)) {
      if (this.expandToFitSpace) {
        titleRect = new Rectangle2D.Double(chartArea.getX(), chartArea.getY(),
                                           titleArea.getWidth(), chartArea.getHeight()) ;
      } else {
        titleRect = new Rectangle2D.Double(chartArea.getX(), getTitleYPosition(chartArea, titleArea),
                                           titleArea.getWidth(), titleArea.getHeight()) ;
      }
    } else if (position.equals(RectangleEdge.RIGHT)) {
      if (this.expandToFitSpace) {
        titleRect = new Rectangle2D.Double(chartArea.getMaxX() - titleArea.getHeight(),
                                           chartArea.getY(), titleArea.getWidth(),
                                           chartArea.getHeight()) ;
      } else {
        titleRect = new Rectangle2D.Double(chartArea.getMaxX() - titleArea.getHeight(),
                                           getTitleYPosition(chartArea, titleArea),
                                           titleArea.getWidth(), titleArea.getHeight()) ;
      }
    }

    //ͼζ
    RectangleTextChartShape textShape = new RectangleTextChartShape(titleRect, getText()) ;
    textShape.setTextVerticalAlign(getVerticalAlignment()) ;
    textShape.setTextHorizontalAlign(getHorizontalAlignment()) ;
    textShape.setTextFont(this.textFont) ;
    textShape.setTextPaint(this.textPaint) ;
    textShape.setMargin(getPadding()) ;
    textShape.setTooltipText(getToolTipText()) ;
    textShape.setURL(getURLText()) ;
    textShape.setDrawBorder(isBorderVisible()) ;
    textShape.setBorderStroke(getBorderStroke()) ;
    textShape.setBorderPaint(getBorderPaint()) ;
    textShape.setFillShape(isFillBackground()) ;
    textShape.setFillPaint(getBackgroundPaint()) ;
    textShape.setFillAlpha(getBackgroundAlpha()) ;
    textShape.setForceShowAllText(true) ;

    if (RectangleEdge.isLeftOrRight(position)) {
      textShape.setRotate(IAngle.ANGLE_UP_90) ;
    }

    Rectangle2D clipArea = null ;
    if (RectangleEdge.isLeftOrRight(position)) {
      clipArea = new Rectangle2D.Double(titleRect.getX(), 0, titleRect.getWidth(),
                                        chartArea.getHeight()) ;
    } else if (RectangleEdge.isTopOrBottom(position)) {
      clipArea = new Rectangle2D.Double(0, titleRect.getY(), chartArea.getWidth(),
                                        titleRect.getHeight()) ;
    }

    TitleShapeCollection titleShapes = new TitleShapeCollection() ;
    titleShapes.setDrawArea(clipArea) ;
    titleShapes.addTitleShape(textShape) ;

    shapeCollection.addTitleShapes(titleShapes) ;

    return titleArea ;
  }

  /**
   * <zh>titlexλ.</zh>
   * <en>Calculate the x position of title area.</en>
   * @param chartArea Rectangle2D
   * @param titleArea Rectangle
   * @return double
   */
  private double getTitleXPosition(Rectangle2D chartArea, Rectangle titleArea) {
    double result = chartArea.getX() ;

    HorizontalAlignment hAlign = getHorizontalAlignment() ;
    if (hAlign.equals(HorizontalAlignment.LEFT)) {
      result = chartArea.getX() ;
    } else if (hAlign.equals(HorizontalAlignment.CENTER)) {
      result = chartArea.getX() + (chartArea.getWidth() - titleArea.getWidth()) * 0.5 ;
    } else if (hAlign.equals(HorizontalAlignment.RIGHT)) {
      result = chartArea.getMaxX() - titleArea.getWidth() ;
    }

    return result ;
  }

  /**
   * <zh>titleyλ.</zh>
   * <en>Calculate the y position of title area.</en>
   * @param chartArea Rectangle2D
   * @param titleArea Rectangle
   * @return double
   */
  private double getTitleYPosition(Rectangle2D chartArea, Rectangle titleArea) {
    double result = chartArea.getMaxY() ;

    HorizontalAlignment hAlign = getHorizontalAlignment() ;
    if (hAlign.equals(HorizontalAlignment.LEFT)) {
      result = chartArea.getMaxY() ;
    } else if (hAlign.equals(HorizontalAlignment.CENTER)) {
      result = chartArea.getMaxY() - (chartArea.getHeight() - titleArea.getWidth()) * 0.5 ;
    } else if (hAlign.equals(HorizontalAlignment.RIGHT)) {
      result = chartArea.getY() + titleArea.getWidth() ;
    }

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this title for equality with another object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof TextTitle)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    TextTitle that = (TextTitle) obj ;
    if (!EqualsUtil.equals(this.text, that.text)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textFont, that.textFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.textPaint, that.textPaint)) {
      return false ;
    }
    if (this.expandToFitSpace != that.expandToFitSpace) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.text) ;
    result = 29 * result + HashcodeUtil.hashCode(this.textFont) ;
    result = 29 * result + HashcodeUtil.hashCode(this.textPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.expandToFitSpace) ;

    return result ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of this object.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException never.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.textPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.textPaint = SerialUtil.readPaint(stream) ;
  }

}
