package com.ijchart.xychart.util ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ǶصĹ.</zh>
 * <en>Nice static helpers for working with angle.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AngleUtil {

  /**
   * <zh>Ƕ,޷ֲʾ:</zh>
   * <en>Calculated angle of the quadrant, the quadrant distribution as follows:</en>
   * <br>
   *   <table width="83" border="0" cellpadding="0" cellspacing="0">
   *      <tr>
   *        <td width="37" valign="botom" align="right">2</td>
   *        <td width="9" valign="botom">|</td>
   *        <td width="37" valign="botom"  align="left">1</td>
   *      </tr>
   *      <tr>
   *        <td height="1" colspan="3"><hr width="100%"></td>
   *      </tr>
   *      <tr>
   *        <td valign="top" align="right">3</td>
   *        <td valign="top">|</td>
   *        <td valign="top" align="left">4</td>
   *      </tr>
   *    </table>
   *
   * @param angle double <zh>Ƕ,λΪ.</zh>
   * <en>The angle (in radians).</en>
   *
   * @return int
   */
  public static int getQuadrantByRadianAngle(double angle) {
    final double circle = Math.PI * 2 ;

    double tmpAngle = angle % circle ;
    if (tmpAngle < 0) {
      tmpAngle += circle ;
    }

    if (tmpAngle <= Math.PI * 0.5) {
      return 1 ;
    } else if (tmpAngle <= Math.PI) {
      return 2 ;
    } else if (tmpAngle <= Math.PI * 1.5) {
      return 3 ;
    } else {
      return 4 ;
    }
  }

  /**
   * <zh>Ƕ,޷ֲʾ:</zh>
   * <en>Calculated angle of the quadrant, the quadrant distribution as follows:</en>
   * <br>
   * <table width="83" height="77" border="0" cellpadding="0" cellspacing="0">
   *   <tr>
   *     <td width="37" align="center">2</td>
   *     <td width="9" align="center"><font size="6">|</font></td>
   *     <td width="37" align="center">1</td>
   *   </tr>
   *   <tr>
   *     <td height="5" colspan="3"><hr width="100%"></td>
   *   </tr>
   *   <tr>
   *     <td align="center">3</td>
   *     <td align="center"><font size="6">|</font></td>
   *     <td align="center">4</td>
   *   </tr>
   * </table>
   *
   * @param angle double <zh>Ƕ,λΪǶ.</zh>
   * <en>The angle (in degree).</en>
   *
   * @return int
   */
  public static int getQuadrantByDegreeAngle(double angle) {
    double tmpAngle = angle % 360 ;
    if (tmpAngle < 0) {
      tmpAngle += 360 ;
    }

    if (tmpAngle <= 90) {
      return 1 ;
    } else if (tmpAngle <= 180) {
      return 2 ;
    } else if (tmpAngle <= 270) {
      return 3 ;
    } else {
      return 4 ;
    }
  }

  /**
   * <zh>תǶ,:Ƕڵ2,תǶ4;Ƕڵ3תΪ1.</zh>
   * <en>Conversion angle, the rules are as follows: If the angle of the No. 2 quadrant,
   * the conversion angle to 4 quadrant; If the angle in the third quadrant
   * is converted to 1 quadrant.</en>
   *
   * @param angle double <zh>Ҫ任ĽǶ,λΪ.</zh>
   * <en>The angle (in radians).</en>
   *
   * @return double <zh>תĽǶ.</zh>
   * <en>The conversion angle.</en>
   */
  public static double conversionAngle(double angle) {
    final double circle = Math.PI * 2 ;

    double tmpAngle = angle % circle ;
    if (tmpAngle < 0) {
      tmpAngle += circle ;
    }

    if (tmpAngle <= Math.PI * 0.5) { //4 quadrant
      return tmpAngle ;
    } else if (tmpAngle <= Math.PI) { //3 quadrant
      return tmpAngle - Math.PI ;
    } else if (tmpAngle < Math.PI * 1.5) { //2 quadrant
      return -Math.PI + tmpAngle ;
    } else { //1 quadrant
      return tmpAngle - 2 * Math.PI ;
    }

  }

  /**
   * <zh>жϽǶǷˮƽ,angle0Math.PI .</zh>
   * <en>Test whether or not angle is 0 or Math.PI</en>
   * @param angle double
   * @return boolean
   */
  public static boolean isHorizontal(double angle) {
    final double circle = Math.PI * 2 ;

    double tmpAngle = angle % circle ;
    if (tmpAngle < 0) {
      tmpAngle += circle ;
    }

    return tmpAngle == 0 || tmpAngle == Math.PI ;
  }

  /**
   * <zh>жϽǶǷֱ,angle0.5*Math.PI1.5*Math.PI .</zh>
   * <en>Test whether or not angle is 0.5*Math.PI or 1.5*Math.PI</en>
   * @param angle double
   * @return boolean
   */
  public static boolean isVertical(double angle) {
    final double circle = Math.PI * 2 ;

    double tmpAngle = angle % circle ;
    if (tmpAngle < 0) {
      tmpAngle += circle ;
    }

    return tmpAngle == 0.5 * Math.PI || tmpAngle == 1.5 * Math.PI ;
  }

}
