package com.ijchart.xychart.util ;

import java.util.ArrayList ;
import java.util.Collections ;
import java.util.Comparator ;
import java.util.Date ;
import java.util.List ;

import com.ijchart.xychart.constant.HistogramType ;
import com.ijchart.xychart.data.DateNumberChartData ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.data.OHLCChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SingleRowDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.function.IFunction ;
import com.ijchart.xychart.range.NumberRange ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ݼ.</zh>
 * <en>A util class for dataset.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class DatasetUtil {

  /**
   * <zh>ݼyֵķΧ,ݼ
   * {@link com.ijchart.xychart.render.xy.XYStackedBarRender}.</zh>
   * <en>Calculate the range of y value,this dataset used for
   *  {@link com.ijchart.xychart.render.xy.XYStackedBarRender}.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼеÿ
   * {@link com.ijchart.xychart.data.NumberChartData}.</zh>
   * <en>The dataset,each data is {@link com.ijchart.xychart.data.NumberChartData}
   *  object in dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public static NumberRange getDatasetYRangeForStackBarRender(IMatrixDataset dataset) {
    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    //еݷһarraylist,
    List allDataList = new ArrayList() ;
    int rowCount = dataset.getRowCount() ;
    int cols = dataset.getColumnCount() ;

    for (int i = 0 ; i < rowCount ; i++) {
      if (i == 0) {
        allDataList = new ArrayList(rowCount * cols) ;
      }

      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        Object data = dataset.getData(i, j) ;
        if (data != null) {
          if (!(data instanceof XYNumberChartData)) {
            throw new InvalidChartDataException(
              "Invalid chart data type,musts be 'XYNumberChartData'.") ;
          } else {
            allDataList.add(data) ;
          }
        }
      }

    }

    Collections.sort(allDataList) ;
    double lastX = Double.NaN ;
    maxValue = 0 ; //yֵ֮,Ҫxֵͬ
    minValue = 0 ; //иyֵ֮,Ҫxֵͬ
    double positiveTotalY = 0 ; //һϵеyֵ֮
    double negativeTotalY = 0 ; //һϵеĸyֵ֮
    for (int i = 0 ; i < allDataList.size() ; i++) {
      XYNumberChartData numberData = (XYNumberChartData) allDataList.get(i) ;
      double xValue = numberData.getX() ;
      double yValue = numberData.getY() ;
      if (Double.isNaN(xValue) || Double.isNaN(yValue)) {
        continue ;
      } else {
        if (xValue == lastX) { //ǰֵǰһֵx
          if (yValue >= 0) {
            positiveTotalY += yValue ;
          } else {
            negativeTotalY += yValue ;
          }
        } else { //
          maxValue = Math.max(maxValue, positiveTotalY) ;
          minValue = Math.min(minValue, negativeTotalY) ;

          if (yValue >= 0) {
            positiveTotalY = yValue ;
            negativeTotalY = 0 ;
          } else {
            positiveTotalY = 0 ;
            negativeTotalY = yValue ;
          }

          lastX = xValue ;
        }

      }
    }

    maxValue = Math.max(maxValue, positiveTotalY) ;
    minValue = Math.min(minValue, negativeTotalY) ;

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>ݼСֵֵ.</zh>
   * <en>Calculate the minimum and maximum value of dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ.</zh>
   * <en>The dataset.</en>
   * @param comp Comparator <zh>Ƚ (Ϊnull).</zh>
   * <en>The comparator with which to determine the minimum element.A null value indicates that
   *  the elements' natural ordering should be used (null permitted).</en>
   *
   * @return IChartData[] <zh>(Ϊnull),һֵСֵ,ڶֵֵ.</zh>
   * <en>The data array (maybe null),first is minimum value,second is maximum value.</en>
   */
  public static IChartData[] getMinAndMaxValue(IMatrixDataset dataset, Comparator comp) {
    IChartData minValue = null ;
    IChartData maxValue = null ;

    if (dataset != null) {
      int rowCount = dataset.getRowCount() ;
      for (int i = 0 ; i < rowCount ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null) {
            continue ;
          }

          //Сֵ
          if (minValue == null) {
            minValue = data ;
          } else {
            if (comp != null) {
              if (comp.compare(minValue, data) > 0) {
                minValue = data ;
              }
            } else {
              if (minValue.compareTo(data) > 0) {
                minValue = data ;
              }
            }
          }

          //ֵ
          if (maxValue == null) {
            maxValue = data ;
          } else {
            if (comp != null) {
              if (comp.compare(maxValue, data) < 0) {
                maxValue = data ;
              }
            } else {
              if (maxValue.compareTo(data) < 0) {
                maxValue = data ;
              }
            }
          }

        }
      }

    }

    return new IChartData[] {minValue, maxValue} ;
  }

  /**
   * <zh>ݼƶƽֵݼ,Դݼеݱ{@link OHLCChartData}.</zh>
   * <en>Creates a new {@link IMatrixDataset} containing the moving averages of each series in the
   *  source dataset,and each data is
   * {@link com.ijchart.xychart.data.OHLCChartData} object in source dataset,</en>
   * @param sourceDataset IMatrixDataset <zh>Դݼ (Ϊnull).</zh>
   * <en>The source dataset (null permitted).</en>
   * @param rowKeyPrefix String <zh>ǰ׺䵽ԭrowKeyǰ (Ϊnull).</zh>
   * <en>The string to added in front of thesource rowKey to create new rowKey
   *  names (null permitted).</en>
   * @param rowKeySuffix String <zh>ǰ׺䵽ԭrowKeyĺ (Ϊnull).</zh>
   * <en>The string to append to source rowKey to create new rowKey names (null permitted).</en>
   * @param averageLen int <zh>ƶƽֵõڵݵĸ.</zh>
   * <en>The length of average datas.</en>
   *
   * @return IMatrixDataset <zh>µݼ,datasetΪnull򷵻null,Ϊnull.ݼеÿݶ
   * {@link DateNumberChartData}.</zh>
   * <en>The new dataset,each data is {@link com.ijchart.xychart.data.DateNumberChartData} object
   *  in new dataset.</en>
   */
  public static IMatrixDataset createMovingAverageOHLCDataset(IMatrixDataset sourceDataset,
                                                              String rowKeyPrefix,
                                                              String rowKeySuffix,
                                                              int averageLen) {
    if (sourceDataset == null) {
      return null ;
    }

    if (averageLen <= 0) {
      throw new IllegalArgumentException("Argument 'averageLen' must be >=0.") ;
    }

    DefaultMatrixDataset result = new DefaultMatrixDataset() ;
    //colKey
    int colCount = sourceDataset.getColumnCount() ;
    for (int i = 0 ; i < colCount ; i++) {
      result.addColKey(sourceDataset.getColKey(i)) ;
    }

    //rowKey
    int rowCount = sourceDataset.getRowCount() ;
    if (StringUtil.isEmpty(rowKeyPrefix) && StringUtil.isEmpty(rowKeySuffix)) {
      for (int i = 0 ; i < rowCount ; i++) {
        result.addRowKey(sourceDataset.getRowKey(i)) ;
      }
    } else {
      for (int i = 0 ; i < rowCount ; i++) {
        result.addRowKey(StringUtil.nullToEmpty(rowKeyPrefix) + sourceDataset.getRowKey(i)
                         + StringUtil.nullToEmpty(rowKeySuffix)) ;
      }
    }

    //
    double[] arr = new double[averageLen] ; //ʱ
    for (int i = 0 ; i < rowCount ; i++) {
      for (int j = 0 ; j < averageLen ; j++) {
        arr[j] = 0 ;
      }

      result.addRowData() ;
      double total = 0.0D ;
      int currRowCols = sourceDataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        int currIndex = j % averageLen ; //ǰλ
        Object data = sourceDataset.getData(i, j) ;
        if (data != null && data instanceof OHLCChartData) {
          OHLCChartData ohlc = (OHLCChartData) data ;
          Date date = ohlc.getDate() ;
          double close = ohlc.getClose() ;

          total = total - arr[currIndex] + close ;
          arr[currIndex] = close ;

          double average = total / (((j + 1) < averageLen) ? (j + 1) : averageLen) ;
          result.addData(i, new DateNumberChartData(date, new Double(average))) ;
        } else {
          arr[currIndex] = 0.0D ;
          result.addData(i, null) ;
        }
      }

    }

    return result ;
  }

  /**
   * <zh>ѰݼСֵֵ.</zh>
   * <en>Find the minimum and maximum data of the dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   *
   * @return IChartData[] <zh>һֵСֵ,ڶֵֵ.</zh>
   * <en>The first is the minimum data,the second is the maximum data.</en>
   */
  public static IChartData[] findMinAndMaxChartData(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    int rows = dataset.getRowCount() ;
    List allDatas = new ArrayList() ;

    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        allDatas.add(data) ;
      }
    }

    Collections.sort(allDatas) ;

    IChartData[] result = new IChartData[2] ;
    int size = allDatas.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = allDatas.get(i) ;
      if (obj != null) {
        result[0] = (IChartData) obj ;
        break ;
      }
    }

    for (int i = size - 1 ; i >= 0 ; i--) {
      Object obj = allDatas.get(i) ;
      if (obj != null) {
        result[1] = (IChartData) obj ;
        break ;
      }
    }

    return result ;
  }

  /**
   * <zh>ͼݼݵĺ,Сڵ0ݻ.</zh>
   * <en>Calculates the total of all the values in a {@link SingleRowDataset}.If
   * the dataset contains negative or null values, they are ignored.</en>
   *
   * @param dataset SingleRowDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @return double <zh>ݵĺ.</zh>
   * <en>The total of all the values.</en>
   */
  public static double calculateDatasetTotalForPieChart(SingleRowDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }

    double totalValue = 0 ;
    int rows = dataset.getRowCount() ;
    if (rows > 0) {
      int currRowCols = dataset.getColumnCount(0) ;
      for (int i = 0 ; i < currRowCols ; i++) {
        IChartData chartData = dataset.getData(0, i) ;
        if (chartData != null) {
          if (chartData instanceof NumberChartData) {
            NumberChartData data = (NumberChartData) chartData ;
            double value = data.getValue() ;
            if (!Double.isNaN(value) && value > 0) {
              totalValue += value ;
            }
          } else {
            throw new IllegalArgumentException("Data must be 'NumberChartData'.") ;
          }
        }
      }
    }

    return totalValue ;
  }

  /**
   * <zh>ֵ͵ݼתΪֱͼݼ.</zh>
   * <en>Convert a number dataset to histogram datasete.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼеÿһ{@link NumberChartData}.
   * (Ϊnull).</zh>
   * <en>A dataset,each data is {@link com.ijchart.xychart.data.NumberChartData}
   *  object in dataset. (null not permitted).</en>
   * @param bins int <zh>ֵĸ,ڵ1.</zh>
   * <en>The number of bins,must be greater than 0.</en>
   * @param type HistogramType <zh>ֱͼͼ (Ϊnull).</zh>
   * <en>The histogram type (null not permitted).</en>
   *
   * @return IMatrixDataset <zh>תݼ.</zh>
   * <en>The result dataset.</en>
   */
  public static IMatrixDataset convertToHistogramDataset(IMatrixDataset dataset, int bins,
                                                         HistogramType type) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    int rows = dataset.getRowCount() ;
    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData chartData = dataset.getData(i, j) ;
        if (chartData == null) {
          continue ;
        }

        if (chartData instanceof NumberChartData) {
          double value = ((NumberChartData) chartData).getValue() ;
          if (!Double.isNaN(value)) {
            if (Double.isNaN(minValue)) {
              minValue = value ;
            } else {
              minValue = Math.min(minValue, value) ;
            }

            if (Double.isNaN(maxValue)) {
              maxValue = value ;
            } else {
              maxValue = Math.max(maxValue, value) ;
            }
          }
        } else {
          throw new IllegalArgumentException("Data must be 'NumberChartData'.") ;
        }
      }
    }

    return convertToHistogramDataset(dataset, bins, minValue, maxValue, type) ;
  }

  /**
   * <zh>ֵ͵ݼתΪֱͼݼ.</zh>
   * <en>Convert a number dataset to histogram datasete.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼеÿһ{@link NumberChartData}.
   * (Ϊnull).</zh>
   * <en>A dataset,each data is {@link com.ijchart.xychart.data.NumberChartData}
   *  object in dataset. (null not permitted).</en>
   * @param bins int <zh>ֵĸ,ڵ1.</zh>
   * <en>The number of bins,must be greater than 0.</en>
   * @param minimum double <zh>binΧСֵ.</zh>
   * <en>The lower bound of the bin range.</en>
   * @param maximum double <zh>binΧֵ.</zh>
   * <en>The upper bound of the bin range.</en>
   * @param type HistogramType <zh>ֱͼͼ (Ϊnull).</zh>
   * <en>The histogram type (null not permitted).</en>
   *
   * @return IMatrixDataset <zh>תݼ.</zh>
   * <en>The result dataset.</en>
   */
  public static IMatrixDataset convertToHistogramDataset(IMatrixDataset dataset, int bins,
                                                         double minimum, double maximum,
                                                         HistogramType type) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }
    if (bins < 1) {
      throw new IllegalArgumentException("The 'bins' value must be at least 1.") ;
    }
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument") ;
    }

    int intType = -1 ;
    if (type.equals(HistogramType.FREQUENCY)) {
      intType = 1 ;
    }
    if (type.equals(HistogramType.RELATIVE_FREQUENCY)) {
      intType = 2 ;
    }
    if (type.equals(HistogramType.SCALE_AREA_TO_1)) {
      intType = 3 ;
    }

    DefaultMatrixDataset result = new DefaultMatrixDataset() ;
    int rows = dataset.getRowCount() ;
    int cols = dataset.getColumnCount() ;

    for (int i = 0 ; i < cols ; i++) { //add all column keys
      result.addColKey(dataset.getColKey(i)) ;
    }

    double binLength = maximum - minimum ;
    double binWidth = binLength / bins ;
    if (binLength == 0) {
      binLength = 1 ;
    }
    double lower = minimum ;
    double upper = Double.NaN ;

    ArrayList binList = new ArrayList(bins) ;
    for (int k = 0 ; k < bins ; k++) {
      double[] histogram = null ;
      if (k == bins - 1) {
        histogram = new double[] {lower, maximum, 0} ;
      } else {
        upper = minimum + (k + 1) * binWidth ;
        histogram = new double[] {lower, upper, 0} ;
        lower = upper ;
      }

      binList.add(histogram) ;
    }

    for (int i = 0 ; i < rows ; i++) {
      result.addRowData() ;
      result.addRowKey(dataset.getRowKey(i)) ;

      List newBinList = (ArrayList) binList.clone() ;
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData chartData = dataset.getData(i, j) ;
        if (chartData == null) {
          continue ;
        }

        if (chartData instanceof NumberChartData) {
          NumberChartData data = (NumberChartData) chartData ;
          double value = data.getValue() ;

          if (!Double.isNaN(value)) {
            int binIndex = bins - 1 ;
            if (value < maximum) {
              double fraction = (value - minimum) / binLength ;
              if (fraction < 0.0) {
                fraction = 0.0 ;
              }
              binIndex = (int) (fraction * bins) ;
              if (binIndex >= bins) {
                binIndex = bins - 1 ;
              }
            }

            double[] bin = (double[]) newBinList.get(binIndex) ;
            bin[2]++ ;
          }
        } else {
          throw new IllegalArgumentException("Data must be 'NumberChartData'.") ;
        }
      }

      for (int k = 0 ; k < bins ; k++) {
        double[] bin = (double[]) newBinList.get(k) ;
        double endY = Double.NaN ;

        if (intType == 1) {
          endY = bin[2] ;
        } else if (intType == 2) {
          int colLen = dataset.getColumnCount(i) ;
          if (colLen == 0) {
            endY = 0 ;
          } else {
            endY = bin[2] / colLen ;
          }
        } else if (intType == 3) {
          int colLen = dataset.getColumnCount(i) ;
          if (colLen == 0) {
            endY = 0 ;
          } else {
            endY = bin[2] / (colLen * binWidth) ;
          }
        }

        IntervalXYNumberChartData xydata = new IntervalXYNumberChartData(bin[0], bin[1], 0, endY) ;
        result.addData(i, xydata) ;
      }
    }

    return result ;
  }

  /**
   * <zh>ֱͼ.</zh>
   * <en>Calculate histogram datas.</en>
   *
   * @param values double[] <zh>Ҫתֵ.</zh>
   * <en>The values.</en>
   * @param bins int <zh>ֵĸ,ڵ1.</zh>
   * <en>The number of bins,must be greater than 0.</en>
   * @param type HistogramType <zh>ֱͼͼ (Ϊnull).</zh>
   * <en>The histogram type (null not permitted).</en>
   *
   * @return IMatrixDataset <zh>תݼ.</zh>
   * <en>The result dataset.</en>
   */
  public static IntervalXYNumberChartData[] convertToHistogramData(double[] values, int bins,
                                                                   HistogramType type) {
    if (values == null) {
      throw new IllegalArgumentException("Null 'values' argument") ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    for (int i = 0 ; i < values.length ; i++) {
      double value = values[i] ;
      if (!Double.isNaN(value)) {
        if (Double.isNaN(minValue)) {
          minValue = value ;
        } else {
          minValue = Math.min(minValue, value) ;
        }

        if (Double.isNaN(maxValue)) {
          maxValue = value ;
        } else {
          maxValue = Math.max(maxValue, value) ;
        }
      }
    }

    return convertToHistogramData(values, bins, minValue, maxValue, type) ;
  }

  /**
   * <zh>ֱͼ.</zh>
   * <en>Calculate histogram datas.</en>
   *
   * @param values double[] <zh>Ҫתֵ.</zh>
   * <en>The values.</en>
   * @param bins int <zh>ֵĸ,ڵ1.</zh>
   * <en>The number of bins,must be greater than 0.</en>
   * @param minimum double <zh>binΧСֵ.</zh>
   * <en>The lower bound of the bin range.</en>
   * @param maximum double <zh>binΧֵ.</zh>
   * <en>The upper bound of the bin range.</en>
   * @param type HistogramType <zh>ֱͼͼ (Ϊnull).</zh>
   * <en>The histogram type (null not permitted).</en>
   *
   * @return IMatrixDataset <zh>תݼ.</zh>
   * <en>The result dataset.</en>
   */
  public static IntervalXYNumberChartData[] convertToHistogramData(double[] values, int bins,
                                                                   double minimum, double maximum,
                                                                   HistogramType type) {
    if (values == null) {
      throw new IllegalArgumentException("Null 'values' argument") ;
    }
    if (bins < 1) {
      throw new IllegalArgumentException("The 'bins' value must be at least 1.") ;
    }
    if (type == null) {
      throw new IllegalArgumentException("Null 'type' argument") ;
    }

    if (values.length == 0) {
      return null ;
    }

    int intType = -1 ;
    if (type.equals(HistogramType.FREQUENCY)) {
      intType = 1 ;
    }
    if (type.equals(HistogramType.RELATIVE_FREQUENCY)) {
      intType = 2 ;
    }
    if (type.equals(HistogramType.SCALE_AREA_TO_1)) {
      intType = 3 ;
    }

    double binLength = maximum - minimum ;
    double binWidth = binLength / bins ;
    if (binLength == 0) {
      binLength = 1 ;
    }
    double lower = minimum ;
    double upper = Double.NaN ;

    ArrayList binList = new ArrayList(bins) ;
    for (int k = 0 ; k < bins ; k++) {
      double[] histogram = null ;
      if (k == bins - 1) {
        histogram = new double[] {lower, maximum, 0} ;
      } else {
        upper = minimum + (k + 1) * binWidth ;
        histogram = new double[] {lower, upper, 0} ;
        lower = upper ;
      }

      binList.add(histogram) ;
    }

    for (int i = 0 ; i < values.length ; i++) {
      double value = values[i] ;
      if (!Double.isNaN(value)) {
        int binIndex = bins - 1 ;
        if (value < maximum) {
          double fraction = (value - minimum) / binLength ;
          if (fraction < 0.0) {
            fraction = 0.0 ;
          }
          binIndex = (int) (fraction * bins) ;
          if (binIndex >= bins) {
            binIndex = bins - 1 ;
          }
        }

        double[] bin = (double[]) binList.get(binIndex) ;
        bin[2]++ ;
      }

    }

    IntervalXYNumberChartData[] datas = new IntervalXYNumberChartData[bins] ;
    for (int i = 0 ; i < bins ; i++) {
      double[] bin = (double[]) binList.get(i) ;
      double endY = Double.NaN ;

      if (intType == 1) {
        endY = bin[2] ;
      } else if (intType == 2) {
        if (values.length == 0) {
          endY = 0 ;
        } else {
          endY = bin[2] / values.length ;
        }
      } else if (intType == 3) {
        if (values.length == 0) {
          endY = 0 ;
        } else {
          endY = bin[2] / (values.length * binWidth) ;
        }
      }

      datas[i] = new IntervalXYNumberChartData(bin[0], bin[1], 0, endY) ;
    }

    return datas ;
  }

  /**
   * <zh>ݺͼֵ.</zh>
   * <en>Calculate chart datas by a function.</en>
   *
   * @param function IFunction <zh> (Ϊnull).</zh>
   * <en>The function (null not permitted).</en>
   * @param start double <zh>ʼֵ.</zh>
   * <en>The start value for the range.</en>
   * @param end double <zh>ֵֹ.</zh>
   * <en>The end value for the range.</en>
   * @param samples int <zh>,1.</zh>
   * <en>The number of sample points.</en>
   * @return XYNumberChartData[]
   */
  public static XYNumberChartData[] getDatasByFunction(IFunction function, double start,
                                                       double end, int samples) {
    if (function == null) {
      throw new IllegalArgumentException("Null 'function' argument.") ;
    }
    if (start >= end) {
      throw new IllegalArgumentException("Requires 'start' < 'end'.") ;
    }
    if (samples < 2) {
      throw new IllegalArgumentException("Requires 'samples' > 1") ;
    }

    XYNumberChartData[] datas = new XYNumberChartData[samples + 1] ;
    double step = (end - start) / samples ;
    for (int i = 0 ; i <= samples ; i++) {
      double x = start + (step * i) ;
      datas[i] = new XYNumberChartData(x, function.getY(x)) ;
    }

    return datas ;
  }

  /**
   * <zh>ƶƽֵ.</zh>
   * <en>Calculate moving average datas.</en>
   *
   * @param xyNumberData List <zh>ݼ,ÿһ{@link XYNumberChartData} (Ϊnull).</zh>
   * <en>The list of {@link XYNumberChartData} data (null not permitted).</en>
   * @param skip int <zh>Եݸ.</zh>
   * <en>The number of datas to skip.</en>
   * @param averageLen int <zh>ƶƽֵõڵݵĸ.</zh>
   * <en>The length of average datas.</en>
   *
   * @return XYNumberChartData[] <zh>.</zh>
   * <en>The average datas.</en>
   */
  public static XYNumberChartData[] calculateMovingAverageDatas(List xyNumberData, int skip,
                                                                int averageLen) {
    if (xyNumberData == null) {
      throw new IllegalArgumentException("Null 'xyNumberData' argument") ;
    }
    if (averageLen < 1) {
      throw new IllegalArgumentException("'averageLen' must be greater than or equal to 1.") ;
    }

    int size = xyNumberData.size() ;
    for (int i = 0 ; i < size ; i++) {
      if (!(xyNumberData.get(i) instanceof XYNumberChartData)) {
        throw new IllegalArgumentException("Elements in 'xyNumberData' must be XYNumberChartData.") ;
      }
    }

    XYNumberChartData[] result = new XYNumberChartData[size] ;
    XYNumberChartData[] sourceDatas = (XYNumberChartData[])
                                      xyNumberData.toArray(new XYNumberChartData[0]) ;

    for (int i = size - 1 ; i >= skip ; i--) {
      XYNumberChartData source = sourceDatas[i] ;
      if (source != null && !Double.isNaN(source.getX())) {
        double currY = source.getY() ;

        double total = 0.0D ;
        int count = 0 ;
        if (!Double.isNaN(currY)) {
          total = currY ;
          count++ ;
        }

        for (int j = 1 ; j <= averageLen - 1 ; j++) {
          int idx = i - j ;
          if (idx >= 0) {
            XYNumberChartData nextData = sourceDatas[idx] ;
            if (nextData != null) {
              double nextY = nextData.getY() ;
              if (!Double.isNaN(nextY)) {
                total += nextY ;
                count++ ;
              }
            }
          }
        }

        double average = Double.NaN ;
        if (count != 0) {
          average = total / count ;
        }

        result[i] = new XYNumberChartData(source.getX(), average) ;
      }
    }

    return result ;
  }

  /**
   * <zh>ƶƽֵ.</zh>
   * <en>Calculate the month moving average datas.</en>
   *
   * @param xyNumberData List <zh>ݼ,ÿһ{@link XYNumberChartData} (Ϊnull).</zh>
   * <en>The list of {@link XYNumberChartData} data (null not permitted).</en>
   * @param skipDays int <zh>Ե.</zh>
   * <en>The number of days to skip.</en>
   * @param averageLen int <zh>ƶƽֵõڵݵĸ.</zh>
   * <en>The length of average datas.</en>
   *
   * @return XYNumberChartData[] <zh>.</zh>
   * <en>The average datas.</en>
   */
  public static XYNumberChartData[] calculateDateMovingAverageDatas(List xyNumberData, int skipDays,
                                                                    int averageLen) {
    if (xyNumberData == null) {
      throw new IllegalArgumentException("Null 'xyNumberData' argument") ;
    }
    if (averageLen < 1) {
      throw new IllegalArgumentException("'averageLen' must be greater than or equal to 1.") ;
    }

    int size = xyNumberData.size() ;
    for (int i = 0 ; i < size ; i++) {
      if (!(xyNumberData.get(i) instanceof XYNumberChartData)) {
        throw new IllegalArgumentException("Elements in 'xyNumberData' must be XYNumberChartData.") ;
      }
    }

    XYNumberChartData[] result = new XYNumberChartData[size] ;
    XYNumberChartData[] sourceDatas = (XYNumberChartData[])
                                      xyNumberData.toArray(new XYNumberChartData[0]) ;

    double baseX = 0.0 ;
    for (int i = 0 ; i < size ; i++) {
      XYNumberChartData source = sourceDatas[i] ;
      double x = source.getX() ;
      if (!Double.isNaN(x)) {
        baseX = x ;
        break ;
      }
    }

    final long oneDayInMills = 24 * 60 * 60 * 1000L ;
    final long averageDayInMills = oneDayInMills * averageLen ;
    baseX += oneDayInMills * skipDays ;
    for (int i = size - 1 ; i >= 0 ; i--) {
      XYNumberChartData source = sourceDatas[i] ;
      double currX = source.getX() ;

      if (source != null && !Double.isNaN(currX)) {
        if (currX < baseX) {
          break ;
        }

        double currY = source.getY() ;
        double total = 0.0D ;
        int count = 0 ;
        if (!Double.isNaN(currY)) {
          total = currY ;
          count++ ;
        }

        for (int j = 1 ; j <= averageLen - 1 ; j++) {
          int idx = i - j ;
          if (idx >= 0) {
            XYNumberChartData nextData = sourceDatas[idx] ;
            if (nextData != null) {
              double nextX = nextData.getX() ;
              if (!Double.isNaN(nextX)) {
                if (currX - nextX >= averageDayInMills) {
                  break ;
                }

                double nextY = nextData.getY() ;
                if (!Double.isNaN(nextY)) {
                  total += nextY ;
                  count++ ;
                }
              }
            }
          }
        }

        double average = Double.NaN ;
        if (count != 0) {
          average = total / count ;
        }
        result[i] = new XYNumberChartData(currX, average) ;
      }
    }

    return result ;
  }

  /**
   * <zh>Ѱݼеλ.</zh>
   * <en>Find the position for specified data.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼеÿ
   * {@link com.ijchart.xychart.data.NumberChartData}.</zh>
   * <en>The dataset,each data is {@link com.ijchart.xychart.data.NumberChartData}
   *  object in dataset.</en>
   * @param data IChartData <zh>ָݶ (Ϊnull).</zh>
   * <en>The specified data object (null permitted).</en>
   *
   * @return int[] <zh>ĳΪ2,һֵʾ,ڶֵ.ûҵ򷵻null</zh>
   * <en>The length of the array is 2, the first value indicates the number of rows,
   * the second value is the number of columns,or null if not found.</en>
   */
  public static int[] findDataPosition(IMatrixDataset dataset, IChartData data) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }

    int rowCount = dataset.getRowCount() ;
    int colCount = dataset.getColumnCount() ;
    for (int m = 0 ; m < rowCount ; m++) {
      for (int n = 0 ; n < colCount ; n++) {
        if (data == dataset.getData(m, n)) {
          return new int[] {m, n} ;
        }
      }
    }

    return null ;
  }
}
